@@ -58,12 +58,6 @@
 #include "csync_rename.h"
 #include "c_jhash.h"
 
-
-#ifdef USE_NEON
-// Breaking the abstraction for fun and profit.
-#include "csync_owncloud.h"
-#endif
-
 static int _key_cmp(const void *key, const void *data) {
   uint64_t a;
   csync_file_stat_t *b;
@@ -124,6 +118,8 @@ int csync_create(CSYNC **csync, const char *local, const char *remote) {
 
   ctx->abort = false;
 
+  ctx->ignore_hidden_files = true;
+
   *csync = ctx;
   return 0;
 }
@@ -152,9 +148,6 @@ int csync_init(CSYNC *ctx) {
 
   ctx->local.type = LOCAL_REPLICA;
 
-#ifdef USE_NEON
-  owncloud_init(ctx);
-#endif
   ctx->remote.type = REMOTE_REPLICA;
 
   if (c_rbtree_create(&ctx->local.tree, _key_cmp, _data_cmp) < 0) {
@@ -214,14 +207,6 @@ int csync_update(CSYNC *ctx) {
       CSYNC_LOG(CSYNC_LOG_PRIORITY_DEBUG, "No exclude file loaded or defined!");
   }
 
-#ifdef USE_NEON
-  /* This is not actually connecting, just setting the info for neon. The legacy propagator can use it.. */
-  if (dav_connect( ctx, ctx->remote.uri ) < 0) {
-      ctx->status_code = CSYNC_STATUS_CONNECT_ERROR;
-      return -1;
-  }
-#endif
-
   /* update detection for local replica */
   csync_gettime(&start);
   ctx->current = LOCAL_REPLICA;
@@ -434,7 +419,10 @@ static int _csync_treewalk_visitor(void *obj, void *data) {
       trav.inode        = cur->inode;
 
       trav.error_status = cur->error_status;
-      trav.should_update_etag = cur->should_update_etag;
+      trav.should_update_metadata = cur->should_update_metadata;
+      trav.has_ignored_files = cur->has_ignored_files;
+      trav.checksum = cur->checksum;
+      trav.checksumTypeId = cur->checksumTypeId;
 
       if( other_node ) {
           csync_file_stat_t *other_stat = (csync_file_stat_t*)other_node->data;
@@ -592,6 +580,7 @@ int csync_commit(CSYNC *ctx) {
   ctx->remote.read_from_db = 0;
   ctx->read_remote_from_db = true;
   ctx->db_is_empty = false;
+  ctx->ignore_hidden_files = true; // do NOT sync hidden files by default.
 
 
   /* Create new trees */
@@ -642,10 +631,6 @@ int csync_destroy(CSYNC *ctx) {
   SAFE_FREE(ctx->remote.uri);
   SAFE_FREE(ctx->error_string);
 
-#ifdef USE_NEON
-  owncloud_destroy(ctx);
-#endif
-
 #ifdef WITH_ICONV
   c_close_iconv();
 #endif
@@ -668,21 +653,6 @@ void csync_clear_exclude_list(CSYNC *ctx)
     csync_exclude_clear(ctx);
 }
 
-int csync_set_auth_callback(CSYNC *ctx, csync_auth_callback cb) {
-  if (ctx == NULL || cb == NULL) {
-    return -1;
-  }
-
-  if (ctx->status & CSYNC_STATUS_INIT) {
-    ctx->status_code = CSYNC_STATUS_CSYNC_STATUS_ERROR;
-    fprintf(stderr, "This function must be called before initialization.");
-    return -1;
-  }
-  ctx->callbacks.auth_function = cb;
-
-  return 0;
-}
-
 void *csync_get_userdata(CSYNC *ctx) {
   if (ctx == NULL) {
     return NULL;
@@ -774,17 +744,7 @@ void csync_file_stat_free(csync_file_stat_t *st)
     SAFE_FREE(st->directDownloadCookies);
     SAFE_FREE(st->etag);
     SAFE_FREE(st->destpath);
+    SAFE_FREE(st->checksum);
     SAFE_FREE(st);
   }
 }
-
-int csync_set_module_property(CSYNC* ctx, const char* key, void* value)
-{
-#ifdef USE_NEON
-    return owncloud_set_property(ctx, key, value);
-#else
-    (void)ctx, (void)key, (void)value;
-    return 0;
-#endif
-}
-
