@@ -18,9 +18,17 @@
 class QSslError;
 class QSslCertificate;
 
+namespace OCC {
+
+/**
+ * @brief The SimpleSslErrorHandler class
+ * @ingroup cmd
+ */
 class SimpleSslErrorHandler : public OCC::AbstractSslErrorHandler {
 public:
     bool handleErrors(QList<QSslError> errors, const QSslConfiguration &conf, QList<QSslCertificate> *certs, OCC::AccountPtr) Q_DECL_OVERRIDE;
 };
 
+}
+
 #endif // SIMPLESSLERRORHANDLER_H
