@@ -32,9 +32,11 @@
 #include "utility.h"
 #include "clientproxy.h"
 #include "sharedialog.h"
-
-#include "updater/updater.h"
+#include "accountmanager.h"
 #include "creds/abstractcredentials.h"
+#include "updater/ocupdater.h"
+#include "excludedfiles.h"
+#include "owncloudsetupwizard.h"
 
 #include "config.h"
 
@@ -62,21 +64,27 @@ static const char optionsC[] =
         "  --logwindow          : open a window to show log output.\n"
         "  --logfile <filename> : write log output to file <filename>.\n"
         "  --logdir <name>      : write each sync log output in a new file\n"
-        "                         in directory <name>.\n"
+        "                         in folder <name>.\n"
         "  --logexpire <hours>  : removes logs older than <hours> hours.\n"
         "                         (to be used with --logdir)\n"
         "  --logflush           : flush the log file after every write.\n"
-        "  --confdir <dirname>  : Use the given configuration directory.\n"
+        "  --confdir <dirname>  : Use the given configuration folder.\n"
         ;
 
 QString applicationTrPath()
 {
+    QString devTrPath = qApp->applicationDirPath() + QString::fromLatin1("/../src/gui/");
+    if (QDir(devTrPath).exists()) {
+        // might miss Qt, QtKeyChain, etc.
+        qDebug() << "Running from build location! Translations may be incomplete!";
+        return devTrPath;
+    }
 #if defined(Q_OS_WIN)
    return QApplication::applicationDirPath();
 #elif defined(Q_OS_MAC)
     return QApplication::applicationDirPath()+QLatin1String("/../Resources/Translations"); // path defaults to app dir.
 #elif defined(Q_OS_UNIX)
-    return QString::fromLatin1(DATADIR "/" APPLICATION_EXECUTABLE "/i18n/");
+    return QString::fromLatin1(SHAREDIR "/" APPLICATION_EXECUTABLE "/i18n/");
 #endif
 }
 }
@@ -88,14 +96,16 @@ Application::Application(int &argc, char **argv) :
     _gui(0),
     _theme(Theme::instance()),
     _helpOnly(false),
-    _startupNetworkError(false),
+    _versionOnly(false),
     _showLogWindow(false),
     _logExpire(0),
     _logFlush(false),
     _userTriggeredConnect(false),
     _debugMode(false)
 {
-// TODO: Can't set this without breaking current config pathes
+    _startedAt.start();
+
+// TODO: Can't set this without breaking current config paths
 //    setOrganizationName(QLatin1String(APPLICATION_VENDOR));
     setOrganizationDomain(QLatin1String(APPLICATION_REV_DOMAIN));
     setApplicationName( _theme->appNameGUI() );
@@ -103,9 +113,10 @@ Application::Application(int &argc, char **argv) :
 #if QT_VERSION > QT_VERSION_CHECK(5, 0, 0)
     setAttribute(Qt::AA_UseHighDpiPixmaps, true);
 #endif
+
     parseOptions(arguments());
     //no need to waste time;
-    if ( _helpOnly ) return;
+    if ( _helpOnly || _versionOnly ) return;
 
     if (isRunning())
         return;
@@ -127,27 +138,23 @@ Application::Application(int &argc, char **argv) :
     setupLogging();
     setupTranslations();
 
+    // Setup global excludes
+    ConfigFile cfg;
+    ExcludedFiles& excludes = ExcludedFiles::instance();
+    excludes.addExcludeFilePath( cfg.excludeFile(ConfigFile::SystemScope) );
+    excludes.addExcludeFilePath( cfg.excludeFile(ConfigFile::UserScope) );
+    excludes.reloadExcludes();
+
     _folderManager.reset(new FolderMan);
 
     connect(this, SIGNAL(messageReceived(QString, QObject*)), SLOT(slotParseMessage(QString, QObject*)));
 
-    // Create the account info manager to ensure it's listening to the
-    // account manager.
-    AccountStateManager::instance();
+    AccountManager::instance()->restore();
 
-    AccountPtr account = Account::restore();
-    if (account) {
-        account->setSslErrorHandler(new SslDialogErrorHandler);
-        AccountManager::instance()->setAccount(account);
-    }
-
-    FolderMan::instance()->setSyncEnabled(false);
+    FolderMan::instance()->setSyncEnabled(true);
 
     setQuitOnLastWindowClosed(false);
 
-    qRegisterMetaType<Progress::Info>("Progress::Info");
-
-    ConfigFile cfg;
     _theme->setSystrayUseMonoIcons(cfg.monoIcons());
     connect (_theme, SIGNAL(systrayUseMonoIconsChanged(bool)), SLOT(slotUseMonoIconsChanged(bool)));
 
@@ -159,12 +166,15 @@ Application::Application(int &argc, char **argv) :
         _gui->slotToggleLogBrowser(); // _showLogWindow is set in parseOptions.
     }
 
-    connect(AccountStateManager::instance(), SIGNAL(accountStateAdded(AccountState*)),
+    // Enable word wrapping of QInputDialog (#4197)
+    setStyleSheet("QInputDialog QLabel { qproperty-wordWrap:1; }");
+
+    connect(AccountManager::instance(), SIGNAL(accountAdded(AccountState*)),
             SLOT(slotAccountStateAdded(AccountState*)));
-    connect(AccountStateManager::instance(), SIGNAL(accountStateRemoved(AccountState*)),
+    connect(AccountManager::instance(), SIGNAL(accountRemoved(AccountState*)),
             SLOT(slotAccountStateRemoved(AccountState*)));
-    if (AccountState *ai = AccountStateManager::instance()->accountState()) {
-        slotAccountStateAdded(ai);
+    foreach (auto ai , AccountManager::instance()->accounts()) {
+        slotAccountStateAdded(ai.data());
     }
 
     connect(FolderMan::instance()->socketApi(), SIGNAL(shareCommandReceived(QString, QString, bool)),
@@ -172,173 +182,147 @@ Application::Application(int &argc, char **argv) :
 
     // startup procedure.
     connect(&_checkConnectionTimer, SIGNAL(timeout()), this, SLOT(slotCheckConnection()));
-    _checkConnectionTimer.setInterval(32 * 1000); // check for connection every 32 seconds.
+    _checkConnectionTimer.setInterval(ConnectionValidator::defaultCallingIntervalMsec()); // check for connection every 32 seconds.
     _checkConnectionTimer.start();
-    // Also check immediatly
+    // Also check immediately
     QTimer::singleShot( 0, this, SLOT( slotCheckConnection() ));
 
-    if( cfg.skipUpdateCheck() ) {
-        qDebug() << Q_FUNC_INFO << "Skipping update check";
-    } else {
-        QTimer::singleShot( 3000, this, SLOT( slotStartUpdateDetector() ));
-    }
+    // Can't use onlineStateChanged because it is always true on modern systems because of many interfaces
+    connect(&_networkConfigurationManager, SIGNAL(configurationChanged(QNetworkConfiguration)),
+                this, SLOT(slotSystemOnlineConfigurationChanged(QNetworkConfiguration)));
 
+    // Update checks
+    UpdaterScheduler *updaterScheduler = new UpdaterScheduler(this);
+    connect(updaterScheduler, SIGNAL(updaterAnnouncement(QString, QString)),
+            _gui, SLOT(slotShowTrayMessage(QString, QString)));
+    connect(updaterScheduler, SIGNAL(requestRestart()),
+            _folderManager.data(), SLOT(slotScheduleAppRestart()));
+
+    // Cleanup at Quit.
     connect (this, SIGNAL(aboutToQuit()), SLOT(slotCleanup()));
-
 }
 
 Application::~Application()
 {
+    // Make sure all folders are gone, otherwise removing the
+    // accounts will remove the associated folders from the settings.
+    if (_folderManager) {
+        _folderManager->unloadAndDeleteAllFolders();
+    }
+
     // Remove the account from the account manager so it can be deleted.
-    AccountManager::instance()->setAccount(AccountPtr());
-}
-
-void Application::slotLogin()
-{
-    AccountState *a = AccountStateManager::instance()->accountState();
-    if (a) {
-        FolderMan::instance()->setupFolders();
-        a->setSignedOut(false);
-    }
-}
-
-void Application::slotLogout()
-{
-    AccountState* ai = AccountStateManager::instance()->accountState();
-    if (ai) {
-        AccountPtr a = ai->account();
-        // invalidate & forget token/password
-        a->credentials()->invalidateToken();
-        // terminate all syncs and unload folders
-        FolderMan *folderMan = FolderMan::instance();
-        folderMan->setSyncEnabled(false);
-        folderMan->terminateSyncProcess();
-        ai->setSignedOut(true);
-        // show result
-        _gui->slotComputeOverallSyncStatus();
-    }
+    AccountManager::instance()->shutdown();
 }
 
 void Application::slotAccountStateRemoved(AccountState *accountState)
 {
-    disconnect(accountState, SIGNAL(stateChanged(int)), _gui, SLOT(slotAccountStateChanged()));
-    disconnect(accountState, SIGNAL(stateChanged(int)), this, SLOT(slotAccountStateChanged(int)));
-    connect(accountState->quotaInfo(), SIGNAL(quotaUpdated(qint64,qint64)),
-            _gui, SLOT(slotRefreshQuotaDisplay(qint64,qint64)));
+    if (_gui) {
+        disconnect(accountState, SIGNAL(stateChanged(int)),
+                   _gui, SLOT(slotAccountStateChanged()));
+    }
+    if (_folderManager) {
+        disconnect(accountState, SIGNAL(stateChanged(int)),
+                   _folderManager.data(), SLOT(slotAccountStateChanged()));
+    }
+
+    // if there is no more account, show the wizard.
+    if( AccountManager::instance()->accounts().isEmpty() ) {
+        // allow to add a new account if there is non any more. Always think
+        // about single account theming!
+        OwncloudSetupWizard::runWizard(this, SLOT(slotownCloudWizardDone(int)));
+    }
 }
 
 void Application::slotAccountStateAdded(AccountState *accountState)
 {
-    connect(accountState, SIGNAL(stateChanged(int)), _gui, SLOT(slotAccountStateChanged()));
-    connect(accountState, SIGNAL(stateChanged(int)), this, SLOT(slotAccountStateChanged(int)));
-    connect(accountState->quotaInfo(), SIGNAL(quotaUpdated(qint64,qint64)),
-            _gui, SLOT(slotRefreshQuotaDisplay(qint64,qint64)));
+    connect(accountState, SIGNAL(stateChanged(int)),
+            _gui, SLOT(slotAccountStateChanged()));
+    connect(accountState, SIGNAL(stateChanged(int)),
+            _folderManager.data(), SLOT(slotAccountStateChanged()));
 }
 
 void Application::slotCleanup()
 {
-    // explicitly close windows. This is somewhat of a hack to ensure
-    // that saving the geometries happens ASAP during a OS shutdown
-    AccountPtr account = AccountManager::instance()->account();
-    if (account) {
-        account->save();
-    }
+    AccountManager::instance()->save();
     FolderMan::instance()->unloadAndDeleteAllFolders();
 
     _gui->slotShutdown();
     _gui->deleteLater();
 }
 
-void Application::slotStartUpdateDetector()
+// FIXME: This is not ideal yet since a ConnectionValidator might already be running and is in
+// progress of timing out in some seconds.
+// Maybe we need 2 validators, one triggered by timer, one by network configuration changes?
+void Application::slotSystemOnlineConfigurationChanged(QNetworkConfiguration cnf)
 {
-    Updater *updater = Updater::instance();
-    updater->backgroundCheckForUpdate();
+    if (cnf.state() & QNetworkConfiguration::Active) {
+        //qDebug() << "Trying fast reconnect";
+        QMetaObject::invokeMethod(this, "slotCheckConnection", Qt::QueuedConnection);
+    }
 }
 
 void Application::slotCheckConnection()
 {
-    AccountState *accountState = AccountStateManager::instance()->accountState();
+    auto list = AccountManager::instance()->accounts();
+    foreach (const auto &accountState , list) {
+        AccountState::State state = accountState->state();
 
-    if( accountState ) {
-        accountState->checkConnectivity();
+        // Don't check if we're manually signed out or
+        // when the error is permanent.
+        if (state != AccountState::SignedOut
+                && state != AccountState::ConfigurationError) {
+            accountState->checkConnectivity();
+        }
+    }
 
-    } else {
+    if (list.isEmpty()) {
         // let gui open the setup wizard
-        _gui->slotOpenSettingsDialog( true );
+        _gui->slotOpenSettingsDialog();
 
         _checkConnectionTimer.stop(); // don't popup the wizard on interval;
     }
 }
 
-void Application::slotAccountStateChanged(int state)
-{
-    FolderMan* folderMan = FolderMan::instance();
-    switch (state) {
-    case AccountState::Connected:
-        qDebug() << "Enabling sync scheduler, scheduling all folders";
-        folderMan->setSyncEnabled(true);
-        folderMan->slotScheduleAllFolders();
-        break;
-    case AccountState::ServiceUnavailable:
-    case AccountState::SignedOut:
-    case AccountState::ConfigurationError:
-    case AccountState::NetworkError:
-    case AccountState::Disconnected:
-        qDebug() << "Disabling sync scheduler, terminating sync";
-        folderMan->setSyncEnabled(false);
-        folderMan->terminateSyncProcess();
-        break;
-    }
-
-    // Stop checking the connection if we're manually signed out or
-    // when the error is permanent.
-    if (state == AccountState::SignedOut
-            || state == AccountState::ConfigurationError) {
-        _checkConnectionTimer.stop();
-    } else if (! _checkConnectionTimer.isActive()) {
-        _checkConnectionTimer.start();
-    }
-
-    slotUpdateConnectionErrors(state);
-}
-
 void Application::slotCrash()
 {
     Utility::crash();
 }
 
-void Application::slotUpdateConnectionErrors(int accountState)
-{
-    bool isConnected = accountState == AccountState::Connected;
-    if( !isConnected ) {
-        _startupNetworkError = accountState == AccountState::NetworkError;
-    }
-
-    AccountState *as = AccountStateManager::instance()->accountState();
-    if (as) {
-        _gui->setConnectionErrors( isConnected, as->connectionErrors() );
-    }
-}
-
 void Application::slotownCloudWizardDone( int res )
 {
+    AccountManager *accountMan = AccountManager::instance();
     FolderMan *folderMan = FolderMan::instance();
+
+    // During the wizard, scheduling of new syncs is disabled
+    folderMan->setSyncEnabled(true);
+
     if( res == QDialog::Accepted ) {
-        int cnt = folderMan->setupFolders();
-        qDebug() << "Set up " << cnt << " folders.";
-        // We have some sort of configuration. Enable autostart
-        Utility::setLaunchOnStartup(_theme->appName(), _theme->appNameGUI(), true);
-        if (cnt == 0) {
-            // The folder configuration was skipped
-            _gui->slotShowSettings();
+        // Open the settings page for the new account if no folders
+        // were configured. Using the last account for this check is
+        // not exactly correct, but good enough.
+        if (!accountMan->accounts().isEmpty()) {
+            AccountStatePtr newAccount = accountMan->accounts().last();
+            bool hasFolder = false;
+            foreach (Folder* folder, folderMan->map()) {
+                if (folder->accountState() == newAccount.data()) {
+                    hasFolder = true;
+                    break;
+                }
+            }
+
+            if (!hasFolder) {
+                _gui->slotShowSettings();
+            }
         }
-    }
-    folderMan->setSyncEnabled( true );
-    if( res == QDialog::Accepted ) {
+
+        // Check connectivity of the newly created account
         _checkConnectionTimer.start();
         slotCheckConnection();
-    }
 
+        // The very first time an account is configured: enabled autostart
+        // TODO: Doing this every time the account wizard finishes will annoy users.
+        Utility::setLaunchOnStartup(_theme->appName(), _theme->appNameGUI(), true);
+    }
 }
 
 void Application::setupLogging()
@@ -371,6 +355,12 @@ void Application::slotParseMessage(const QString &msg, QObject*)
         parseOptions(options);
         setupLogging();
     } else if (msg.startsWith(QLatin1String("MSG_SHOWSETTINGS"))) {
+        qDebug() << "Running for" << _startedAt.elapsed()/1000.0 << "sec";
+        if (_startedAt.elapsed() < 10*1000) {
+            // This call is mirrored with the one in int main()
+            qWarning() << "Ignoring MSG_SHOWSETTINGS, possibly double-invocation of client via session restore and auto start";
+            return;
+        }
         showSettingsDialog();
     }
 }
@@ -394,19 +384,19 @@ void Application::parseOptions(const QStringList &options)
             if (it.hasNext() && !it.peekNext().startsWith(QLatin1String("--"))) {
                _logFile = it.next();
             } else {
-                setHelp();
+                showHint("Log file not specified");
             }
         } else if (option == QLatin1String("--logdir")) {
             if (it.hasNext() && !it.peekNext().startsWith(QLatin1String("--"))) {
                _logDir = it.next();
             } else {
-                setHelp();
+                showHint("Log dir not specified");
             }
         } else if (option == QLatin1String("--logexpire")) {
             if (it.hasNext() && !it.peekNext().startsWith(QLatin1String("--"))) {
                 _logExpire = it.next().toInt();
             } else {
-                setHelp();
+                showHint("Log expiration not specified");
             }
         } else if (option == QLatin1String("--logflush")) {
             _logFlush = true;
@@ -414,17 +404,17 @@ void Application::parseOptions(const QStringList &options)
             if (it.hasNext() && !it.peekNext().startsWith(QLatin1String("--"))) {
                 QString confDir = it.next();
                 if (!ConfigFile::setConfDir( confDir )) {
-                    std::cerr << "Invalid path passed to --confdir" << std::endl;
-                    std::exit(1);
+                    showHint("Invalid path passed to --confdir");
                 }
             } else {
-                showHelp();
+                showHint("Path for confdir not specified");
             }
         } else if (option == QLatin1String("--debug")) {
             _debugMode = true;
+        } else if (option == QLatin1String("--version")) {
+            _versionOnly = true;
         } else {
-            setHelp();
-            break;
+            showHint("Unrecognized option '" + option.toStdString() + "'");
         }
     }
 }
@@ -473,6 +463,26 @@ void Application::showHelp()
     displayHelpText(helpText);
 }
 
+void Application::showVersion()
+{
+    QString helpText;
+    QTextStream stream(&helpText);
+    stream << _theme->appName().toLatin1().constData()
+           << QLatin1String(" version ")
+           << _theme->version().toLatin1().constData() << endl;
+    stream << "Using Qt " << qVersion() << endl;
+
+    displayHelpText(helpText);
+}
+
+void Application::showHint(std::string errorHint)
+{
+    static QString binName = QFileInfo(QCoreApplication::applicationFilePath()).fileName();
+    std::cerr << errorHint << std::endl;
+    std::cerr << "Try '" << binName.toStdString() << " --help' for more information" << std::endl;
+    std::exit(1);
+}
+
 bool Application::debugMode()
 {
     return _debugMode;
@@ -483,33 +493,9 @@ void Application::setHelp()
     _helpOnly = true;
 }
 
-#if defined(Q_OS_WIN) && QT_VERSION < QT_VERSION_CHECK(5,0,0)
-bool Application::winEventFilter(MSG *pMsg, long *result)
-{
-    if (pMsg->message == WM_POWERBROADCAST) {
-        switch(pMsg->wParam) {
-        case PBT_APMPOWERSTATUSCHANGE:
-            qDebug() << "WM_POWERBROADCAST: Power state changed";
-            break;
-        case PBT_APMSUSPEND:
-            qDebug() << "WM_POWERBROADCAST: Entering low power state";
-            break;
-        case PBT_APMRESUMEAUTOMATIC:
-            qDebug() << "WM_POWERBROADCAST: Resuming from low power state";
-            break;
-        default:
-            break;
-        }
-        return true;
-    }
-
-    return SharedTools::QtSingleApplication::winEventFilter(pMsg, result);
-}
-#endif
-
 QString substLang(const QString &lang)
 {
-    // Map the more apropriate script codes
+    // Map the more appropriate script codes
     // to country codes as used by Qt and
     // transifex translation conventions.
 
@@ -551,7 +537,7 @@ void Application::setupTranslations()
             // Permissive approach: Qt and keychain translations
             // may be missing, but Qt translations must be there in order
             // for us to accept the language. Otherwise, we try with the next.
-            // "en" is an exeption as it is the default language and may not
+            // "en" is an exception as it is the default language and may not
             // have a translation file provided.
             qDebug() << Q_FUNC_INFO << "Using" << lang << "translation";
             setProperty("ui_lang", lang);
@@ -560,7 +546,9 @@ void Application::setupTranslations()
             const QString qtBaseTrFile = QLatin1String("qtbase_") + lang;
             if (!qtTranslator->load(qtTrFile, qtTrPath)) {
                 if (!qtTranslator->load(qtTrFile, trPath)) {
-                    qtTranslator->load(qtBaseTrFile, trPath);
+                    if (!qtTranslator->load(qtBaseTrFile, qtTrPath)) {
+                        qtTranslator->load(qtBaseTrFile, trPath);
+                    }
                 }
             }
             const QString qtkeychainTrFile = QLatin1String("qtkeychain_") + lang;
@@ -578,6 +566,10 @@ void Application::setupTranslations()
         if (property("ui_lang").isNull())
             setProperty("ui_lang", "C");
     }
+// Work around Qt 5 < 5.5.0 regression, see https://bugreports.qt.io/browse/QTBUG-43447
+#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0) && QT_VERSION < QT_VERSION_CHECK(5, 5, 0)
+    setLayoutDirection(QApplication::tr("QT_LAYOUT_DIRECTION") == QLatin1String("RTL") ? Qt::RightToLeft : Qt::LeftToRight);
+#endif
 }
 
 bool Application::giveHelp()
@@ -585,6 +577,11 @@ bool Application::giveHelp()
     return _helpOnly;
 }
 
+bool Application::versionOnly()
+{
+    return _versionOnly;
+}
+
 void Application::showSettingsDialog()
 {
     _gui->slotShowSettings();
