@@ -19,9 +19,10 @@
 #include "theme.h"
 #include "socketapi.h"
 #include "account.h"
-#include "accountmigrator.h"
 #include "accountstate.h"
+#include "accountmanager.h"
 #include "filesystem.h"
+#include <syncengine.h>
 
 #ifdef Q_OS_MAC
 #include <CoreServices/CoreServices.h>
@@ -40,29 +41,16 @@ namespace OCC {
 
 FolderMan* FolderMan::_instance = 0;
 
-/**
- * The minimum time between a sync being requested and it
- * being executed in milliseconds.
- *
- * This delay must be large enough to ensure fileIsStillChanging()
- * in the upload propagator doesn't decide to skip the file because
- * the modification was too recent.
- */
-static qint64 msBetweenRequestAndSync = 2000;
-
 FolderMan::FolderMan(QObject *parent) :
     QObject(parent),
-    _syncEnabled( true )
+    _currentSyncFolder(0),
+    _syncEnabled( true ),
+    _appRestartRequired(false)
 {
-    _folderChangeSignalMapper = new QSignalMapper(this);
-    connect(_folderChangeSignalMapper, SIGNAL(mapped(const QString &)),
-            this, SIGNAL(folderSyncStateChange(const QString &)));
-
     Q_ASSERT(!_instance);
     _instance = this;
 
     _socketApi = new SocketApi(this);
-    _socketApi->slotReadExcludes();
 
     ConfigFile cfg;
     int polltime = cfg.remotePollInterval();
@@ -75,7 +63,7 @@ FolderMan::FolderMan(QObject *parent) :
     connect(&_startScheduledSyncTimer, SIGNAL(timeout()),
             SLOT(slotStartScheduledFolderSync()));
 
-    connect(AccountStateManager::instance(), SIGNAL(accountStateRemoved(AccountState*)),
+    connect(AccountManager::instance(), SIGNAL(accountRemoved(AccountState*)),
             SLOT(slotRemoveFoldersForAccount(AccountState*)));
 }
 
@@ -95,21 +83,29 @@ OCC::Folder::Map FolderMan::map()
     return _folderMap;
 }
 
-void FolderMan::unloadFolder( const QString& alias )
+void FolderMan::unloadFolder( Folder *f )
 {
-    Folder* f = folder(alias);
     if( !f ) {
         return;
     }
 
     if( _socketApi ) {
-        _socketApi->slotUnregisterPath(alias);
+        _socketApi->slotUnregisterPath(f->alias());
     }
 
-    if( _folderWatchers.contains(alias)) {
-        _folderWatchers.remove(alias);
+    if( _folderWatchers.contains(f->alias())) {
+        _folderWatchers.remove(f->alias());
     }
-    _folderMap.remove( alias );
+    _folderMap.remove( f->alias() );
+
+    disconnect(f, SIGNAL(scheduleToSync(Folder*)),
+               this, SLOT(slotScheduleSync(Folder*)));
+    disconnect(f, SIGNAL(syncStarted()),
+               this, SLOT(slotFolderSyncStarted()));
+    disconnect(f, SIGNAL(syncFinished(SyncResult)),
+               this, SLOT(slotFolderSyncFinished(SyncResult)));
+    disconnect(f, SIGNAL(syncStateChange()),
+               this, SLOT(slotForwardFolderSyncStateChange()));
 }
 
 int FolderMan::unloadAndDeleteAllFolders()
@@ -121,13 +117,14 @@ int FolderMan::unloadAndDeleteAllFolders()
     while (i.hasNext()) {
         i.next();
         Folder* f = i.value();
-        unloadFolder(i.key());
+        unloadFolder(f);
         delete f;
         cnt++;
     }
-    _lastSyncFolder.clear();
-    _currentSyncFolder.clear();
+    _lastSyncFolder = 0;
+    _currentSyncFolder = 0;
     _scheduleQueue.clear();
+    emit scheduleQueueChanged();
 
     Q_ASSERT(_folderMap.count() == 0);
     return cnt;
@@ -142,9 +139,6 @@ void FolderMan::registerFolderMonitor( Folder *folder )
 
     if( !_folderWatchers.contains(folder->alias() ) ) {
         FolderWatcher *fw = new FolderWatcher(folder->path(), folder);
-        ConfigFile cfg;
-        fw->addIgnoreListFile( cfg.excludeFile(ConfigFile::SystemScope) );
-        fw->addIgnoreListFile( cfg.excludeFile(ConfigFile::UserScope) );
 
         // Connect the pathChanged signal, which comes with the changed path,
         // to the signal mapper which maps to the folder alias. The changed path
@@ -186,41 +180,75 @@ void FolderMan::removeMonitorPath( const QString& alias, const QString& path )
 
 int FolderMan::setupFolders()
 {
-  qDebug() << "* Setup folders from " << _folderConfigPath;
+    unloadAndDeleteAllFolders();
 
-  unloadAndDeleteAllFolders();
-
-  ConfigFile cfg;
-  QDir storageDir(cfg.configPath());
-  storageDir.mkpath(QLatin1String("folders"));
-  _folderConfigPath = cfg.configPath() + QLatin1String("folders");
-
-  QDir dir( _folderConfigPath );
-  //We need to include hidden files just in case the alias starts with '.'
-  dir.setFilter(QDir::Files | QDir::Hidden);
-  QStringList list = dir.entryList();
-
-  if( list.count() == 0 ) {
-      // maybe the account was just migrated.
-      AccountPtr acc = AccountManager::instance()->account();
-      if ( acc && acc->wasMigrated() ) {
-          AccountMigrator accMig;
-          list = accMig.migrateFolderDefinitons();
-      }
-  }
-
-  foreach ( const QString& alias, list ) {
-    Folder *f = setupFolderFromConfigFile( alias );
-    if( f ) {
-        slotScheduleSync(alias);
-        emit( folderSyncStateChange( f->alias() ) );
+    auto settings = Account::settingsWithGroup(QLatin1String("Accounts"));
+    const auto accountsWithSettings = settings->childGroups();
+    if (accountsWithSettings.isEmpty()) {
+        int r = setupFoldersMigration();
+        if (r > 0) {
+            AccountManager::instance()->save(false); // don't save credentials, they had not been loaded from keychain
+        }
+        return r;
     }
-  }
 
-  emit folderListLoaded(_folderMap);
+    qDebug() << "* Setup folders from settings file";
 
-  // return the number of valid folders.
-  return _folderMap.size();
+    foreach (const auto& account, AccountManager::instance()->accounts()) {
+        const auto id = account->account()->id();
+        if (!accountsWithSettings.contains(id)) {
+            continue;
+        }
+        settings->beginGroup(id);
+        settings->beginGroup(QLatin1String("Folders"));
+        foreach (const auto& folderAlias, settings->childGroups()) {
+            FolderDefinition folderDefinition;
+            if (FolderDefinition::load(*settings, folderAlias, &folderDefinition)) {
+                Folder* f = addFolderInternal(folderDefinition);
+                if (f) {
+                    f->setAccountState( account.data() );
+                    slotScheduleSync(f);
+                    emit folderSyncStateChange(f);
+                }
+            }
+        }
+        settings->endGroup(); // Folders
+        settings->endGroup(); // <account>
+    }
+
+    emit folderListChanged(_folderMap);
+
+    return _folderMap.size();
+}
+
+int FolderMan::setupFoldersMigration()
+{
+    ConfigFile cfg;
+    QDir storageDir(cfg.configPath());
+    storageDir.mkpath(QLatin1String("folders"));
+    _folderConfigPath = cfg.configPath() + QLatin1String("folders");
+
+    qDebug() << "* Setup folders from " << _folderConfigPath << "(migration)";
+
+    QDir dir( _folderConfigPath );
+    //We need to include hidden files just in case the alias starts with '.'
+    dir.setFilter(QDir::Files | QDir::Hidden);
+    QStringList list = dir.entryList();
+
+    // Normally there should be only one account when migrating.
+    AccountState* accountState = AccountManager::instance()->accounts().value(0).data();
+    foreach ( const QString& alias, list ) {
+        Folder *f = setupFolderFromOldConfigFile( alias, accountState );
+        if( f ) {
+            slotScheduleSync(f);
+            emit folderSyncStateChange(f);
+        }
+    }
+
+    emit folderListChanged(_folderMap);
+
+    // return the number of valid folders.
+    return _folderMap.size();
 }
 
 bool FolderMan::ensureJournalGone(const QString &localPath)
@@ -280,7 +308,7 @@ SocketApi *FolderMan::socketApi()
     return this->_socketApi;
 }
 
-QString FolderMan::unescapeAlias( const QString& alias ) const
+QString FolderMan::unescapeAlias( const QString& alias )
 {
     QString a(alias);
 
@@ -302,13 +330,14 @@ QString FolderMan::unescapeAlias( const QString& alias ) const
 
 // filename is the name of the file only, it does not include
 // the configuration directory path
-Folder* FolderMan::setupFolderFromConfigFile(const QString &file) {
+Folder* FolderMan::setupFolderFromOldConfigFile(const QString &file, AccountState *accountState )
+{
     Folder *folder = 0;
 
     qDebug() << "  ` -> setting up:" << file;
     QString escapedAlias(file);
-    // check the unescaped variant (for the case the filename comes out
-    // of the directory listing. If the file is not existing, escape the
+    // check the unescaped variant (for the case when the filename comes out
+    // of the directory listing). If the file does not exist, escape the
     // file and try again.
     QFileInfo cfgFile( _folderConfigPath, file);
 
@@ -318,7 +347,7 @@ Folder* FolderMan::setupFolderFromConfigFile(const QString &file) {
         cfgFile.setFile( _folderConfigPath, escapedAlias );
     }
     if( !cfgFile.isReadable() ) {
-        qDebug() << "Can not read folder definition for alias " << cfgFile.filePath();
+        qDebug() << "Cannot read folder definition for alias " << cfgFile.filePath();
         return folder;
     }
 
@@ -339,7 +368,6 @@ Folder* FolderMan::setupFolderFromConfigFile(const QString &file) {
     QString backend = settings.value(QLatin1String("backend")).toString();
     QString targetPath = settings.value( QLatin1String("targetPath")).toString();
     bool paused = settings.value( QLatin1String("paused"), false).toBool();
-    QStringList blackList = settings.value( QLatin1String("blackList")).toStringList();
     // QString connection = settings.value( QLatin1String("connection") ).toString();
     QString alias = unescapeAlias( escapedAlias );
 
@@ -353,55 +381,51 @@ Folder* FolderMan::setupFolderFromConfigFile(const QString &file) {
         targetPath.remove(0,1);
     }
 
-    AccountState* accountState = AccountStateManager::instance()->accountState();
     if (!accountState) {
         qWarning() << "can't create folder without an account";
         return 0;
     }
 
-    folder = new Folder( accountState, alias, path, targetPath, this );
-    folder->setConfigFile(cfgFile.absoluteFilePath());
-    folder->setSelectiveSyncBlackList(blackList);
-    qDebug() << "Adding folder to Folder Map " << folder;
-    _folderMap[alias] = folder;
-    if (paused) {
-        folder->setSyncPaused(paused);
-        _disabledFolders.insert(folder);
+    FolderDefinition folderDefinition;
+    folderDefinition.alias = alias;
+    folderDefinition.localPath = path;
+    folderDefinition.targetPath = targetPath;
+    folderDefinition.paused = paused;
+    folderDefinition.ignoreHiddenFiles = ignoreHiddenFiles();
+
+    folder = addFolderInternal(folderDefinition);
+    if (folder) {
+        folder->setAccountState(accountState);
+
+        QStringList blackList = settings.value( QLatin1String("blackList")).toStringList();
+        if (!blackList.empty()) {
+            //migrate settings
+            folder->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList, blackList);
+            settings.remove(QLatin1String("blackList"));
+        }
+
+        folder->saveToSettings();
     }
-
-    /* Use a signal mapper to connect the signals to the alias */
-    connect(folder, SIGNAL(scheduleToSync(const QString&)), SLOT(slotScheduleSync(const QString&)));
-    connect(folder, SIGNAL(syncStateChange()), _folderChangeSignalMapper, SLOT(map()));
-    connect(folder, SIGNAL(syncStarted()), SLOT(slotFolderSyncStarted()));
-    connect(folder, SIGNAL(syncFinished(SyncResult)), SLOT(slotFolderSyncFinished(SyncResult)));
-
-    _folderChangeSignalMapper->setMapping( folder, folder->alias() );
-
-    registerFolderMonitor(folder);
+    qDebug() << "Migrated!" << folder;
+    settings.sync();
     return folder;
 }
 
-void FolderMan::slotSetFolderPaused( const QString& alias, bool paused )
+void FolderMan::slotSetFolderPaused( Folder *f, bool paused )
 {
-    Folder *f = folder(alias);
     if( !f ) {
-        qDebug() << "!! Can not enable alias " << alias << ", can not be found in folderMap.";
+        qWarning() << "!! slotSetFolderPaused called with empty folder";
         return;
     }
 
-    slotScheduleSync(alias);
-
-    // FIXME: Use ConfigFile
-    QSettings settings(f->configFile(), QSettings::IniFormat);
-    settings.beginGroup(escapeAlias(f->alias()));
+    f->setSyncPaused(paused);
     if (!paused) {
-        settings.remove("paused");
         _disabledFolders.remove(f);
+        slotScheduleSync(f);
     } else {
-        settings.setValue("paused", true);
         _disabledFolders.insert(f);
     }
-    emit folderSyncStateChange(alias);
+    emit folderSyncStateChange(f);
 }
 
 // this really terminates the current sync process
@@ -409,7 +433,7 @@ void FolderMan::slotSetFolderPaused( const QString& alias, bool paused )
 // csync still remains in a stable state, regardless of that.
 void FolderMan::terminateSyncProcess()
 {
-    Folder *f = folder(_currentSyncFolder);
+    Folder *f = _currentSyncFolder;
     if( f ) {
         // This will, indirectly and eventually, call slotFolderSyncFinished
         // and thereby clear _currentSyncFolder.
@@ -427,53 +451,54 @@ Folder *FolderMan::folder( const QString& alias )
     return 0;
 }
 
-SyncResult FolderMan::syncResult( const QString& alias )
-{
-    Folder *f = folder( alias );
-    return f ? f->syncResult() : SyncResult();
-}
-
 void FolderMan::slotScheduleAllFolders()
 {
     foreach( Folder *f, _folderMap.values() ) {
-        if (f && ! f->syncPaused()) {
-            slotScheduleSync( f->alias() );
+        if (f && f->canSync()) {
+            slotScheduleSync( f );
         }
     }
 }
 
+void FolderMan::slotScheduleAppRestart()
+{
+    _appRestartRequired = true;
+    qDebug() << "## Application restart requested!";
+}
+
 /*
   * if a folder wants to be synced, it calls this slot and is added
   * to the queue. The slot to actually start a sync is called afterwards.
   */
-void FolderMan::slotScheduleSync( const QString& alias )
+void FolderMan::slotScheduleSync( Folder *f )
 {
-    Folder* f = folder(alias);
     if( !f ) {
-        qDebug() << "Not scheduling sync for empty or unknown folder" << alias;
+        qWarning() << "slotScheduleSync called with null folder";
         return;
     }
+    auto alias = f->alias();
 
     if( _socketApi ) {
         // We want the SocketAPI to already now update so that it can show the EVAL icon
         // for files/folders. Only do this when not syncing, else we might get a lot
         // of those notifications.
-        _socketApi->slotUpdateFolderView(alias);
+        _socketApi->slotUpdateFolderView(f);
     }
 
     qDebug() << "Schedule folder " << alias << " to sync!";
 
-    if( ! _scheduleQueue.contains(alias) ) {
-        if( !f->syncPaused() ) {
-            f->prepareToSync();
-        } else {
-            qDebug() << "Folder is not enabled, not scheduled!";
+    if( ! _scheduleQueue.contains(f) ) {
+        if( !f->canSync() ) {
+            qDebug() << "Folder is not ready to sync, not scheduled!";
             if( _socketApi ) {
-                _socketApi->slotUpdateFolderView(f->alias());
+                _socketApi->slotUpdateFolderView(f);
             }
             return;
         }
-        _scheduleQueue.enqueue(alias);
+        f->prepareToSync();
+        emit folderSyncStateChange(f);
+        _scheduleQueue.enqueue(f);
+        emit scheduleQueueChanged();
     } else {
         qDebug() << " II> Sync for folder " << alias << " already scheduled, do not enqueue!";
     }
@@ -498,35 +523,78 @@ void FolderMan::slotEtagJobDestroyed(QObject* /*o*/)
 void FolderMan::slotRunOneEtagJob()
 {
     if (_currentEtagJob.isNull()) {
-        QString alias;
+        Folder *folder;
         foreach(Folder *f, _folderMap) {
             if (f->etagJob()) {
                 // Caveat: always grabs the first folder with a job, but we think this is Ok for now and avoids us having a seperate queue.
                 _currentEtagJob = f->etagJob();
-                alias = f->alias();
+                folder = f;
                 break;
             }
         }
         if (_currentEtagJob.isNull()) {
-            qDebug() << "No more remote ETag check jobs to schedule.";
+            //qDebug() << "No more remote ETag check jobs to schedule.";
+
+            /* now it might be a good time to check for restarting... */
+            if( _currentSyncFolder == NULL && _appRestartRequired ) {
+                restartApplication();
+            }
         } else {
-            qDebug() << "Scheduling" << alias << "to check remote ETag";
+            qDebug() << "Scheduling" << folder->remoteUrl().toString() << "to check remote ETag";
             _currentEtagJob->start(); // on destroy/end it will continue the queue via slotEtagJobDestroyed
         }
     }
 }
 
-// only enable or disable foldermans will to schedule and do syncs.
+void FolderMan::slotAccountStateChanged()
+{
+    AccountState * accountState = qobject_cast<AccountState*>(sender());
+    if (! accountState) {
+        return;
+    }
+    QString accountName = accountState->account()->displayName();
+
+    if (accountState->isConnected()) {
+        qDebug() << "Account" << accountName << "connected, scheduling its folders";
+
+        foreach (Folder *f, _folderMap.values()) {
+            if (f
+                    && f->canSync()
+                    && f->accountState() == accountState) {
+                slotScheduleSync(f);
+            }
+        }
+    } else {
+        qDebug() << "Account" << accountName << "disconnected, "
+                    "terminating or descheduling sync folders";
+
+        if (_currentSyncFolder
+                && _currentSyncFolder->accountState() == accountState) {
+            _currentSyncFolder->slotTerminateSync();
+        }
+
+        QMutableListIterator<Folder*> it(_scheduleQueue);
+        while (it.hasNext()) {
+            Folder* f = it.next();
+            if (f->accountState() == accountState) {
+                it.remove();
+            }
+        }
+        emit scheduleQueueChanged();
+    }
+}
+
+// only enable or disable foldermans will schedule and do syncs.
 // this is not the same as Pause and Resume of folders.
 void FolderMan::setSyncEnabled( bool enabled )
 {
     if (!_syncEnabled && enabled && !_scheduleQueue.isEmpty()) {
-        // We have things in our queue that were waiting the the connection to go back on.
+        // We have things in our queue that were waiting for the connection to come back on.
         startScheduledSyncSoon();
     }
     _syncEnabled = enabled;
     // force a redraw in case the network connect status changed
-    emit( folderSyncStateChange(QString::null) );
+    emit( folderSyncStateChange(0) );
 }
 
 void FolderMan::startScheduledSyncSoon(qint64 msMinimumDelay)
@@ -537,7 +605,7 @@ void FolderMan::startScheduledSyncSoon(qint64 msMinimumDelay)
     if (_scheduleQueue.empty()) {
         return;
     }
-    if (! _currentSyncFolder.isEmpty()) {
+    if (_currentSyncFolder) {
         return;
     }
 
@@ -545,7 +613,7 @@ void FolderMan::startScheduledSyncSoon(qint64 msMinimumDelay)
     qint64 msSinceLastSync = 0;
 
     // Require a pause based on the duration of the last sync run.
-    if (Folder* lastFolder = folder(_lastSyncFolder)) {
+    if (Folder* lastFolder = _lastSyncFolder) {
         msSinceLastSync = lastFolder->msecSinceLastSync();
 
         //  1s   -> 1.5s pause
@@ -557,7 +625,7 @@ void FolderMan::startScheduledSyncSoon(qint64 msMinimumDelay)
     }
 
     // Punish consecutive follow-up syncs with longer delays.
-    if (Folder* nextFolder = folder(_scheduleQueue.head())) {
+    if (Folder* nextFolder = _scheduleQueue.head()) {
         int followUps = nextFolder->consecutiveFollowUpSyncs();
         if (followUps >= 2) {
             // This is okay due to the 1min maximum delay limit below.
@@ -575,7 +643,7 @@ void FolderMan::startScheduledSyncSoon(qint64 msMinimumDelay)
 
     // A minimum of delay here is essential as the sync will not upload
     // files that were changed too recently.
-    msDelay = qMax(msBetweenRequestAndSync, msDelay);
+    msDelay = qMax(SyncEngine::minimumFileAgeForUpload, msDelay);
 
     qDebug() << "Scheduling a sync in" << (msDelay/1000) << "seconds";
     _startScheduledSyncTimer.start(msDelay);
@@ -588,8 +656,8 @@ void FolderMan::startScheduledSyncSoon(qint64 msMinimumDelay)
   */
 void FolderMan::slotStartScheduledFolderSync()
 {
-    if( !_currentSyncFolder.isEmpty() ) {
-        qDebug() << "Currently folder " << _currentSyncFolder << " is running, wait for finish!";
+    if( _currentSyncFolder ) {
+        qDebug() << "Currently folder " << _currentSyncFolder->remoteUrl().toString() << " is running, wait for finish!";
         return;
     }
 
@@ -603,26 +671,23 @@ void FolderMan::slotStartScheduledFolderSync()
         return;
     }
 
-    // Try to start the top scheduled sync.
-    const QString alias = _scheduleQueue.dequeue();
-    Folder *f = folder(alias);
-    if( !f ) {
-        qDebug() << "FolderMan: Not syncing queued folder" << alias << ": not in folder map anymore";
-        return;
+    // Find the first folder in the queue that can be synced.
+    Folder* f = 0;
+    while( !_scheduleQueue.isEmpty() ) {
+        f = _scheduleQueue.dequeue();
+        Q_ASSERT(f);
+
+        if( f->canSync() ) {
+            break;
+        }
     }
 
+    emit scheduleQueueChanged();
+
     // Start syncing this folder!
-    if( !f->syncPaused() ) {
-        _currentSyncFolder = alias;
-
+    if( f ) {
+        _currentSyncFolder = f;
         f->startSync( QStringList() );
-
-        // reread the excludes of the socket api
-        // FIXME: the excludes need rework.
-        if( _socketApi ) {
-            _socketApi->slotClearExcludesList();
-            _socketApi->slotReadExcludes();
-        }
     }
 }
 
@@ -632,66 +697,56 @@ void FolderMan::slotEtagPollTimerTimeout()
     ConfigFile cfg;
     int polltime = cfg.remotePollInterval();
 
-    QSet<QString> folderAliases = _folderMap.keys().toSet();
-    QMutableSetIterator<QString> i(folderAliases);
-    while (i.hasNext()) {
-        QString alias = i.next();
-        if (_currentSyncFolder == alias) {
-            i.remove();
+    foreach (Folder *f, _folderMap) {
+        if (!f) {
             continue;
         }
-        if (_scheduleQueue.contains(alias)) {
-            i.remove();
+        if (_currentSyncFolder == f) {
             continue;
         }
-        Folder *f = _folderMap.value(alias);
-        if (f && _disabledFolders.contains(f)) {
-            i.remove();
+        if (_scheduleQueue.contains(f)) {
             continue;
         }
-        if (f && (f->etagJob() || f->isBusy() || f->syncPaused())) {
-            i.remove();
+        if (_disabledFolders.contains(f)) {
             continue;
         }
-        if (f && f->msecSinceLastSync() < polltime) {
-            i.remove();
+        if (f->etagJob() || f->isBusy() || !f->canSync()) {
             continue;
         }
-    }
-
-    if (folderAliases.isEmpty()) {
-        qDebug() << Q_FUNC_INFO << "No folders need to check for the remote ETag";
-    } else {
-        qDebug() << Q_FUNC_INFO << "The following folders need to check for the remote ETag:" << folderAliases;
-        i = folderAliases; // reset
-         while (i.hasNext()) {
-             QString alias = i.next();
-             QMetaObject::invokeMethod(_folderMap.value(alias), "slotRunEtagJob", Qt::QueuedConnection);
-         }
+        if (f->msecSinceLastSync() < polltime) {
+            continue;
+        }
+        QMetaObject::invokeMethod(f, "slotRunEtagJob", Qt::QueuedConnection);
     }
 }
 
 void FolderMan::slotRemoveFoldersForAccount(AccountState* accountState)
 {
-    QStringList foldersToRemove;
+    QVarLengthArray<Folder *, 16> foldersToRemove;
     Folder::MapIterator i(_folderMap);
     while (i.hasNext()) {
         i.next();
         Folder* folder = i.value();
         if (folder->accountState() == accountState) {
-            foldersToRemove.append(folder->alias());
+            foldersToRemove.append(folder);
         }
     }
 
-    qDebug() << "Account was removed, removing associated folders:" << foldersToRemove;
-    foreach (const QString& alias, foldersToRemove) {
-        slotRemoveFolder(alias);
+    foreach (const auto &f, foldersToRemove) {
+        slotRemoveFolder(f);
+    }
+}
+
+void FolderMan::slotForwardFolderSyncStateChange()
+{
+    if (Folder* f = qobject_cast<Folder*>(sender())) {
+        emit folderSyncStateChange(f);
     }
 }
 
 void FolderMan::slotFolderSyncStarted( )
 {
-    qDebug() << ">===================================== sync started for " << _currentSyncFolder;
+    qDebug() << ">===================================== sync started for " << _currentSyncFolder->remoteUrl().toString();
 }
 
 /*
@@ -702,33 +757,48 @@ void FolderMan::slotFolderSyncStarted( )
   */
 void FolderMan::slotFolderSyncFinished( const SyncResult& )
 {
-    qDebug() << "<===================================== sync finished for " << _currentSyncFolder;
+    qDebug() << "<===================================== sync finished for " << _currentSyncFolder->remoteUrl().toString();
 
     _lastSyncFolder = _currentSyncFolder;
-    _currentSyncFolder.clear();
+    _currentSyncFolder = 0;
 
     startScheduledSyncSoon();
 }
 
-bool FolderMan::addFolderDefinition(const QString& alias, const QString& sourceFolder,
-                                    const QString& targetPath, const QStringList& selectiveSyncBlackList)
+Folder* FolderMan::addFolder(AccountState* accountState, const FolderDefinition& folderDefinition)
 {
-    if (! ensureJournalGone(sourceFolder))
-        return false;
+    if (!ensureJournalGone(folderDefinition.localPath)) {
+        return 0;
+    }
 
-    QString escapedAlias = escapeAlias(alias);
-    // Create a settings file named after the alias
-    QSettings settings( _folderConfigPath + QLatin1Char('/') + escapedAlias, QSettings::IniFormat);
-    settings.beginGroup(escapedAlias);
-    settings.setValue(QLatin1String("localPath"),   sourceFolder );
-    settings.setValue(QLatin1String("targetPath"),  targetPath );
-    // for compat reasons
-    settings.setValue(QLatin1String("backend"),     "owncloud" );
-    settings.setValue(QLatin1String("connection"),  Theme::instance()->appName());
-    settings.setValue(QLatin1String("blackList"), selectiveSyncBlackList);
-    settings.sync();
+    auto folder = addFolderInternal(folderDefinition);
+    if(folder && accountState) {
+        folder->setAccountState(accountState);
+        folder->saveToSettings();
+        emit folderSyncStateChange(folder);
+        emit folderListChanged(_folderMap);
+    }
+    return folder;
+}
 
-    return true;
+Folder* FolderMan::addFolderInternal(const FolderDefinition& folderDefinition)
+{
+    auto folder = new Folder(folderDefinition, this );
+
+    qDebug() << "Adding folder to Folder Map " << folder;
+    _folderMap[folder->alias()] = folder;
+    if (folder->syncPaused()) {
+        _disabledFolders.insert(folder);
+    }
+
+    // See matching disconnects in unloadFolder().
+    connect(folder, SIGNAL(scheduleToSync(Folder*)), SLOT(slotScheduleSync(Folder*)));
+    connect(folder, SIGNAL(syncStarted()), SLOT(slotFolderSyncStarted()));
+    connect(folder, SIGNAL(syncFinished(SyncResult)), SLOT(slotFolderSyncFinished(SyncResult)));
+    connect(folder, SIGNAL(syncStateChange()), SLOT(slotForwardFolderSyncStateChange()));
+
+    registerFolderMonitor(folder);
+    return folder;
 }
 
 Folder *FolderMan::folderForPath(const QString &path)
@@ -736,7 +806,7 @@ Folder *FolderMan::folderForPath(const QString &path)
     QString absolutePath = QDir::cleanPath(path)+QLatin1Char('/');
 
     foreach(Folder* folder, this->map().values()) {
-        const QString folderPath = QDir::cleanPath(folder->path())+QLatin1Char('/');
+        const QString folderPath = folder->cleanPath()+QLatin1Char('/');
 
         if(absolutePath.startsWith(folderPath)) {
             //qDebug() << "found folder: " << folder->path() << " for " << absolutePath;
@@ -747,50 +817,60 @@ Folder *FolderMan::folderForPath(const QString &path)
     return 0;
 }
 
-void FolderMan::removeAllFolderDefinitions()
+QStringList FolderMan::findFileInLocalFolders( const QString& relPath, const AccountPtr acc )
 {
-    foreach( Folder *f, _folderMap.values() ) {
-        if(f) {
-            slotRemoveFolder( f->alias() );
+    QStringList re;
+
+    foreach(Folder* folder, this->map().values()) {
+        if (acc != 0 && folder->accountState()->account() != acc) {
+            continue;
+        }
+        QString path = folder->cleanPath();
+        QString remRelPath;
+        // cut off the remote path from the server path.
+        remRelPath = relPath.mid(folder->remotePath().length());
+        path += "/";
+        path += remRelPath;
+        if( QFile::exists(path) ) {
+            re.append( path );
         }
     }
-    // clear the queue.
-    _scheduleQueue.clear();
-
+    return re;
 }
 
-void FolderMan::slotRemoveFolder( const QString& alias )
+void FolderMan::slotRemoveFolder( Folder *f )
 {
-    Folder *f = folder(alias);
     if( !f ) {
-        qDebug() << "!! Can not remove " << alias << ", not in folderMap.";
+        qWarning() << "!! Can not remove null folder";
         return;
     }
 
-    qDebug() << "Removing " << alias;
+    qDebug() << "Removing " << f->alias();
 
-    const bool currentlyRunning = (_currentSyncFolder == alias);
+    const bool currentlyRunning = (_currentSyncFolder == f);
     if( currentlyRunning ) {
-        // let the folder delete itself when done and
         // abort the sync now
-        connect(f, SIGNAL(syncFinished(SyncResult)), f, SLOT(deleteLater()));
         terminateSyncProcess();
     }
 
-    _scheduleQueue.removeAll(alias);
+    if (_scheduleQueue.removeAll(f) > 0) {
+        emit scheduleQueueChanged();
+    }
 
     f->wipe();
     f->setSyncPaused(true);
 
     // remove the folder configuration
-    QFile file(f->configFile() );
-    if( file.exists() ) {
-        qDebug() << "Remove folder config file " << file.fileName();
-        file.remove();
-    }
+    f->removeFromSettings();
 
-    unloadFolder( alias );
-    if( !currentlyRunning ) {
+    unloadFolder( f);
+    if( currentlyRunning ) {
+        // We want to schedule the next folder once this is done
+        connect(f, SIGNAL(syncFinished(SyncResult)),
+                SLOT(slotFolderSyncFinished(SyncResult)));
+        // Let the folder delete itself when done.
+        connect(f, SIGNAL(syncFinished(SyncResult)), f, SLOT(deleteLater()));
+    } else {
         delete f;
     }
 }
@@ -802,12 +882,12 @@ QString FolderMan::getBackupName( QString fullPathName ) const
 
     if( fullPathName.isEmpty() ) return QString::null;
 
-     QString newName = fullPathName + QLatin1String(".oC_bak");
+     QString newName = fullPathName + tr(" (backup)");
      QFileInfo fi( newName );
-     int cnt = 1;
+     int cnt = 2;
      do {
          if( fi.exists() ) {
-             newName = fullPathName + QString( ".oC_bak_%1").arg(cnt++);
+             newName = fullPathName + tr(" (backup %1)").arg(cnt++);
              fi.setFile(newName);
          }
      } while( fi.exists() );
@@ -846,7 +926,7 @@ bool FolderMan::startFromScratch( const QString& localFolder )
                     _socketApi->slotUnregisterPath(f->alias());
                 }
                 f->journalDb()->close();
-                f->slotTerminateSync(); // Normaly it should not be running, but viel hilft viel
+                f->slotTerminateSync(); // Normally it should not be running, but viel hilft viel
             }
         }
 
@@ -876,7 +956,7 @@ void FolderMan::setDirtyProxy(bool value)
 
             if (f->accountState() && f->accountState()->account()
                     && f->accountState()->account()->networkAccessManager()) {
-                // Need to do this have us not use the old determined system proxy
+                // Need to do this so we do not use the old determined system proxy
                 f->accountState()->account()->networkAccessManager()->setProxy(
                             QNetworkProxy(QNetworkProxy::DefaultProxy));
             }
@@ -902,7 +982,7 @@ SyncResult FolderMan::accountStatus(const QList<Folder*> &folders)
     int cnt = folders.count();
 
     // if one folder: show the state of the one folder.
-    // if more folder:
+    // if more folders:
     // if one of them has an error -> show error
     // if one is paused, but others ok, show ok
     // do not show "problem" in the tray
@@ -953,13 +1033,13 @@ SyncResult FolderMan::accountStatus(const QList<Folder*> &folders)
     } else {
         int errorsSeen = 0;
         int goodSeen = 0;
-        int abortSeen = 0;
+        int abortOrPausedSeen = 0;
         int runSeen = 0;
         int various = 0;
 
         foreach ( Folder *folder, folders ) {
             if( folder->syncPaused() ) {
-                abortSeen++;
+                abortOrPausedSeen++;
             } else {
                 SyncResult folderResult = folder->syncResult();
                 SyncResult::Status syncStatus = folderResult.status();
@@ -967,9 +1047,9 @@ SyncResult FolderMan::accountStatus(const QList<Folder*> &folders)
                 switch( syncStatus ) {
                 case SyncResult::Undefined:
                 case SyncResult::NotYetStarted:
-                case SyncResult::SyncPrepare:
                     various++;
                     break;
+                case SyncResult::SyncPrepare:
                 case SyncResult::SyncRunning:
                     runSeen++;
                     break;
@@ -983,7 +1063,7 @@ SyncResult FolderMan::accountStatus(const QList<Folder*> &folders)
                     break;
                 case SyncResult::SyncAbortRequested:
                 case SyncResult::Paused:
-                    abortSeen++;
+                    abortOrPausedSeen++;
                     // no default case on purpose, check compiler warnings
                 }
             }
@@ -993,7 +1073,7 @@ SyncResult FolderMan::accountStatus(const QList<Folder*> &folders)
             overallResult.setStatus(SyncResult::Error);
             set = true;
         }
-        if( !set && abortSeen > 0 && abortSeen == cnt ) {
+        if( !set && abortOrPausedSeen > 0 && abortOrPausedSeen == cnt ) {
             // only if all folders are paused
             overallResult.setStatus(SyncResult::Paused);
             set = true;
@@ -1019,7 +1099,7 @@ QString FolderMan::statusToString( SyncResult syncStatus, bool paused ) const
         folderMessage = tr( "Undefined State." );
         break;
     case SyncResult::NotYetStarted:
-        folderMessage = tr( "Waits to start syncing." );
+        folderMessage = tr( "Waiting to start syncing." );
         break;
     case SyncResult::SyncPrepare:
         folderMessage = tr( "Preparing for sync." );
@@ -1053,4 +1133,116 @@ QString FolderMan::statusToString( SyncResult syncStatus, bool paused ) const
     return folderMessage;
 }
 
+QString FolderMan::checkPathValidityForNewFolder(const QString& path, bool forNewDirectory)
+{
+    if (path.isEmpty()) {
+        return tr("No valid folder selected!");
+    }
+
+    QFileInfo selFile( path );
+    QString userInput = selFile.canonicalFilePath();
+
+    if (!selFile.exists()) {
+        return checkPathValidityForNewFolder(selFile.dir().path(), true);
+    }
+
+    if( !selFile.isDir() ) {
+        return tr("The selected path is not a folder!");
+    }
+
+    if ( !selFile.isWritable() ) {
+        return tr("You have no permission to write to the selected folder!");
+    }
+
+    // check if the local directory isn't used yet in another ownCloud sync
+
+    for (auto i = _folderMap.constBegin(); i != _folderMap.constEnd(); ++i ) {
+        Folder *f = static_cast<Folder*>(i.value());
+        QString folderDir = QDir( f->path() ).canonicalPath();
+        if( folderDir.isEmpty() ) {
+            continue;
+        }
+        if( ! folderDir.endsWith(QLatin1Char('/')) ) folderDir.append(QLatin1Char('/'));
+
+        if (QDir::cleanPath(f->path()) == QDir::cleanPath(userInput)
+                && QDir::cleanPath(QDir(f->path()).canonicalPath()) == QDir(userInput).canonicalPath()) {
+            return tr("The local folder %1 is already used in a folder sync connection. "
+                      "Please pick another one!")
+                .arg(QDir::toNativeSeparators(userInput));
+        }
+        if (!forNewDirectory && QDir::cleanPath(folderDir).startsWith(QDir::cleanPath(userInput)+'/')) {
+            return tr("The local folder %1 already contains a folder used in a folder sync connection. "
+                      "Please pick another one!")
+                .arg(QDir::toNativeSeparators(userInput));
+        }
+
+        QString absCleanUserFolder = QDir::cleanPath(QDir(userInput).canonicalPath())+'/';
+        if (!forNewDirectory && QDir::cleanPath(folderDir).startsWith(absCleanUserFolder) ) {
+            return tr("The local folder %1 is a symbolic link. "
+                      "The link target already contains a folder used in a folder sync connection. "
+                      "Please pick another one!")
+                .arg(QDir::toNativeSeparators(userInput));
+        }
+
+        if (QDir::cleanPath(QString(userInput)).startsWith( QDir::cleanPath(folderDir)+'/')) {
+            return tr("The local folder %1 is already contained in a folder used in a folder sync connection. "
+                      "Please pick another one!")
+                .arg(QDir::toNativeSeparators(userInput));
+        }
+
+        if (absCleanUserFolder.startsWith( QDir::cleanPath(folderDir)+'/')) {
+            return tr("The local folder %1 is a symbolic link. "
+                      "The link target is already contained in a folder used in a folder sync connection. "
+                      "Please pick another one!")
+                .arg(QDir::toNativeSeparators(userInput));
+        }
+    }
+
+    return QString();
+
+}
+
+bool FolderMan::ignoreHiddenFiles() const
+{
+    if (_folderMap.empty()) {
+        return true;
+    }
+    return _folderMap.begin().value()->ignoreHiddenFiles();
+}
+
+void FolderMan::setIgnoreHiddenFiles(bool ignore)
+{
+    // Note that the setting will revert to 'true' if all folders
+    // are deleted...
+    foreach (Folder* folder, _folderMap) {
+        folder->setIgnoreHiddenFiles(ignore);
+        folder->saveToSettings();
+    }
+}
+
+QQueue<Folder*> FolderMan::scheduleQueue() const
+{
+    return _scheduleQueue;
+}
+
+Folder *FolderMan::currentSyncFolder() const
+{
+    return _currentSyncFolder;
+}
+
+void FolderMan::restartApplication()
+{
+    if( Utility::isLinux() ) {
+        // restart:
+        qDebug() << "### Restarting application NOW, PID" << qApp->applicationPid() << "is ending.";
+        qApp->quit();
+        QStringList args = qApp->arguments();
+        QString prg = args.takeFirst();
+
+        QProcess::startDetached(prg, args);
+    } else {
+        qDebug() << "On this platform we do not restart.";
+    }
+}
+
 } // namespace OCC
