@@ -19,6 +19,7 @@
 #include "networkjobs.h"
 #include "account.h"
 #include "selectivesyncdialog.h"
+#include "accountstate.h"
 #include "creds/abstractcredentials.h"
 
 #include <QDebug>
@@ -68,8 +69,16 @@ FolderWizardLocalPath::FolderWizardLocalPath()
     _ui.localFolderLineEdit->setToolTip(tr("Enter the path to the local folder."));
 
     registerField(QLatin1String("alias*"), _ui.aliasLineEdit);
-    _ui.aliasLineEdit->setText( Theme::instance()->appNameGUI() );
-    _ui.aliasLineEdit->setToolTip(tr("The directory alias is a descriptive name for this sync connection."));
+
+    QString newAlias = Theme::instance()->appName();
+    int count = 0;
+    while (FolderMan::instance()->folder(newAlias)) {
+        // There is already a folder configured with this name and folder names need to be unique
+        newAlias = Theme::instance()->appName() + QString::number(++count);
+    }
+    _ui.aliasLineEdit->setText( newAlias );
+
+    _ui.aliasLineEdit->setToolTip(tr("The folder alias is a descriptive name for this sync connection."));
     _ui.warnLabel->setTextFormat(Qt::RichText);
     _ui.warnLabel->hide();
 }
@@ -91,76 +100,14 @@ void FolderWizardLocalPath::cleanupPage()
 
 bool FolderWizardLocalPath::isComplete() const
 {
-  QFileInfo selFile( QDir::fromNativeSeparators(_ui.localFolderLineEdit->text()) );
-  QString   userInput = selFile.canonicalFilePath();
+    QString errorStr = FolderMan::instance()->checkPathValidityForNewFolder(
+        QDir::fromNativeSeparators(_ui.localFolderLineEdit->text()));
 
-  QStringList warnStrings;
-
-  bool isOk = selFile.isDir();
-  if( !isOk ) {
-    warnStrings.append(tr("No valid local folder selected!"));
-  }
-
-  if (isOk && !selFile.isWritable()) {
-      isOk = false;
-      warnStrings.append(tr("You have no permission to write to the selected folder!"));
-  }
-
-  // check if the local directory isn't used yet in another ownCloud sync
-  Folder::Map map = FolderMan::instance()->map();
-
-  if( isOk ) {
-    Folder::Map::const_iterator i = map.constBegin();
-    while( isOk && i != map.constEnd() ) {
-      Folder *f = static_cast<Folder*>(i.value());
-      QString folderDir = QDir( f->path() ).canonicalPath();
-      if( folderDir.isEmpty() )
-      {
-        isOk = true;
-        qDebug() << "Absolute path for folder: " << f->path() << " doesn't exist. Skipping.";
-        i++;
-        continue;
-      }
-      if( ! folderDir.endsWith(QLatin1Char('/')) ) folderDir.append(QLatin1Char('/'));
-
-      qDebug() << "Checking local path: " << folderDir << " <-> " << userInput;
-      if( QDir::cleanPath(f->path())  == QDir::cleanPath(userInput)  &&
-              QDir::cleanPath(QDir(f->path()).canonicalPath()) == QDir(userInput).canonicalPath() ) {
-        isOk = false;
-        warnStrings.append( tr("The local path %1 is already an upload folder. Please pick another one!")
-                           .arg(QDir::toNativeSeparators(userInput)) );
-      }
-      if( isOk && QDir::cleanPath(folderDir).startsWith(QDir::cleanPath(userInput)+'/') ) {
-        qDebug() << "A already configured folder is child of the current selected";
-        warnStrings.append( tr("An already configured folder is contained in the current entry."));
-        isOk = false;
-      }
-
-      QString absCleanUserFolder = QDir::cleanPath(QDir(userInput).canonicalPath())+'/';
-      if( isOk && QDir::cleanPath(folderDir).startsWith(absCleanUserFolder) ) {
-          qDebug() << "A already configured folder is child of the current selected";
-          warnStrings.append( tr("The selected folder is a symbolic link. An already configured "
-                                "folder is contained in the folder this link is pointing to."));
-          isOk = false;
-      }
-
-      if( isOk && QDir::cleanPath(QString(userInput)).startsWith( QDir::cleanPath(folderDir)+'/') ) {
-        qDebug() << "An already configured folder is parent of the current selected";
-        warnStrings.append( tr("An already configured folder contains the currently entered folder."));
-        isOk = false;
-      }
-      if( isOk && absCleanUserFolder.startsWith( QDir::cleanPath(folderDir)+'/') ) {
-          qDebug() << "The selected folder is a symbolic link. An already configured folder is\n"
-                      "the parent of the current selected contains the folder this link is pointing to.";
-          warnStrings.append( tr("The selected folder is a symbolic link. An already configured folder "
-                                "is the parent of the current selected contains the folder this link is "
-                                "pointing to."));
-          isOk = false;
-      }
-
-      i++;
+    bool isOk = errorStr.isEmpty();
+    QStringList warnStrings;
+    if (!isOk) {
+        warnStrings << errorStr;
     }
-  }
 
   // check if the alias is unique.
   QString alias = _ui.aliasLineEdit->text();
@@ -169,6 +116,7 @@ bool FolderWizardLocalPath::isComplete() const
     isOk = false;
   }
 
+  auto map = FolderMan::instance()->map();
   Folder::Map::const_iterator i = map.constBegin();
   bool goon = true;
   while( goon && i != map.constEnd() ) {
@@ -217,6 +165,11 @@ void FolderWizardLocalPath::slotChooseLocalFolder()
 
         QDir pickedDir(dir);
         QString newAlias = pickedDir.dirName();
+        int count = 0;
+        while (FolderMan::instance()->folder(newAlias)) {
+            // There is already a folder configured with this name and folder names need to be unique
+            newAlias = pickedDir.dirName() + QString::number(++count);
+        }
         if( !newAlias.isEmpty() ) {
             _ui.aliasLineEdit->setText(newAlias);
         }
@@ -287,7 +240,7 @@ void FolderWizardRemotePath::slotCreateRemoteFolder(const QString &folder)
     /* check the owncloud configuration file and query the ownCloud */
     connect(job, SIGNAL(finished(QNetworkReply::NetworkError)),
                  SLOT(slotCreateRemoteFolderFinished(QNetworkReply::NetworkError)));
-    connect(job, SIGNAL(networkError(QNetworkReply*)), SLOT(slotHandleNetworkError(QNetworkReply*)));
+    connect(job, SIGNAL(networkError(QNetworkReply*)), SLOT(slotHandleMkdirNetworkError(QNetworkReply*)));
     job->start();
 }
 
@@ -302,7 +255,7 @@ void FolderWizardRemotePath::slotCreateRemoteFolderFinished(QNetworkReply::Netwo
     }
 }
 
-void FolderWizardRemotePath::slotHandleNetworkError(QNetworkReply *reply)
+void FolderWizardRemotePath::slotHandleMkdirNetworkError(QNetworkReply *reply)
 {
     qDebug() << "** webdav mkdir request failed:" << reply->error();
     if( reply && !_account->credentials()->stillValid(reply) ) {
@@ -313,6 +266,12 @@ void FolderWizardRemotePath::slotHandleNetworkError(QNetworkReply *reply)
     }
 }
 
+void FolderWizardRemotePath::slotHandleLsColNetworkError(QNetworkReply *reply)
+{
+    showWarn(tr("Failed to list a folder. Error: %1")
+             .arg(errorMessage(reply->errorString(), reply->readAll())));
+}
+
 static QTreeWidgetItem* findFirstChild(QTreeWidgetItem *parent, const QString& text)
 {
     for (int i = 0; i < parent->childCount(); ++i) {
@@ -403,11 +362,7 @@ void FolderWizardRemotePath::slotUpdateDirectories(const QStringList &list)
 
 void FolderWizardRemotePath::slotRefreshFolders()
 {
-    LsColJob *job = new LsColJob(_account, "/", this);
-    job->setProperties(QList<QByteArray>() << "resourcetype");
-    connect(job, SIGNAL(directoryListingSubfolders(QStringList)),
-            SLOT(slotUpdateDirectories(QStringList)));
-    job->start();
+    runLsColJob("/");
     _ui.folderTreeWidget->clear();
     _ui.folderEntry->clear();
 }
@@ -415,11 +370,7 @@ void FolderWizardRemotePath::slotRefreshFolders()
 void FolderWizardRemotePath::slotItemExpanded(QTreeWidgetItem *item)
 {
     QString dir = item->data(0, Qt::UserRole).toString();
-    LsColJob *job = new LsColJob(_account, dir, this);
-    job->setProperties(QList<QByteArray>() << "resourcetype");
-    connect(job, SIGNAL(directoryListingSubfolders(QStringList)),
-            SLOT(slotUpdateDirectories(QStringList)));
-    job->start();
+    runLsColJob(dir);
 }
 
 void FolderWizardRemotePath::slotCurrentItemChanged(QTreeWidgetItem *item)
@@ -452,11 +403,11 @@ void FolderWizardRemotePath::slotLsColFolderEntry()
     if (path.startsWith(QLatin1Char('/')))
         path = path.mid(1);
 
-    LsColJob *job = new LsColJob(_account, path, this);
-    job->setProperties(QList<QByteArray>() << "resourcetype");
+    LsColJob *job = runLsColJob(path);
+    // no error handling, no updating, we do this manually
+    disconnect(job, 0, this, 0);
     connect(job, SIGNAL(directoryListingSubfolders(QStringList)),
             SLOT(slotTypedPathFound(QStringList)));
-    job->start();
 }
 
 void FolderWizardRemotePath::slotTypedPathFound(const QStringList& subpaths)
@@ -465,6 +416,19 @@ void FolderWizardRemotePath::slotTypedPathFound(const QStringList& subpaths)
     selectByPath(_ui.folderEntry->text());
 }
 
+LsColJob* FolderWizardRemotePath::runLsColJob(const QString& path)
+{
+    LsColJob *job = new LsColJob(_account, path, this);
+    job->setProperties(QList<QByteArray>() << "resourcetype");
+    connect(job, SIGNAL(directoryListingSubfolders(QStringList)),
+            SLOT(slotUpdateDirectories(QStringList)));
+    connect(job, SIGNAL(finishedWithError(QNetworkReply*)),
+            SLOT(slotHandleLsColNetworkError(QNetworkReply*)));
+    job->start();
+
+    return job;
+}
+
 FolderWizardRemotePath::~FolderWizardRemotePath()
 {
 }
@@ -485,6 +449,9 @@ bool FolderWizardRemotePath::isComplete() const
     Folder::Map::const_iterator i = map.constBegin();
     for(i = map.constBegin();i != map.constEnd(); i++ ) {
         Folder *f = static_cast<Folder*>(i.value());
+        if (f->accountState()->account() != _account) {
+            continue;
+        }
         QString curDir = f->remotePath();
         if (!curDir.startsWith(QLatin1Char('/'))) {
             curDir.prepend(QLatin1Char('/'));
@@ -592,9 +559,9 @@ FolderWizard::FolderWizard(AccountPtr account, QWidget *parent)
     }
     setPage(Page_SelectiveSync, _folderWizardSelectiveSyncPage);
 
-    setWindowTitle( tr("Add Folder") );
+    setWindowTitle( tr("Add Folder Sync Connection") );
     setOptions(QWizard::CancelButtonOnLeft);
-    setButtonText(QWizard::FinishButton, tr("Add Folder"));
+    setButtonText(QWizard::FinishButton, tr("Add Sync Connection"));
 }
 
 FolderWizard::~FolderWizard()
