@@ -15,14 +15,20 @@
 #define MIRALL_GENERALSETTINGS_H
 
 #include <QWidget>
-
+#include <QPointer>
 
 namespace OCC {
+class IgnoreListEditor;
+class SyncLogDialog;
 
 namespace Ui {
 class GeneralSettings;
 }
 
+/**
+ * @brief The GeneralSettings class
+ * @ingroup gui
+ */
 class GeneralSettings : public QWidget
 {
     Q_OBJECT
@@ -30,17 +36,22 @@ class GeneralSettings : public QWidget
 public:
     explicit GeneralSettings(QWidget *parent = 0);
     ~GeneralSettings();
+    QSize sizeHint() const;
 
 private slots:
     void saveMiscSettings();
     void slotToggleLaunchOnStartup(bool);
     void slotToggleOptionalDesktopNotifications(bool);
     void slotUpdateInfo();
+    void slotIgnoreFilesEditor();
+    void slotOpenAccountWizard();
 
 private:
     void loadMiscSettings();
 
     Ui::GeneralSettings *_ui;
+    QPointer<IgnoreListEditor> _ignoreEditor;
+    QPointer<SyncLogDialog> _syncLogDialog;
 };
 
 
