@@ -23,6 +23,10 @@ namespace Ui {
 class NetworkSettings;
 }
 
+/**
+ * @brief The NetworkSettings class
+ * @ingroup gui
+ */
 class NetworkSettings : public QWidget
 {
     Q_OBJECT
@@ -30,6 +34,7 @@ class NetworkSettings : public QWidget
 public:
     explicit NetworkSettings(QWidget *parent = 0);
     ~NetworkSettings();
+    QSize sizeHint() const;
 
 private slots:
     void saveProxySettings();
