@@ -29,6 +29,7 @@
 #include "account.h"
 #include "networkjobs.h"
 #include "sslerrordialog.h"
+#include "accountmanager.h"
 
 #include "creds/credentialsfactory.h"
 #include "creds/abstractcredentials.h"
@@ -77,18 +78,9 @@ void OwncloudSetupWizard::runWizard(QObject* obj, const char* amember, QWidget *
 
 void OwncloudSetupWizard::startWizard()
 {
-    FolderMan *folderMan = FolderMan::instance();
-    bool multiFolderSetup = folderMan->map().count() > 1;
-    // ###
-    AccountPtr account = Account::restore();
-    if (!account) {
-        _ocWizard->setConfigExists(false);
-        account = Account::create();
-        account->setCredentials(CredentialsFactory::create("dummy"));
-    } else {
-        _ocWizard->setConfigExists(true);
-    }
-    account->setSslErrorHandler(new SslDialogErrorHandler);
+    AccountPtr account = AccountManager::createAccount();
+    account->setCredentials(CredentialsFactory::create("dummy"));
+    account->setUrl(Theme::instance()->overrideServerUrl());
     _ocWizard->setAccount(account);
     _ocWizard->setOCUrl(account->url().toString());
 
@@ -102,14 +94,6 @@ void OwncloudSetupWizard::startWizard()
         localFolder = QDir::homePath() + QDir::separator() + localFolder;
     }
 
-    if (!multiFolderSetup) {
-        QList<Folder*> folders = folderMan->map().values();
-        if (!folders.isEmpty()) {
-            Folder* folder = folders.first();
-            localFolder = QDir(folder->path()).absolutePath();
-        }
-    }
-
     _ocWizard->setProperty("oldLocalFolder", localFolder);
     _ocWizard->setProperty("localFolder", localFolder);
 
@@ -127,9 +111,6 @@ void OwncloudSetupWizard::startWizard()
 
     _ocWizard->restart();
 
-    // settings re-initialized in initPage must be set here after restart
-    _ocWizard->setMultipleFoldersExist( multiFolderSetup );
-
     _ocWizard->open();
     _ocWizard->raise();
 }
@@ -148,7 +129,7 @@ void OwncloudSetupWizard::slotDetermineAuthType(const QString &urlString)
     // Reset the proxy which might had been determined previously in ConnectionValidator::checkServerAndAuth()
     // when there was a previous account.
     account->networkAccessManager()->setProxy(QNetworkProxy(QNetworkProxy::DefaultProxy));
-    // Set fake credentials beforfe we check what credential it actually is.
+    // Set fake credentials before we check what credential it actually is.
     account->setCredentials(CredentialsFactory::create("dummy"));
     CheckServerJob *job = new CheckServerJob(_ocWizard->account(), this);
     job->setIgnoreCredentialFailure(true);
@@ -189,6 +170,10 @@ void OwncloudSetupWizard::slotNoOwnCloudFoundAuth(QNetworkReply *reply)
                             .arg(Theme::instance()->appNameGUI(),
                                  reply->url().toString(),
                                  reply->errorString()), checkDowngradeAdvised(reply));
+
+    // Allow the credentials dialog to pop up again for the same URL.
+    // Maybe the user just clicked 'Cancel' by accident or changed his mind.
+    _ocWizard->account()->resetSslCertErrorState();
 }
 
 void OwncloudSetupWizard::slotNoOwnCloudFoundAuthTimeout(const QUrl&url)
@@ -263,11 +248,12 @@ void OwncloudSetupWizard::slotAuthError()
 
     // Provide messages for other errors, such as invalid credentials.
     } else if (reply->error() != QNetworkReply::NoError) {
-        errorMsg = reply->errorString();
         if (!_ocWizard->account()->credentials()->stillValid(reply)) {
             errorMsg = tr("Access forbidden by server. To verify that you have proper access, "
                           "<a href=\"%1\">click here</a> to access the service with your browser.")
                        .arg(_ocWizard->account()->url().toString());
+        } else {
+            errorMsg = errorMessage(reply->errorString(), reply->readAll());
         }
 
     // Something else went wrong, maybe the response was 200 but with invalid data.
@@ -453,15 +439,7 @@ bool OwncloudSetupWizard::ensureStartFromScratch(const QString &localFolder) {
     return renameOk;
 }
 
-void OwncloudSetupWizard::replaceDefaultAccountWith(AccountPtr newAccount)
-{
-    // new Account
-    AccountManager *mgr = AccountManager::instance();
-    mgr->setAccount(newAccount);
-    newAccount->save();
-}
-
-// Method executed when the user ends has finished the basic setup.
+// Method executed when the user end has finished the basic setup.
 void OwncloudSetupWizard::slotAssistantFinished( int result )
 {
     FolderMan *folderMan = FolderMan::instance();
@@ -473,22 +451,33 @@ void OwncloudSetupWizard::slotAssistantFinished( int result )
         // This may or may not wipe all folder definitions, depending
         // on whether a new account is activated or the existing one
         // is changed.
-        applyAccountChanges();
+        auto account = applyAccountChanges();
 
-        // But if the user went through with the folder config wizard,
-        // we assume they always want to have only that folder configured.
-        folderMan->removeAllFolderDefinitions();
-
-        QString localFolder = QDir::fromNativeSeparators(_ocWizard->localFolder());
-        if( !localFolder.endsWith(QLatin1Char('/'))) {
-            localFolder.append(QLatin1Char('/'));
-        }
+        QString localFolder = FolderDefinition::prepareLocalPath(_ocWizard->localFolder());
 
         bool startFromScratch = _ocWizard->field("OCSyncFromScratch").toBool();
         if (!startFromScratch || ensureStartFromScratch(localFolder)) {
             qDebug() << "Adding folder definition for" << localFolder << _remoteFolder;
-            folderMan->addFolderDefinition(Theme::instance()->appName(),
-                                           localFolder, _remoteFolder, _ocWizard->selectiveSyncBlacklist() );
+            FolderDefinition folderDefinition;
+            auto alias = Theme::instance()->appName();
+            int count = 0;
+            folderDefinition.alias = alias;
+            while (folderMan->folder(folderDefinition.alias)) {
+                // There is already a folder configured with this name and folder names need to be unique
+                folderDefinition.alias = alias + QString::number(++count);
+            }
+            folderDefinition.localPath = localFolder;
+            folderDefinition.targetPath = _remoteFolder;
+            folderDefinition.ignoreHiddenFiles = folderMan->ignoreHiddenFiles();
+
+            auto f = folderMan->addFolder(account, folderDefinition);
+            if (f) {
+                f->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList,
+                                                     _ocWizard->selectiveSyncBlacklist());
+                // The user already accepted the selective sync dialog. everything is in the white list
+                f->journalDb()->setSelectiveSyncList(SyncJournalDb::SelectiveSyncWhiteList,
+                                                     QStringList() << QLatin1String("/"));
+            }
             _ocWizard->appendToConfigurationLog(tr("<font color=\"green\"><b>Local sync folder %1 successfully created!</b></font>").arg(localFolder));
         }
     }
@@ -507,29 +496,14 @@ void OwncloudSetupWizard::slotSkipFolderConfiguration()
     emit ownCloudWizardDone( QDialog::Accepted );
 }
 
-void OwncloudSetupWizard::applyAccountChanges()
+AccountState *OwncloudSetupWizard::applyAccountChanges()
 {
     AccountPtr newAccount = _ocWizard->account();
-    AccountPtr origAccount = AccountManager::instance()->account();
+    auto manager = AccountManager::instance();
 
-    bool isInitialSetup = (origAccount == 0);
-
-    // check if either the account changed in a major way
-    bool reinitRequired =
-            newAccount->changed(origAccount, true /* ignoreProtocol, allows http->https */);
-
-    // If this is a completely new account, replace it entirely
-    // thereby clearing all folder definitions.
-    if (isInitialSetup || reinitRequired) {
-        replaceDefaultAccountWith(newAccount);
-        qDebug() << "Significant changes or first setup: switched to new account";
-    }
-    // Otherwise, set only URL and credentials
-    else {
-        origAccount->setUrl(newAccount->url());
-        origAccount->setCredentials(_ocWizard->getCredentials());
-        qDebug() << "Only password or schema was changed, adjusted existing account";
-    }
+    auto newState = manager->addAccount(newAccount);
+    manager->save();
+    return newState;
 }
 
 
