@@ -17,6 +17,7 @@
 
 #include "progressdispatcher.h"
 #include "macpreferenceswindow.h"
+#include "owncloudgui.h"
 
 class QStandardItemModel;
 class QListWidgetItem;
@@ -28,7 +29,14 @@ class ProtocolWidget;
 class Application;
 class FolderMan;
 class ownCloudGui;
+class Folder;
+class AccountState;
+class ActivitySettings;
 
+/**
+ * @brief The SettingsDialogMac class
+ * @ingroup gui
+ */
 class SettingsDialogMac : public MacPreferencesWindow
 {
     Q_OBJECT
@@ -36,18 +44,19 @@ class SettingsDialogMac : public MacPreferencesWindow
 public:
     explicit SettingsDialogMac(ownCloudGui *gui, QWidget *parent = 0);
 
-    void setGeneralErrors( const QStringList& errors );
-
 public slots:
-    void slotSyncStateChange(const QString& alias);
     void showActivityPage();
+    void slotRefreshActivity(AccountState *accountState );
 
+private slots:
+    void accountAdded(AccountState *);
+    void accountRemoved(AccountState *);
 private:
     void closeEvent(QCloseEvent *event);
 
-    AccountSettings *_accountSettings;
-    QListWidgetItem *_accountItem;
     ProtocolWidget  *_protocolWidget;
+    ActivitySettings *_activitySettings;
+    ownCloudGui     *_gui;
 
     int _protocolIdx;
 };
