@@ -16,6 +16,7 @@
 
 #include "socketapi.h"
 
+#include "config.h"
 #include "configfile.h"
 #include "folderman.h"
 #include "folder.h"
@@ -25,7 +26,10 @@
 #include "syncfileitem.h"
 #include "filesystem.h"
 #include "version.h"
+#include "account.h"
 #include "accountstate.h"
+#include "account.h"
+#include "capabilities.h"
 
 #include <QDebug>
 #include <QUrl>
@@ -36,6 +40,7 @@
 #include <QDir>
 #include <QApplication>
 #include <QLocalSocket>
+#include <QStringBuilder>
 
 #include <sqlite3.h>
 
@@ -50,29 +55,12 @@
 // The second number should be changed when there are new features.
 #define MIRALL_SOCKET_API_VERSION "1.0"
 
-extern "C" {
-
-enum csync_exclude_type_e {
-  CSYNC_NOT_EXCLUDED   = 0,
-  CSYNC_FILE_SILENTLY_EXCLUDED,
-  CSYNC_FILE_EXCLUDE_AND_REMOVE,
-  CSYNC_FILE_EXCLUDE_LIST,
-  CSYNC_FILE_EXCLUDE_INVALID_CHAR,
-  CSYNC_FILE_EXCLUDE_LONG_FILENAME
-};
-typedef enum csync_exclude_type_e CSYNC_EXCLUDE_TYPE;
-
-CSYNC_EXCLUDE_TYPE csync_excluded_no_ctx(c_strlist_t *excludes, const char *path, int filetype);
-int csync_exclude_load(const char *fname, c_strlist_t **list);
-}
-
 namespace OCC {
 
 #define DEBUG qDebug() << "SocketApi: "
 
 SocketApi::SocketApi(QObject* parent)
     : QObject(parent)
-    , _excludes(0)
 {
     QString socketPath;
 
@@ -84,13 +72,10 @@ SocketApi::SocketApi(QObject* parent)
         // See issue #2388
         // + Theme::instance()->appName();
     } else if (Utility::isMac()) {
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-        // Always using Qt5 on OS X
-        QString runtimeDir = QStandardPaths::writableLocation(QStandardPaths::GenericCacheLocation);
-        socketPath = runtimeDir + "/SyncStateHelper/" + Theme::instance()->appName() + ".socket";
-        // We use the generic SyncStateHelper name on OS X since the different branded clients
-        // should unfortunately not mention that they are ownCloud :-)
-#endif
+        // This must match the code signing Team setting of the extension
+        // Example for developer builds (with ad-hoc signing identity): "" "com.owncloud.desktopclient" ".socketApi"
+        // Example for official signed packages: "9B5WD74GWJ." "com.owncloud.desktopclient" ".socketApi"
+        socketPath = SOCKETAPI_TEAM_IDENTIFIER_PREFIX APPLICATION_REV_DOMAIN ".socketApi";
     } else if( Utility::isLinux() || Utility::isBSD() ) {
         QString runtimeDir;
 #if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
@@ -108,7 +93,7 @@ SocketApi::SocketApi(QObject* parent)
 	DEBUG << "An unexpected system detected";
     }
 
-    QLocalServer::removeServer(socketPath);
+    SocketApiServer::removeServer(socketPath);
     QFileInfo info(socketPath);
     if (!info.dir().exists()) {
         bool result = info.dir().mkpath(".");
@@ -127,11 +112,11 @@ SocketApi::SocketApi(QObject* parent)
     connect(&_localServer, SIGNAL(newConnection()), this, SLOT(slotNewConnection()));
 
     // folder watcher
-    connect(FolderMan::instance(), SIGNAL(folderSyncStateChange(QString)), this, SLOT(slotUpdateFolderView(QString)));
-    connect(ProgressDispatcher::instance(), SIGNAL(jobCompleted(QString,SyncFileItem)),
-            SLOT(slotJobCompleted(QString,SyncFileItem)));
-    connect(ProgressDispatcher::instance(), SIGNAL(syncItemDiscovered(QString,SyncFileItem)),
-            this, SLOT(slotSyncItemDiscovered(QString,SyncFileItem)));
+    connect(FolderMan::instance(), SIGNAL(folderSyncStateChange(Folder*)), this, SLOT(slotUpdateFolderView(Folder*)));
+    connect(ProgressDispatcher::instance(), SIGNAL(itemCompleted(QString, const SyncFileItem &, const PropagatorJob &)),
+            SLOT(slotItemCompleted(QString, const SyncFileItem &)));
+    connect(ProgressDispatcher::instance(), SIGNAL(syncItemDiscovered(QString, const SyncFileItem &)),
+            this, SLOT(slotSyncItemDiscovered(QString, const SyncFileItem &)));
 }
 
 SocketApi::~SocketApi()
@@ -141,33 +126,11 @@ SocketApi::~SocketApi()
     // All remaining sockets will be destroyed with _localServer, their parent
     Q_ASSERT(_listeners.isEmpty() || _listeners.first()->parent() == &_localServer);
     _listeners.clear();
-    slotClearExcludesList();
-}
-
-void SocketApi::slotClearExcludesList()
-{
-    c_strlist_clear(_excludes);
-}
-
-void SocketApi::slotReadExcludes()
-{
-    ConfigFile cfgFile;
-    slotClearExcludesList();
-    QString excludeList = cfgFile.excludeFile( ConfigFile::SystemScope );
-    if( !excludeList.isEmpty() ) {
-        qDebug() << "==== added system ignore list to socketapi:" << excludeList.toUtf8();
-        csync_exclude_load(excludeList.toUtf8(), &_excludes);
-    }
-    excludeList = cfgFile.excludeFile( ConfigFile::UserScope );
-    if( !excludeList.isEmpty() ) {
-        qDebug() << "==== added user defined ignore list to csync:" << excludeList.toUtf8();
-        csync_exclude_load(excludeList.toUtf8(), &_excludes);
-    }
 }
 
 void SocketApi::slotNewConnection()
 {
-    QLocalSocket* socket = _localServer.nextPendingConnection();
+    QIODevice* socket = _localServer.nextPendingConnection();
 
     if( ! socket ) {
         return;
@@ -179,16 +142,6 @@ void SocketApi::slotNewConnection()
 
     _listeners.append(socket);
 
-#ifdef Q_OS_MAC
-    // We want to tell our location so it can load the icons
-    // e.g. "/Users/guruz/woboq/owncloud/client/buildmirall/owncloud.app/Contents/MacOS/"
-    QString iconPath = qApp->applicationDirPath() + "/../Resources/icons/";
-    if (!QDir(iconPath).exists()) {
-        DEBUG << "Icon path " << iconPath << " does not exist, did you forget make install?";
-    }
-    broadcastMessage(QLatin1String("ICON_PATH"), iconPath );
-#endif
-
     foreach( Folder *f, FolderMan::instance()->map() ) {
         QString message = buildRegisterPathMessage(f->path());
         sendMessage(socket, message);
@@ -199,7 +152,7 @@ void SocketApi::onLostConnection()
 {
     DEBUG << "Lost connection " << sender();
 
-    QLocalSocket* socket = qobject_cast<QLocalSocket*>(sender());
+    QIODevice* socket = qobject_cast<QIODevice*>(sender());
     _listeners.removeAll(socket);
     socket->deleteLater();
 }
@@ -207,20 +160,21 @@ void SocketApi::onLostConnection()
 
 void SocketApi::slotReadSocket()
 {
-    QLocalSocket* socket = qobject_cast<QLocalSocket*>(sender());
+    QIODevice* socket = qobject_cast<QIODevice*>(sender());
     Q_ASSERT(socket);
 
     while(socket->canReadLine()) {
-        QString line = QString::fromUtf8(socket->readLine()).trimmed();
-        QString command = line.split(":").first();
+        QString line = QString::fromUtf8(socket->readLine());
+        line.chop(1); // remove the '\n'
+        QString command = line.split(":").value(0);
         QString function = QString(QLatin1String("command_")).append(command);
 
-        QString functionWithArguments = function + QLatin1String("(QString,QLocalSocket*)");
+        QString functionWithArguments = function + QLatin1String("(QString,QIODevice*)");
         int indexOfMethod = this->metaObject()->indexOfMethod(functionWithArguments.toAscii());
 
-        QString argument = line.remove(0, command.length()+1).trimmed();
+        QString argument = line.remove(0, command.length()+1);
         if(indexOfMethod != -1) {
-            QMetaObject::invokeMethod(this, function.toAscii(), Q_ARG(QString, argument), Q_ARG(QLocalSocket*, socket));
+            QMetaObject::invokeMethod(this, function.toAscii(), Q_ARG(QString, argument), Q_ARG(QIODevice*, socket));
         } else {
             DEBUG << "The command is not supported by this version of the client:" << command << "with argument:" << argument;
         }
@@ -232,7 +186,7 @@ void SocketApi::slotRegisterPath( const QString& alias )
     Folder *f = FolderMan::instance()->folder(alias);
     if (f) {
         QString message = buildRegisterPathMessage(f->path());
-        foreach(QLocalSocket *socket, _listeners) {
+        foreach(QIODevice *socket, _listeners) {
             sendMessage(socket, message);
         }
     }
@@ -245,14 +199,14 @@ void SocketApi::slotUnregisterPath( const QString& alias )
         broadcastMessage(QLatin1String("UNREGISTER_PATH"), f->path(), QString::null, true );
 
         if( _dbQueries.contains(f)) {
-            SqlQuery *h = _dbQueries[f];
+            auto h = _dbQueries[f];
             if( h ) {
                 h->finish();
             }
             _dbQueries.remove(f);
         }
         if( _openDbs.contains(f) ) {
-            SqlDatabase *db = _openDbs[f];
+            auto db = _openDbs[f];
             if( db ) {
                 db->close();
             }
@@ -261,13 +215,12 @@ void SocketApi::slotUnregisterPath( const QString& alias )
     }
 }
 
-void SocketApi::slotUpdateFolderView(const QString& alias)
+void SocketApi::slotUpdateFolderView(Folder *f)
 {
     if (_listeners.isEmpty()) {
         return;
     }
 
-    Folder *f = FolderMan::instance()->folder(alias);
     if (f) {
         // do only send UPDATE_VIEW for a couple of status
         if( f->syncResult().status() == SyncResult::SyncPrepare ||
@@ -278,16 +231,16 @@ void SocketApi::slotUpdateFolderView(const QString& alias)
                 f->syncResult().status() == SyncResult::SetupError ) {
 
             broadcastMessage(QLatin1String("STATUS"), f->path() ,
-                             this->fileStatus(f, "", _excludes).toSocketAPIString());
+                             this->fileStatus(f, "").toSocketAPIString());
 
             broadcastMessage(QLatin1String("UPDATE_VIEW"), f->path() );
         } else {
-            qDebug() << "Not sending UPDATE_VIEW for" << alias << "because status() is" << f->syncResult().status();
+            qDebug() << "Not sending UPDATE_VIEW for" << f->alias() << "because status() is" << f->syncResult().status();
         }
     }
 }
 
-void SocketApi::slotJobCompleted(const QString &folder, const SyncFileItem &item)
+void SocketApi::slotItemCompleted(const QString &folder, const SyncFileItem &item)
 {
     if (_listeners.isEmpty()) {
         return;
@@ -298,13 +251,9 @@ void SocketApi::slotJobCompleted(const QString &folder, const SyncFileItem &item
         return;
     }
 
+    auto status = this->fileStatus(f, item.destination());
     const QString path = f->path() + item.destination();
-
-    QString command = QLatin1String("OK");
-    if (Progress::isWarningKind(item._status)) {
-        command = QLatin1String("ERROR");
-    }
-    broadcastMessage(QLatin1String("STATUS"), path, command);
+    broadcastMessage(QLatin1String("STATUS"), path, status.toSocketAPIString());
 }
 
 void SocketApi::slotSyncItemDiscovered(const QString &folder, const SyncFileItem &item)
@@ -313,10 +262,6 @@ void SocketApi::slotSyncItemDiscovered(const QString &folder, const SyncFileItem
         return;
     }
 
-    if (item._instruction == CSYNC_INSTRUCTION_NONE) {
-        return;
-    }
-
     Folder *f = FolderMan::instance()->folder(folder);
     if (!f) {
         return;
@@ -325,7 +270,7 @@ void SocketApi::slotSyncItemDiscovered(const QString &folder, const SyncFileItem
     QString path = f->path() + item.destination();
 
     // the trailing slash for directories must be appended as the filenames coming in
-    // from the plugins have that too. Otherwise the according entry item is not found
+    // from the plugins have that too. Otherwise the matching entry item is not found
     // in the plugin.
     if( item._type == SyncFileItem::Type::Directory ) {
         path += QLatin1Char('/');
@@ -337,7 +282,7 @@ void SocketApi::slotSyncItemDiscovered(const QString &folder, const SyncFileItem
 
 
 
-void SocketApi::sendMessage(QLocalSocket *socket, const QString& message, bool doWait)
+void SocketApi::sendMessage(QIODevice *socket, const QString& message, bool doWait)
 {
     DEBUG << "Sending message: " << message;
     QString localMessage = message;
@@ -370,14 +315,12 @@ void SocketApi::broadcastMessage( const QString& verb, const QString& path, cons
         msg.append(QDir::toNativeSeparators(fi.absoluteFilePath()));
     }
 
-    // sendMessage already has a debug output
-    //DEBUG << "Broadcasting to" << _listeners.count() << "listeners: " << msg;
-    foreach(QLocalSocket *socket, _listeners) {
+    foreach(QIODevice *socket, _listeners) {
         sendMessage(socket, msg, doWait);
     }
 }
 
-void SocketApi::command_RETRIEVE_FOLDER_STATUS(const QString& argument, QLocalSocket* socket)
+void SocketApi::command_RETRIEVE_FOLDER_STATUS(const QString& argument, QIODevice* socket)
 {
     // This command is the same as RETRIEVE_FILE_STATUS
 
@@ -385,7 +328,7 @@ void SocketApi::command_RETRIEVE_FOLDER_STATUS(const QString& argument, QLocalSo
     command_RETRIEVE_FILE_STATUS(argument, socket);
 }
 
-void SocketApi::command_RETRIEVE_FILE_STATUS(const QString& argument, QLocalSocket* socket)
+void SocketApi::command_RETRIEVE_FILE_STATUS(const QString& argument, QIODevice* socket)
 {
     if( !socket ) {
         qDebug() << "No valid socket object.";
@@ -402,18 +345,17 @@ void SocketApi::command_RETRIEVE_FILE_STATUS(const QString& argument, QLocalSock
         DEBUG << "folder offline or not watched:" << argument;
         statusString = QLatin1String("NOP");
     } else {
-        const QString file = QDir::cleanPath(argument).mid(QDir::cleanPath(syncFolder->path()).length()+1);
-        SyncFileStatus fileStatus = this->fileStatus(syncFolder, file, _excludes);
+        const QString file = QDir::cleanPath(argument).mid(syncFolder->cleanPath().length()+1);
+        SyncFileStatus fileStatus = this->fileStatus(syncFolder, file);
 
         statusString = fileStatus.toSocketAPIString();
     }
 
-    QString message = QLatin1String("STATUS:")+statusString+QLatin1Char(':')
-            +QDir::toNativeSeparators(argument);
+    const QString message = QLatin1String("STATUS:") % statusString % QLatin1Char(':') %  QDir::toNativeSeparators(argument);
     sendMessage(socket, message);
 }
 
-void SocketApi::command_SHARE(const QString& localFile, QLocalSocket* socket)
+void SocketApi::command_SHARE(const QString& localFile, QIODevice* socket)
 {
     if (!socket) {
         qDebug() << Q_FUNC_INFO << "No valid socket object.";
@@ -432,10 +374,28 @@ void SocketApi::command_SHARE(const QString& localFile, QLocalSocket* socket)
         // if the folder isn't connected, don't open the share dialog
         sendMessage(socket, message);
     } else {
-        const QString folderForPath = shareFolder->path();
-        const QString remotePath = shareFolder->remotePath() + localFile.right(localFile.count()-folderForPath.count()+1);
+        const QString localFileClean = QDir::cleanPath(localFile);
+        const QString file = localFileClean.mid(shareFolder->cleanPath().length()+1);
+        SyncFileStatus fileStatus = this->fileStatus(shareFolder, file);
 
-        SyncJournalFileRecord rec = dbFileRecord_capi(shareFolder, localFile);
+        // Verify the file is on the server (to our knowledge of course)
+        if (fileStatus.tag() != SyncFileStatus::STATUS_UPTODATE &&
+            fileStatus.tag() != SyncFileStatus::STATUS_UPDATED) {
+            const QString message = QLatin1String("SHARE:NOTSYNCED:")+QDir::toNativeSeparators(localFile);
+            sendMessage(socket, message);
+            return;
+        }
+
+        const QString remotePath = shareFolder->remotePath() + QLatin1Char('/') + file;
+
+        // Can't share root folder
+        if (remotePath == "/") {
+            const QString message = QLatin1String("SHARE:CANNOTSHAREROOT:")+QDir::toNativeSeparators(localFile);
+            sendMessage(socket, message);
+            return;
+        }
+
+        SyncJournalFileRecord rec = dbFileRecord_capi(shareFolder, localFileClean);
 
         bool allowReshare = true; // lets assume the good
         if( rec.isValid() ) {
@@ -447,16 +407,60 @@ void SocketApi::command_SHARE(const QString& localFile, QLocalSocket* socket)
         const QString message = QLatin1String("SHARE:OK:")+QDir::toNativeSeparators(localFile);
         sendMessage(socket, message);
 
-        emit shareCommandReceived(remotePath, localFile, allowReshare);
+        emit shareCommandReceived(remotePath, localFileClean, allowReshare);
     }
 }
 
-void SocketApi::command_VERSION(const QString&, QLocalSocket* socket)
+void SocketApi::command_VERSION(const QString&, QIODevice* socket)
 {
     sendMessage(socket, QLatin1String("VERSION:" MIRALL_VERSION_STRING ":" MIRALL_SOCKET_API_VERSION));
 }
 
-void SocketApi::command_SHARE_MENU_TITLE(const QString &, QLocalSocket* socket)
+void SocketApi::command_SHARE_STATUS(const QString &localFile, QIODevice *socket)
+{
+    if (!socket) {
+        qDebug() << Q_FUNC_INFO << "No valid socket object.";
+        return;
+    }
+
+    qDebug() << Q_FUNC_INFO << localFile;
+
+    Folder *shareFolder = FolderMan::instance()->folderForPath(localFile);
+
+    if (!shareFolder) {
+        const QString message = QLatin1String("SHARE_STATUS:NOP:")+QDir::toNativeSeparators(localFile);
+        sendMessage(socket, message);
+    } else {
+        const QString file = QDir::cleanPath(localFile).mid(shareFolder->cleanPath().length()+1);
+        SyncFileStatus fileStatus = this->fileStatus(shareFolder, file);
+
+        // Verify the file is on the server (to our knowledge of course)
+        if (fileStatus.tag() != SyncFileStatus::STATUS_UPTODATE &&
+            fileStatus.tag() != SyncFileStatus::STATUS_UPDATED) {
+            const QString message = QLatin1String("SHARE_STATUS:NOTSYNCED:")+QDir::toNativeSeparators(localFile);
+            sendMessage(socket, message);
+            return;
+        }
+
+        const Capabilities capabilities = shareFolder->accountState()->account()->capabilities();
+
+        if (!capabilities.shareAPI()) {
+            const QString message = QLatin1String("SHARE_STATUS:DISABLED:")+QDir::toNativeSeparators(localFile);
+            sendMessage(socket, message);
+        } else {
+            QString available = "USER,GROUP";
+
+            if (capabilities.sharePublicLink()) {
+                available += ",LINK";
+            }
+
+            const QString message = QLatin1String("SHARE_STATUS:") + available + ":" + QDir::toNativeSeparators(localFile);
+            sendMessage(socket, message);
+        }
+    }
+}
+
+void SocketApi::command_SHARE_MENU_TITLE(const QString &, QIODevice* socket)
 {
     sendMessage(socket, QLatin1String("SHARE_MENU_TITLE:") + tr("Share with %1", "parameter is ownCloud").arg(Theme::instance()->appNameGUI()));
 }
@@ -476,32 +480,31 @@ SqlQuery* SocketApi::getSqlQuery( Folder *folder )
     }
 
     if( _dbQueries.contains(folder) ) {
-        return _dbQueries[folder];
+        return _dbQueries[folder].data();
     }
 
     /* No valid sql query object yet for this folder */
     int rc;
-    const QString sql("SELECT inode, mode, modtime, type, md5, fileid, remotePerm FROM "
+    const QString sql("SELECT inode, modtime, type, md5, fileid, remotePerm FROM "
                       "metadata WHERE phash=?1");
     QString dbFileName = folder->journalDb()->databaseFilePath();
 
     QFileInfo fi(dbFileName);
     if( fi.exists() ) {
-        SqlDatabase *db = new SqlDatabase;
+        auto db = QSharedPointer<SqlDatabase>::create();
 
         if( db && db->openReadOnly(dbFileName) ) {
             _openDbs.insert(folder, db);
 
-            SqlQuery *query = new SqlQuery(*db);
+            QSharedPointer<SqlQuery> query(new SqlQuery(*db));
             rc = query->prepare(sql);
 
             if( rc != SQLITE_OK ) {
-                delete query;
                 qDebug() << "Unable to prepare the query statement:" << rc;
                 return 0; // do not insert into hash
             }
             _dbQueries.insert( folder, query);
-            return query;
+            return query.data();
         } else {
             qDebug() << "Unable to open db" << dbFileName;
         }
@@ -536,12 +539,11 @@ SyncJournalFileRecord SocketApi::dbFileRecord_capi( Folder *folder, QString file
         if (query->next()) {
             rec._path    = fileName;
             rec._inode   = query->int64Value(0);
-            rec._mode    = query->intValue(1);
-            rec._modtime = Utility::qDateTimeFromTime_t( query->int64Value(2));
-            rec._type    = query->intValue(3);
-            rec._etag    = query->baValue(4);
-            rec._fileId  = query->baValue(5);
-            rec._remotePerm = query->baValue(6);
+            rec._modtime = Utility::qDateTimeFromTime_t( query->int64Value(1));
+            rec._type    = query->intValue(2);
+            rec._etag    = query->baValue(3);
+            rec._fileId  = query->baValue(4);
+            rec._remotePerm = query->baValue(5);
         }
         query->reset();
     }
@@ -551,14 +553,14 @@ SyncJournalFileRecord SocketApi::dbFileRecord_capi( Folder *folder, QString file
 /**
  * Get status about a single file.
  */
-SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileName, c_strlist_t *excludes )
+SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileName)
 {
     QString file = folder->path();
     QString fileName = systemFileName.normalized(QString::NormalizationForm_C);
     QString fileNameSlash = fileName;
 
-    if( fileName != QLatin1String("/") && !fileName.isEmpty() ) {
-        file = folder->path() + fileName;
+    if(fileName != QLatin1String("/") && !fileName.isEmpty()) {
+        file += fileName;
     }
 
     if( fileName.endsWith(QLatin1Char('/')) ) {
@@ -568,7 +570,8 @@ SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileNa
         fileNameSlash += QLatin1Char('/');
     }
 
-    if( !FileSystem::fileExists(file) ) {
+    const QFileInfo fi(file);
+    if( !FileSystem::fileExists(file, fi) ) {
         qDebug() << "OO File " << file << " is not existing";
         return SyncFileStatus(SyncFileStatus::STATUS_STAT_ERROR);
     }
@@ -576,7 +579,6 @@ SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileNa
     // file is ignored?
     // Qt considers .lnk files symlinks on Windows so we need to work
     // around that here.
-    const QFileInfo fi(file);
     if( fi.isSymLink()
 #ifdef Q_OS_WIN
             && fi.suffix() != "lnk"
@@ -591,13 +593,12 @@ SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileNa
     }
 
     // Is it excluded?
-    CSYNC_EXCLUDE_TYPE excl = csync_excluded_no_ctx(excludes, fileName.toUtf8(), type);
-    if( excl != CSYNC_NOT_EXCLUDED ) {
+    if( folder->isFileExcludedRelative(fileName) ) {
         return SyncFileStatus(SyncFileStatus::STATUS_IGNORE);
     }
 
-    // Error if it is in the selective sync blacklistr
-    foreach(const auto &s, folder->selectiveSyncBlackList()) {
+    // Error if it is in the selective sync blacklist
+    foreach(const auto &s, folder->journalDb()->getSelectiveSyncList(SyncJournalDb::SelectiveSyncBlackList)) {
         if (fileNameSlash.startsWith(s)) {
             return SyncFileStatus(SyncFileStatus::STATUS_ERROR);
         }
@@ -607,7 +608,7 @@ SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileNa
     SyncJournalFileRecord rec = dbFileRecord_capi(folder, fileName );
 
     if (folder->estimateState(fileName, type, &status)) {
-        qDebug() << Q_FUNC_INFO << "Folder estimated status for" << fileName << "to" << status.toSocketAPIString();
+        qDebug() << "Folder estimated status for" << fileName << "to" << status.toSocketAPIString();
     } else if (fileName == "") {
         // sync folder itself
         switch (folder->syncResult().status()) {
@@ -620,7 +621,7 @@ SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileNa
 
         case SyncResult::Success:
         case SyncResult::Problem:
-            status.set(SyncFileStatus::STATUS_SYNC);
+            status.set(SyncFileStatus::STATUS_UPTODATE);
             return status;
 
         case SyncResult::Error:
@@ -635,15 +636,15 @@ SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileNa
         }
     } else if (type == CSYNC_FTW_TYPE_DIR) {
         if (rec.isValid()) {
-            status.set(SyncFileStatus::STATUS_SYNC);
+            status.set(SyncFileStatus::STATUS_UPTODATE);
         } else {
-            qDebug() << Q_FUNC_INFO << "Could not determine state for folder" << fileName << "will set STATUS_NEW";
+            qDebug() << "Could not determine state for folder" << fileName << "will set STATUS_NEW";
             status.set(SyncFileStatus::STATUS_NEW);
         }
     } else if (type == CSYNC_FTW_TYPE_FILE) {
         if (rec.isValid()) {
             if( FileSystem::getModTime(fi.absoluteFilePath()) == Utility::qDateTimeToTime_t(rec._modtime) ) {
-                status.set(SyncFileStatus::STATUS_SYNC);
+                status.set(SyncFileStatus::STATUS_UPTODATE);
             } else {
                 if (rec._remotePerm.isNull() || rec._remotePerm.contains("W") ) {
                     status.set(SyncFileStatus::STATUS_EVAL);
@@ -652,7 +653,7 @@ SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileNa
                 }
             }
         } else {
-            qDebug() << Q_FUNC_INFO << "Could not determine state for file" << fileName << "will set STATUS_NEW";
+            qDebug() << "Could not determine state for file" << fileName << "will set STATUS_NEW";
             status.set(SyncFileStatus::STATUS_NEW);
         }
     }
@@ -661,7 +662,7 @@ SyncFileStatus SocketApi::fileStatus(Folder *folder, const QString& systemFileNa
         if (rec._remotePerm.isNull()) {
             // probably owncloud 6, that does not have permissions flag yet.
             QString url = folder->remoteUrl().toString() + fileName;
-            if (url.contains(QLatin1String("/remote.php/webdav/Shared/"))) {
+            if (url.contains( folder->accountState()->account()->davPath() + QLatin1String("Shared/") )) {
                 status.setSharedWithMe(true);
             }
         } else if (rec._remotePerm.contains("S")) {
