@@ -26,6 +26,10 @@ namespace OCC {
 
 class AccountState;
 
+/**
+ * @brief The SslButton class
+ * @ingroup gui
+ */
 class SslButton : public QToolButton
 {
     Q_OBJECT
@@ -41,6 +45,7 @@ private:
     QMenu* buildCertMenu(QMenu *parent, const QSslCertificate& cert,
                          const QList<QSslCertificate>& userApproved, int pos);
     QPointer<AccountState> _accountState;
+    QMenu* _menu;
 };
 
 } // namespace OCC
