@@ -13,14 +13,13 @@
 
 #include "account.h"
 #include "cookiejar.h"
-#include "theme.h"
 #include "networkjobs.h"
 #include "configfile.h"
 #include "accessmanager.h"
-#include "owncloudtheme.h"
 #include "creds/abstractcredentials.h"
-#include "creds/credentialsfactory.h"
 #include "../3rdparty/certificates/p12topem.h"
+#include "capabilities.h"
+#include "theme.h"
 
 #include <QSettings>
 #include <QMutex>
@@ -35,47 +34,14 @@
 
 namespace OCC {
 
-static const char urlC[] = "url";
-static const char authTypeC[] = "authType";
-static const char userC[] = "user";
-static const char httpUserC[] = "http_user";
-static const char caCertsKeyC[] = "CaCertificates";
-
-AccountManager *AccountManager::_instance = 0;
-
-AccountManager *AccountManager::instance()
-{
-    static QMutex mutex;
-    if (!_instance)
-    {
-        QMutexLocker lock(&mutex);
-        if (!_instance) {
-            _instance = new AccountManager;
-        }
-    }
-
-    return _instance;
-}
-
-void AccountManager::setAccount(AccountPtr account)
-{
-    if (_account) {
-        emit accountRemoved(_account);
-    }
-    _account = account;
-    if (account) {
-        emit accountAdded(account);
-    }
-}
-
 
 Account::Account(QObject *parent)
     : QObject(parent)
-    , _url(Theme::instance()->overrideServerUrl())
+    , _capabilities(QVariantMap())
     , _am(0)
     , _credentials(0)
     , _treatSslErrorsAsFailure(false)
-    , _davPath("remote.php/webdav/")
+    , _davPath( Theme::instance()->webDavPath() )
     , _wasMigrated(false)
 {
     qRegisterMetaType<AccountPtr>("AccountPtr");
@@ -94,6 +60,17 @@ Account::~Account()
     delete _am;
 }
 
+QString Account::davPath() const
+{
+    // make sure to have a trailing slash
+    if( !_davPath.endsWith('/') ) {
+        QString dp(_davPath);
+        dp.append('/');
+        return dp;
+    }
+    return _davPath;
+}
+
 void Account::setSharedThis(AccountPtr sharedThis)
 {
     _sharedThis = sharedThis.toWeakRef();
@@ -104,112 +81,22 @@ AccountPtr Account::sharedFromThis()
     return _sharedThis.toStrongRef();
 }
 
-void Account::save()
+
+QString Account::displayName() const
 {
-    QScopedPointer<QSettings> settings(settingsWithGroup(Theme::instance()->appName()));
-    settings->setValue(QLatin1String(urlC), _url.toString());
-    if (_credentials) {
-        _credentials->persist();
-        Q_FOREACH(QString key, _settingsMap.keys()) {
-            settings->setValue(key, _settingsMap.value(key));
-        }
-        settings->setValue(QLatin1String(authTypeC), _credentials->authType());
-
-        // HACK: Save http_user also as user
-        if (_settingsMap.contains(httpUserC))
-            settings->setValue(userC, _settingsMap.value(httpUserC));
-    }
-    settings->sync();
-
-    // Save accepted certificates.
-    settings->beginGroup(QLatin1String("General"));
-    qDebug() << "Saving " << approvedCerts().count() << " unknown certs.";
-    QByteArray certs;
-    Q_FOREACH( const QSslCertificate& cert, approvedCerts() ) {
-        certs += cert.toPem() + '\n';
-    }
-    if (!certs.isEmpty()) {
-        settings->setValue( QLatin1String(caCertsKeyC), certs );
-    }
-
-    // Save cookies.
-    if (_am) {
-        CookieJar* jar = qobject_cast<CookieJar*>(_am->cookieJar());
-        if (jar) {
-            qDebug() << "Saving cookies.";
-            jar->save();
-        }
+    auto user = _credentials->user();
+    QString dn = QString("%1@%2").arg(user, _url.host());
+    int port = url().port();
+    if (port > 0 && port != 80 && port != 443) {
+        dn.append(QLatin1Char(':'));
+        dn.append(QString::number(port));
     }
+    return dn;
 }
 
-AccountPtr Account::restore()
+QString Account::id() const
 {
-    // try to open the correctly themed settings
-    QScopedPointer<QSettings> settings(settingsWithGroup(Theme::instance()->appName()));
-
-    AccountPtr acc;
-    bool migratedCreds = false;
-
-    // if the settings file could not be opened, the childKeys list is empty
-    if( settings->childKeys().isEmpty() ) {
-        // Now try to open the original ownCloud settings to see if they exist.
-        QString oCCfgFile = QDir::fromNativeSeparators( settings->fileName() );
-        // replace the last two segments with ownCloud/owncloud.cfg
-        oCCfgFile = oCCfgFile.left( oCCfgFile.lastIndexOf('/'));
-        oCCfgFile = oCCfgFile.left( oCCfgFile.lastIndexOf('/'));
-        oCCfgFile += QLatin1String("/ownCloud/owncloud.cfg");
-
-        qDebug() << "Migrate: checking old config " << oCCfgFile;
-
-        QFileInfo fi( oCCfgFile );
-        if( fi.isReadable() ) {
-            QSettings *oCSettings = new QSettings(oCCfgFile, QSettings::IniFormat);
-            oCSettings->beginGroup(QLatin1String("ownCloud"));
-
-            // Check the theme url to see if it is the same url that the oC config was for
-            QString overrideUrl = Theme::instance()->overrideServerUrl();
-            if( !overrideUrl.isEmpty() ) {
-                if (overrideUrl.endsWith('/')) { overrideUrl.chop(1); }
-                QString oCUrl = oCSettings->value(QLatin1String(urlC)).toString();
-                if (oCUrl.endsWith('/')) { oCUrl.chop(1); }
-
-                // in case the urls are equal reset the settings object to read from
-                // the ownCloud settings object
-                qDebug() << "Migrate oC config if " << oCUrl << " == " << overrideUrl << ":"
-                         << (oCUrl == overrideUrl ? "Yes" : "No");
-                if( oCUrl == overrideUrl ) {
-                    migratedCreds = true;
-                    settings.reset( oCSettings );
-                } else {
-                    delete oCSettings;
-                }
-            }
-        }
-    }
-
-    if (!settings->childKeys().isEmpty()) {
-        acc = AccountPtr(new Account);
-        acc->setSharedThis(acc);
-
-        acc->setUrl(settings->value(QLatin1String(urlC)).toUrl());
-        acc->setCredentials(CredentialsFactory::create(settings->value(QLatin1String(authTypeC)).toString()));
-
-        // We want to only restore settings for that auth type and the user value
-        acc->_settingsMap.insert(QLatin1String(userC), settings->value(userC));
-        QString authTypePrefix = settings->value(authTypeC).toString() + "_";
-        Q_FOREACH(QString key, settings->childKeys()) {
-            if (!key.startsWith(authTypePrefix))
-                continue;
-            acc->_settingsMap.insert(key, settings->value(key));
-        }
-
-        // now the cert, it is in the general group
-        settings->beginGroup(QLatin1String("General"));
-        acc->setApprovedCerts(QSslCertificate::fromData(settings->value(caCertsKeyC).toByteArray()));
-        acc->setMigrated(migratedCreds);
-        return acc;
-    }
-    return AccountPtr();
+    return _id;
 }
 
 static bool isEqualExceptProtocol(const QUrl &url1, const QUrl &url2)
@@ -255,16 +142,24 @@ void Account::setCredentials(AbstractCredentials *cred)
     if (_credentials) {
         credentials()->deleteLater();
     }
-    cred->setAccount(this);
+
+    // The order for these two is important! Reading the credential's
+    // settings accesses the account as well as account->_credentials
     _credentials = cred;
+    cred->setAccount(this);
+
     _am = _credentials->getQNAM();
     if (jar) {
         _am->setCookieJar(jar);
     }
     connect(_am, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)),
-            SLOT(slotHandleErrors(QNetworkReply*,QList<QSslError>)));
+            SLOT(slotHandleSslErrors(QNetworkReply*,QList<QSslError>)));
+    connect(_am, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),
+            SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
     connect(_credentials, SIGNAL(fetched()),
             SLOT(slotCredentialsFetched()));
+    connect(_credentials, SIGNAL(asked()),
+            SLOT(slotCredentialsAsked()));
 }
 
 QUrl Account::davUrl() const
@@ -285,7 +180,7 @@ void Account::clearCookieJar()
 
 /*! This shares our official cookie jar (containing all the tasty
     authentication cookies) with another QNAM while making sure
-    of not loosing its ownership. */
+    of not losing its ownership. */
 void Account::lendCookieJarTo(QNetworkAccessManager *guest)
 {
     auto jar = _am->cookieJar();
@@ -306,7 +201,9 @@ void Account::resetNetworkAccessManager()
     _am = _credentials->getQNAM();
     _am->setCookieJar(jar); // takes ownership of the old cookie jar
     connect(_am, SIGNAL(sslErrors(QNetworkReply*,QList<QSslError>)),
-            SLOT(slotHandleErrors(QNetworkReply*,QList<QSslError>)));
+            SLOT(slotHandleSslErrors(QNetworkReply*,QList<QSslError>)));
+    connect(_am, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)),
+            SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)));
 }
 
 QNetworkAccessManager *Account::networkAccessManager()
@@ -322,6 +219,9 @@ QNetworkReply *Account::headRequest(const QString &relPath)
 QNetworkReply *Account::headRequest(const QUrl &url)
 {
     QNetworkRequest request(url);
+#if QT_VERSION > QT_VERSION_CHECK(4, 8, 4)
+    request.setSslConfiguration(this->getOrCreateSslConfig());
+#endif
     return _am->head(request);
 }
 
@@ -334,7 +234,7 @@ QNetworkReply *Account::getRequest(const QUrl &url)
 {
     QNetworkRequest request(url);
 #if QT_VERSION > QT_VERSION_CHECK(4, 8, 4)
-    request.setSslConfiguration(this->createSslConfig());
+    request.setSslConfiguration(this->getOrCreateSslConfig());
 #endif
     return _am->get(request);
 }
@@ -348,7 +248,7 @@ QNetworkReply *Account::davRequest(const QByteArray &verb, const QUrl &url, QNet
 {
     req.setUrl(url);
 #if QT_VERSION > QT_VERSION_CHECK(4, 8, 4)
-    req.setSslConfiguration(this->createSslConfig());
+    req.setSslConfiguration(this->getOrCreateSslConfig());
 #endif
     return _am->sendCustomRequest(req, verb, data);
 }
@@ -364,16 +264,19 @@ void Account::setSslConfiguration(const QSslConfiguration &config)
     _sslConfiguration = config;
 }
 
-QSslConfiguration Account::createSslConfig()
+QSslConfiguration Account::getOrCreateSslConfig()
 {
+    if (!_sslConfiguration.isNull()) {
+        // Will be set by CheckServerJob::finished()
+        // We need to use a central shared config to get SSL session tickets
+        return _sslConfiguration;
+    }
+
     // if setting the client certificate fails, you will probably get an error similar to this:
     //  "An internal error number 1060 happened. SSL handshake failed, client certificate was requested: SSL error: sslv3 alert handshake failure"
-  
-    // maybe this code must not have to be reevaluated every request?
-    QSslConfiguration sslConfig;
+    QSslConfiguration sslConfig = QSslConfiguration::defaultConfiguration();
     QSslCertificate sslClientCertificate;
     
-    // maybe move this code from createSslConfig to the Account constructor
     ConfigFile cfgFile;
     if(!cfgFile.certificatePath().isEmpty() && !cfgFile.certificatePasswd().isEmpty()) {
         resultP12ToPem certif = p12ToPem(cfgFile.certificatePath().toStdString(), cfgFile.certificatePasswd().toStdString());
@@ -391,14 +294,19 @@ QSslConfiguration Account::createSslConfig()
         QSslKey privateKey(_pemPrivateKey.toLocal8Bit(), QSsl::Rsa, QSsl::Pem, QSsl::PrivateKey , "");
 
         // SSL configuration
-        sslConfig.defaultConfiguration();
         sslConfig.setCaCertificates(QSslSocket::systemCaCertificates());
-        QList<QSslCertificate> caCertifs = sslConfig.caCertificates();
         sslConfig.setLocalCertificate(sslClientCertificate);
         sslConfig.setPrivateKey(privateKey);
         qDebug() << "Added SSL client certificate to the query";
     }
 
+#if QT_VERSION > QT_VERSION_CHECK(5, 2, 0)
+    // Try hard to re-use session for different requests
+    sslConfig.setSslOption(QSsl::SslOptionDisableSessionTickets, false);
+    sslConfig.setSslOption(QSsl::SslOptionDisableSessionSharing, false);
+    sslConfig.setSslOption(QSsl::SslOptionDisableSessionPersistence, false);
+#endif
+
     return sslConfig;
 }
 
@@ -452,14 +360,14 @@ QUrl Account::concatUrlPath(const QUrl &url, const QString &concatPath,
 
 QString Account::_configFileName;
 
-QSettings *Account::settingsWithGroup(const QString& group, QObject *parent)
+std::unique_ptr<QSettings> Account::settingsWithGroup(const QString& group, QObject *parent)
 {
     if (_configFileName.isEmpty()) {
         // cache file name
         ConfigFile cfg;
         _configFileName = cfg.configFile();
     }
-    QSettings *settings = new QSettings(_configFileName, QSettings::IniFormat, parent);
+    std::unique_ptr<QSettings> settings(new QSettings(_configFileName, QSettings::IniFormat, parent));
     settings->beginGroup(group);
     return settings;
 }
@@ -485,7 +393,7 @@ void Account::setCredentialSetting(const QString &key, const QVariant &value)
     }
 }
 
-void Account::slotHandleErrors(QNetworkReply *reply , QList<QSslError> errors)
+void Account::slotHandleSslErrors(QNetworkReply *reply , QList<QSslError> errors)
 {
     NetworkJobTimeoutPauser pauser(reply);
     QString out;
@@ -510,9 +418,14 @@ void Account::slotHandleErrors(QNetworkReply *reply , QList<QSslError> errors)
     if (_sslErrorHandler->handleErrors(errors, reply->sslConfiguration(), &approvedCerts, sharedFromThis())) {
         QSslSocket::addDefaultCaCertificates(approvedCerts);
         addApprovedCerts(approvedCerts);
+        emit wantsAccountSaved(sharedFromThis());
         // all ssl certs are known and accepted. We can ignore the problems right away.
 //         qDebug() << out << "Certs are known and trusted! This is not an actual error.";
-        reply->ignoreSslErrors();
+
+        // Warning: Do *not* use ignoreSslErrors() (without args) here:
+        // it permanently ignores all SSL errors for this host, even
+        // certificate changes.
+        reply->ignoreSslErrors(errors);
     } else {
         _treatSslErrorsAsFailure = true;
         // if during normal operation, a new certificate was MITM'ed, and the user does not
@@ -529,16 +442,13 @@ void Account::slotCredentialsFetched()
     emit credentialsFetched(_credentials);
 }
 
+void Account::slotCredentialsAsked()
+{
+    emit credentialsAsked(_credentials);
+}
+
 void Account::handleInvalidCredentials()
 {
-    // invalidate & forget token/password
-    // but try to re-sign in.
-    if (_credentials->ready()) {
-        _credentials->invalidateAndFetch();
-    } else {
-        _credentials->fetch();
-    }
-
     emit invalidCredentials();
 }
 
@@ -552,14 +462,14 @@ void Account::setMigrated(bool mig)
     _wasMigrated = mig;
 }
 
-QVariantMap Account::capabilities()
+const Capabilities &Account::capabilities() const
 {
     return _capabilities;
 }
 
 void Account::setCapabilities(const QVariantMap &caps)
 {
-    _capabilities = caps;
+    _capabilities = Capabilities(caps);
 }
 
 QString Account::serverVersion()
@@ -567,9 +477,34 @@ QString Account::serverVersion()
     return _serverVersion;
 }
 
+int Account::serverVersionInt()
+{
+    // FIXME: Use Qt 5.5 QVersionNumber
+    auto components = serverVersion().split('.');
+    return  (components.value(0).toInt() << 16)
+                   + (components.value(1).toInt() << 8)
+                   + components.value(2).toInt();
+}
+
 void Account::setServerVersion(const QString& version)
 {
     _serverVersion = version;
 }
 
+bool Account::rootEtagChangesNotOnlySubFolderEtags()
+{
+    return (serverVersionInt() >= 0x080100);
+}
+
+void Account::setNonShib(bool nonShib)
+{
+    if( nonShib ) {
+        _davPath = Theme::instance()->webDavPathNonShib();
+    } else {
+        _davPath = Theme::instance()->webDavPath();
+    } 
+}
+
+
+
 } // namespace OCC
