@@ -26,6 +26,10 @@ namespace OCC {
 
 class ConfigFile;
 
+/**
+ * @brief The ClientProxy class
+ * @ingroup libsync
+ */
 class OWNCLOUDSYNC_EXPORT ClientProxy : public QObject
 {
     Q_OBJECT
@@ -36,7 +40,6 @@ public:
     static void lookupSystemProxyAsync(const QUrl &url, QObject *dst, const char *slot);
 
 public slots:
-    void setCSyncProxy( const QUrl& url, CSYNC *csync_ctx );
     void setupQtProxyFromConfig();
 
 private:
