@@ -19,7 +19,6 @@
 #include "utility.h"
 
 #include "creds/abstractcredentials.h"
-#include "creds/credentialsfactory.h"
 
 #ifndef TOKEN_AUTH_ONLY
 #include <QWidget>
@@ -47,8 +46,11 @@ static const char monoIconsC[] = "monoIcons";
 static const char crashReporterC[] = "crashReporter";
 static const char optionalDesktopNoficationsC[] = "optionalDesktopNotifications";
 static const char skipUpdateCheckC[] = "skipUpdateCheck";
+static const char updateCheckIntervalC[] = "updateCheckInterval";
 static const char geometryC[] = "geometry";
 static const char timeoutC[] = "timeout";
+static const char chunkSizeC[] = "chunkSize";
+static const char transmissionChecksumC[] = "transmissionChecksum";
 
 static const char proxyHostC[] = "Proxy/host";
 static const char proxyTypeC[] = "Proxy/type";
@@ -62,6 +64,9 @@ static const char useDownloadLimitC[] = "BWLimit/useDownloadLimit";
 static const char uploadLimitC[]      = "BWLimit/uploadLimit";
 static const char downloadLimitC[]    = "BWLimit/downloadLimit";
 
+static const char newBigFolderSizeLimitC[] = "newBigFolderSizeLimit";
+static const char useNewBigFolderSizeLimitC[] = "useNewBigFolderSizeLimit";
+
 static const char maxLogLinesC[] = "Logging/maxLogLines";
 
 const char certPath[] = "http_certificatePath";
@@ -91,11 +96,9 @@ bool ConfigFile::setConfDir(const QString &value)
     if( dirPath.isEmpty() ) return false;
 
     QFileInfo fi(dirPath);
-    if ( !fi.exists() && !fi.isAbsolute() ) {
-        QDir::current().mkdir(dirPath);
-        QDir dir = QDir::current();
-        dir.cd(dirPath);
-        fi.setFile(dir.path());
+    if ( !fi.exists()  ) {
+        QDir().mkpath(dirPath);
+        fi.setFile(dirPath);
     }
     if( fi.exists() && fi.isDir() ) {
         dirPath = fi.absoluteFilePath();
@@ -118,6 +121,26 @@ int ConfigFile::timeout() const
     return settings.value(QLatin1String(timeoutC), 300).toInt(); // default to 5 min
 }
 
+quint64 ConfigFile::chunkSize() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+    return settings.value(QLatin1String(chunkSizeC), 5*1024*1024).toLongLong(); // default to 5 MiB
+}
+
+QString ConfigFile::transmissionChecksum() const
+{
+    QSettings settings(configFile(), QSettings::IniFormat);
+
+    QString checksum = settings.value(QLatin1String(transmissionChecksumC), QString()).toString();
+
+    if( checksum.isEmpty() ) {
+        // if the config file setting is empty, maybe the Branding requires it.
+        checksum = Theme::instance()->transmissionChecksum();
+    }
+
+    return checksum;
+}
+
 void ConfigFile::setOptionalDesktopNotifications(bool show)
 {
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -339,7 +362,7 @@ void ConfigFile::setRemotePollInterval(int interval, const QString &connection )
     if( connection.isEmpty() ) con = defaultConnection();
 
     if( interval < 5000 ) {
-        qDebug() << "Remote Poll interval of " << interval << " is below fife seconds.";
+        qDebug() << "Remote Poll interval of " << interval << " is below five seconds.";
         return;
     }
     QSettings settings(configFile(), QSettings::IniFormat);
@@ -366,6 +389,24 @@ quint64 ConfigFile::forceSyncInterval(const QString& connection) const
     return interval;
 }
 
+int ConfigFile::updateCheckInterval( const QString& connection ) const
+{
+    QString con( connection );
+    if( connection.isEmpty() ) con = defaultConnection();
+    QSettings settings(configFile(), QSettings::IniFormat);
+    settings.beginGroup( con );
+
+    int defaultInterval = 1000*60*60*10; // ten hours
+    int interval = settings.value( QLatin1String(updateCheckIntervalC), defaultInterval ).toInt();
+
+    int minInterval = 1000*60*5;
+    if( interval < minInterval) {
+        qDebug() << "Update check interval less than five minutes, setting " << minInterval;
+        interval = minInterval;
+    }
+    return interval;
+}
+
 bool ConfigFile::skipUpdateCheck( const QString& connection ) const
 {
     QString con( connection );
@@ -500,9 +541,9 @@ int ConfigFile::useUploadLimit() const
     return getValue(useUploadLimitC, QString::null, 0).toInt();
 }
 
-bool ConfigFile::useDownloadLimit() const
+int ConfigFile::useDownloadLimit() const
 {
-    return getValue(useDownloadLimitC, QString::null, false).toBool();
+    return getValue(useDownloadLimitC, QString::null, 0).toInt();
 }
 
 void ConfigFile::setUseUploadLimit(int val)
@@ -510,9 +551,9 @@ void ConfigFile::setUseUploadLimit(int val)
     setValue(useUploadLimitC, val);
 }
 
-void ConfigFile::setUseDownloadLimit(bool enable)
+void ConfigFile::setUseDownloadLimit(int val)
 {
-    setValue(useDownloadLimitC, enable);
+    setValue(useDownloadLimitC, val);
 }
 
 int ConfigFile::uploadLimit() const
@@ -535,6 +576,20 @@ void ConfigFile::setDownloadLimit(int kbytes)
     setValue(downloadLimitC, kbytes);
 }
 
+QPair<bool, quint64> ConfigFile::newBigFolderSizeLimit() const
+{
+    auto defaultValue = Theme::instance()->newBigFolderSizeLimit();
+    qint64 value = getValue(newBigFolderSizeLimitC, QString(), defaultValue).toLongLong();
+    bool use = value >= 0 && getValue(useNewBigFolderSizeLimitC, QString(), true).toBool();
+    return qMakePair(use, quint64(qMax<qint64>(0, value)));
+}
+
+void ConfigFile::setNewBigFolderSizeLimit(bool isChecked, quint64 mbytes)
+{
+    setValue(newBigFolderSizeLimitC, mbytes);
+    setValue(useNewBigFolderSizeLimitC, isChecked);
+}
+
 bool ConfigFile::monoIcons() const
 {
     QSettings settings(configFile(), QSettings::IniFormat);
