@@ -16,7 +16,6 @@
 #include <QNetworkProxyFactory>
 
 #include "connectionvalidator.h"
-#include "theme.h"
 #include "account.h"
 #include "networkjobs.h"
 #include "clientproxy.h"
@@ -24,6 +23,10 @@
 
 namespace OCC {
 
+// Make sure the timeout for this job is less than how often we get called
+// This makes sure we get tried often enough without "ConnectionValidator already running"
+static qint64 timeoutToUseMsec = qMax(qint64(1000), ConnectionValidator::defaultCallingIntervalMsec() - 5*1000);
+
 ConnectionValidator::ConnectionValidator(AccountPtr account, QObject *parent)
     : QObject(parent),
       _account(account),
@@ -42,7 +45,7 @@ QString ConnectionValidator::statusString( Status stat )
         return QLatin1String("NotConfigured");
     case ServerVersionMismatch:
         return QLatin1String("Server Version Mismatch");
-    case CredentialsWrong:
+    case CredentialsMissingOrWrong:
         return QLatin1String("Credentials Wrong");
     case StatusNotFound:
         return QLatin1String("Status not found");
@@ -86,7 +89,11 @@ void ConnectionValidator::systemProxyLookupDone(const QNetworkProxy &proxy) {
         return;
     }
 
-    qDebug() << Q_FUNC_INFO << "Setting QNAM proxy to be system proxy" << printQNetworkProxy(proxy);
+    if (proxy.type() != QNetworkProxy::NoProxy) {
+        qDebug() << "Setting QNAM proxy to be system proxy" << printQNetworkProxy(proxy);
+    } else {
+        qDebug() << "No system proxy set by OS";
+    }
     _account->networkAccessManager()->setProxy(proxy);
 
     slotCheckServerAndAuth();
@@ -96,6 +103,7 @@ void ConnectionValidator::systemProxyLookupDone(const QNetworkProxy &proxy) {
 void ConnectionValidator::slotCheckServerAndAuth()
 {
     CheckServerJob *checkJob = new CheckServerJob(_account, this);
+    checkJob->setTimeout(timeoutToUseMsec);
     checkJob->setIgnoreCredentialFailure(true);
     connect(checkJob, SIGNAL(instanceFound(QUrl,QVariantMap)), SLOT(slotStatusFound(QUrl,QVariantMap)));
     connect(checkJob, SIGNAL(networkError(QNetworkReply*)), SLOT(slotNoStatusFound(QNetworkReply*)));
@@ -122,18 +130,10 @@ void ConnectionValidator::slotStatusFound(const QUrl&url, const QVariantMap &inf
     }
 
     // now check the authentication
-    AbstractCredentials *creds = _account->credentials();
-    if (creds->ready()) {
+    if (_account->credentials()->ready())
         QTimer::singleShot( 0, this, SLOT( checkAuthentication() ));
-    } else {
-        // We can't proceed with the auth check because we don't have credentials.
-        // Fetch them now! Once fetched, a new connectivity check will be
-        // initiated anyway.
-        creds->fetch();
-
-        // no result is reported
-        deleteLater();
-    }
+    else
+        reportResult( CredentialsMissingOrWrong );
 }
 
 // status.php could not be loaded (network or server issue!).
@@ -143,7 +143,7 @@ void ConnectionValidator::slotNoStatusFound(QNetworkReply *reply)
     if( reply && ! _account->credentials()->stillValid(reply)) {
         _errors.append(tr("Authentication error: Either username or password are wrong."));
     }  else {
-        _errors.append(tr("Unable to connect to %1").arg(_account->url().toString()));
+        //_errors.append(tr("Unable to connect to %1").arg(_account->url().toString()));
         _errors.append( reply->errorString() );
     }
     reportResult( StatusNotFound );
@@ -151,7 +151,8 @@ void ConnectionValidator::slotNoStatusFound(QNetworkReply *reply)
 
 void ConnectionValidator::slotJobTimeout(const QUrl &url)
 {
-    _errors.append(tr("Unable to connect to %1").arg(url.toString()));
+    Q_UNUSED(url);
+    //_errors.append(tr("Unable to connect to %1").arg(url.toString()));
     _errors.append(tr("timeout"));
     reportResult( Timeout );
 }
@@ -169,6 +170,7 @@ void ConnectionValidator::checkAuthentication()
     // continue in slotAuthCheck here :-)
     qDebug() << "# Check whether authenticated propfind works.";
     PropfindJob *job = new PropfindJob(_account, "/", this);
+    job->setTimeout(timeoutToUseMsec);
     job->setProperties(QList<QByteArray>() << "getlastmodified");
     connect(job, SIGNAL(result(QVariantMap)), SLOT(slotAuthSuccess()));
     connect(job, SIGNAL(networkError(QNetworkReply*)), SLOT(slotAuthFailed(QNetworkReply*)));
@@ -184,7 +186,7 @@ void ConnectionValidator::slotAuthFailed(QNetworkReply *reply)
         qDebug() <<  reply->error() << reply->errorString();
         qDebug() << "******** Password is wrong!";
         _errors << tr("The provided credentials are not correct");
-        stat = CredentialsWrong;
+        stat = CredentialsMissingOrWrong;
 
     } else if( reply->error() != QNetworkReply::NoError ) {
         _errors << errorMessage(reply->errorString(), reply->readAll());
@@ -213,7 +215,7 @@ void ConnectionValidator::slotAuthSuccess()
 void ConnectionValidator::checkServerCapabilities()
 {
     JsonApiJob *job = new JsonApiJob(_account, QLatin1String("ocs/v1.php/cloud/capabilities"), this);
-    QObject::connect(job, SIGNAL(jsonRecieved(QVariantMap)), this, SLOT(slotCapabilitiesRecieved(QVariantMap)));
+    QObject::connect(job, SIGNAL(jsonReceived(QVariantMap, int)), this, SLOT(slotCapabilitiesRecieved(QVariantMap)));
     job->start();
 }
 
