@@ -19,7 +19,7 @@
 namespace OCC
 {
 
-class DummyCredentials : public AbstractCredentials
+class OWNCLOUDSYNC_EXPORT DummyCredentials : public AbstractCredentials
 {
     Q_OBJECT
 
@@ -27,17 +27,17 @@ public:
 
     QString _user;
     QString _password;
-    void syncContextPreInit(CSYNC* ctx) Q_DECL_OVERRIDE;
-    void syncContextPreStart(CSYNC* ctx) Q_DECL_OVERRIDE;
     bool changed(AbstractCredentials* credentials) const Q_DECL_OVERRIDE;
     QString authType() const Q_DECL_OVERRIDE;
     QString user() const Q_DECL_OVERRIDE;
     QNetworkAccessManager* getQNAM() const Q_DECL_OVERRIDE;
     bool ready() const Q_DECL_OVERRIDE;
     bool stillValid(QNetworkReply *reply) Q_DECL_OVERRIDE;
-    void fetch() Q_DECL_OVERRIDE;
+    void fetchFromKeychain() Q_DECL_OVERRIDE;
+    void askFromUser() Q_DECL_OVERRIDE;
     void persist() Q_DECL_OVERRIDE;
     void invalidateToken() Q_DECL_OVERRIDE {}
+    void forgetSensitiveData() Q_DECL_OVERRIDE {};
 };
 
 } // namespace OCC
