@@ -22,6 +22,35 @@
 
 namespace OCC {
 
+
+/* Given a sorted list of paths ending with '/', return whether or not the given path is within one of the paths of the list*/
+static bool findPathInList(const QStringList &list, const QString &path)
+{
+    Q_ASSERT(std::is_sorted(list.begin(), list.end()));
+
+    if (list.size() == 1 && list.first() == QLatin1String("/")) {
+        // Special case for the case "/" is there, it matches everything
+        return true;
+    }
+
+    QString pathSlash = path + QLatin1Char('/');
+
+    // Since the list is sorted, we can do a binary search.
+    // If the path is a prefix of another item or right after in the lexical order.
+    auto it = std::lower_bound(list.begin(), list.end(), pathSlash);
+
+    if (it != list.end() && *it == pathSlash) {
+        return true;
+    }
+
+    if (it == list.begin()) {
+        return false;
+    }
+    --it;
+    Q_ASSERT(it->endsWith(QLatin1Char('/'))); // Folder::setSelectiveSyncBlackList makes sure of that
+    return pathSlash.startsWith(*it);
+}
+
 bool DiscoveryJob::isInSelectiveSyncBlackList(const QString& path) const
 {
     if (_selectiveSyncBlackList.isEmpty()) {
@@ -29,36 +58,60 @@ bool DiscoveryJob::isInSelectiveSyncBlackList(const QString& path) const
         return false;
     }
 
-    // If one of the item in the black list is a prefix of the path, it means this path need not to
-    // be synced.
-    //
-    // We know the list is sorted (for it is done in DiscoveryJob::start)
-    // So we can do a binary search. If the path is a prefix if another item or right after in the lexical order.
+    // Block if it is in the black list
+    return findPathInList(_selectiveSyncBlackList, path);
 
-    QString pathSlash = path + QLatin1Char('/');
-
-    auto it = std::lower_bound(_selectiveSyncBlackList.begin(), _selectiveSyncBlackList.end(), pathSlash);
-
-    if (it != _selectiveSyncBlackList.end() && *it == pathSlash) {
-        return true;
-    }
-
-	if (it == _selectiveSyncBlackList.begin()) {
-        return false;
-    }
-    --it;
-    Q_ASSERT(it->endsWith(QLatin1Char('/'))); // Folder::setSelectiveSyncBlackList makes sure of that
-    if (pathSlash.startsWith(*it)) {
-        return true;
-    }
-    return false;
 }
 
-int DiscoveryJob::isInSelectiveSyncBlackListCallBack(void *data, const char *path)
+int DiscoveryJob::isInSelectiveSyncBlackListCallback(void *data, const char *path)
 {
     return static_cast<DiscoveryJob*>(data)->isInSelectiveSyncBlackList(QString::fromUtf8(path));
 }
 
+bool DiscoveryJob::checkSelectiveSyncNewFolder(const QString& path)
+{
+    // If this path or the parent is in the white list, then we do not block this file
+    if (findPathInList(_selectiveSyncWhiteList, path)) {
+        return false;
+    }
+
+    if (_newBigFolderSizeLimit < 0) {
+        // no limit, everything is allowed;
+        return false;
+    }
+
+    // Go in the main thread to do a PROPFIND to know the size of this folder
+    qint64 result = -1;
+
+    {
+        QMutexLocker locker(&_vioMutex);
+        emit doGetSizeSignal(path, &result);
+        _vioWaitCondition.wait(&_vioMutex);
+    }
+
+    auto limit = _newBigFolderSizeLimit;
+    if (result >= limit) {
+        // we tell the UI there is a new folder
+        emit newBigFolder(path);
+        return true;
+    } else {
+        // it is not too big, put it in the white list (so we will not do more query for the children)
+        // and and do not block.
+        auto p = path;
+        if (!p.endsWith(QLatin1Char('/'))) { p += QLatin1Char('/'); }
+        _selectiveSyncWhiteList.insert(std::upper_bound(_selectiveSyncWhiteList.begin(),
+                                                        _selectiveSyncWhiteList.end(), p), p);
+
+        return false;
+    }
+}
+
+int DiscoveryJob::checkSelectiveSyncNewFolderCallback(void *data, const char *path)
+{
+    return static_cast<DiscoveryJob*>(data)->checkSelectiveSyncNewFolder(QString::fromUtf8(path));
+}
+
+
 void DiscoveryJob::update_job_update_callback (bool local,
                                     const char *dirUrl,
                                     void *userdata)
@@ -107,8 +160,10 @@ int get_errno_from_http_errcode( int err, const QString & reason ) {
     case 423:           /* Locked */
         new_errno = EACCES;
         break;
-    case 400:           /* Bad Request */
     case 403:           /* Forbidden */
+        new_errno = ERRNO_FORBIDDEN;
+        break;
+    case 400:           /* Bad Request */
     case 409:           /* Conflict */
     case 411:           /* Length Required */
     case 412:           /* Precondition Failed */
@@ -154,7 +209,7 @@ int get_errno_from_http_errcode( int err, const QString & reason ) {
 
 
 
-DiscoverySingleDirectoryJob::DiscoverySingleDirectoryJob(AccountPtr account, const QString &path, QObject *parent)
+DiscoverySingleDirectoryJob::DiscoverySingleDirectoryJob(const AccountPtr &account, const QString &path, QObject *parent)
     : QObject(parent), _subPath(path), _account(account), _ignoredFirst(false)
 {
 }
@@ -184,7 +239,7 @@ void DiscoverySingleDirectoryJob::abort()
     }
 }
 
-static csync_vio_file_stat_t* propertyMapToFileStat(QMap<QString,QString> map)
+static csync_vio_file_stat_t* propertyMapToFileStat(const QMap<QString,QString> &map)
 {
     csync_vio_file_stat_t* file_stat = csync_vio_file_stat_new();
 
@@ -217,17 +272,18 @@ static csync_vio_file_stat_t* propertyMapToFileStat(QMap<QString,QString> map)
             file_stat->directDownloadCookies = strdup(value.toUtf8());
             file_stat->fields |= CSYNC_VIO_FILE_STAT_FIELDS_DIRECTDOWNLOADCOOKIES;
         } else if (property == "permissions") {
+            auto v = value.toUtf8();
             if (value.isEmpty()) {
                 // special meaning for our code: server returned permissions but are empty
                 // meaning only reading is allowed for this resource
                 file_stat->remotePerm[0] = ' ';
                 // see _csync_detect_update()
                 file_stat->fields |= CSYNC_VIO_FILE_STAT_FIELDS_PERM;
-            } else if (value.length() < int(sizeof(file_stat->remotePerm))) {
-                strncpy(file_stat->remotePerm, value.toUtf8(), sizeof(file_stat->remotePerm));
+            } else if (v.length() < int(sizeof(file_stat->remotePerm))) {
+                strcpy(file_stat->remotePerm, v.constData());
                 file_stat->fields |= CSYNC_VIO_FILE_STAT_FIELDS_PERM;
             } else {
-                // old server, keep file_stat->remotePerm empty
+                qWarning() << "permissions too large" << v;
             }
         }
     }
@@ -235,7 +291,7 @@ static csync_vio_file_stat_t* propertyMapToFileStat(QMap<QString,QString> map)
     return file_stat;
 }
 
-void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(QString file,QMap<QString,QString> map)
+void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(QString file, const QMap<QString,QString> &map)
 {
     //qDebug() << Q_FUNC_INFO << _subPath << file << map.count() << map.keys() << _account->davPath() << _lsColJob->reply()->request().url().path();
     if (!_ignoredFirst) {
@@ -246,7 +302,7 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(QString file,QMap
         }
 
     } else {
-        // Remove /remote.php/webdav/folder/ from /remote.php/webdav/folder/subfile.txt
+        // Remove <webDAV-Url>/folder/ from <webDAV-Url>/folder/subfile.txt
         file.remove(0, _lsColJob->reply()->request().url().path().length());
         // remove trailing slash
         while (file.endsWith('/')) {
@@ -263,6 +319,12 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(QString file,QMap
         if (!file_stat->etag || strlen(file_stat->etag) == 0) {
             qDebug() << "WARNING: etag of" << file_stat->name << "is" << file_stat->etag << " This must not happen.";
         }
+
+        QStringRef fileRef(&file);
+        int slashPos = file.lastIndexOf(QLatin1Char('/'));
+        if( slashPos > -1 ) {
+            fileRef = file.midRef(slashPos+1);
+        }
         //qDebug() << "!!!!" << file_stat << file_stat->name << file_stat->file_id << map.count();
         _results.append(file_stat);
     }
@@ -270,6 +332,10 @@ void DiscoverySingleDirectoryJob::directoryListingIteratedSlot(QString file,QMap
     //This works in concerto with the RequestEtagJob and the Folder object to check if the remote folder changed.
     if (map.contains("getetag")) {
        _etagConcatenation += map.value("getetag");
+
+       if (_firstEtag.isEmpty()) {
+           _firstEtag = map.value("getetag"); // for directory itself
+       }
     }
 }
 
@@ -282,6 +348,7 @@ void DiscoverySingleDirectoryJob::lsJobFinishedWithoutErrorSlot()
         deleteLater();
         return;
     }
+    emit etag(_firstEtag);
     emit etagConcatenation(_etagConcatenation);
     emit finishedWithResult(_results);
     deleteLater();
@@ -318,10 +385,13 @@ void DiscoveryMainThread::setupHooks(DiscoveryJob *discoveryJob, const QString &
     connect(discoveryJob, SIGNAL(doOpendirSignal(QString,DiscoveryDirectoryResult*)),
             this, SLOT(doOpendirSlot(QString,DiscoveryDirectoryResult*)),
             Qt::QueuedConnection);
+    connect(discoveryJob, SIGNAL(doGetSizeSignal(QString,qint64*)),
+            this, SLOT(doGetSizeSlot(QString,qint64*)),
+            Qt::QueuedConnection);
 }
 
 // Coming from owncloud_opendir -> DiscoveryJob::vio_opendir_hook -> doOpendirSignal
-void DiscoveryMainThread::doOpendirSlot(QString subPath, DiscoveryDirectoryResult *r)
+void DiscoveryMainThread::doOpendirSlot(const QString &subPath, DiscoveryDirectoryResult *r)
 {
     QString fullPath = _pathPrefix;
     if (!_pathPrefix.endsWith('/')) {
@@ -350,6 +420,8 @@ void DiscoveryMainThread::doOpendirSlot(QString subPath, DiscoveryDirectoryResul
                      this, SLOT(singleDirectoryJobFirstDirectoryPermissionsSlot(QString)));
     QObject::connect(_singleDirJob, SIGNAL(etagConcatenation(QString)),
                      this, SIGNAL(etagConcatenation(QString)));
+    QObject::connect(_singleDirJob, SIGNAL(etag(QString)),
+                     this, SIGNAL(etag(QString)));
     _singleDirJob->start();
 }
 
@@ -372,7 +444,7 @@ void DiscoveryMainThread::singleDirectoryJobResultSlot(const QList<FileStatPoint
     _discoveryJob->_vioMutex.unlock();
 }
 
-void DiscoveryMainThread::singleDirectoryJobFinishedWithErrorSlot(int csyncErrnoCode, QString msg)
+void DiscoveryMainThread::singleDirectoryJobFinishedWithErrorSlot(int csyncErrnoCode, const QString &msg)
 {
     if (!_currentDiscoveryDirectoryResult) {
         return; // possibly aborted
@@ -388,7 +460,7 @@ void DiscoveryMainThread::singleDirectoryJobFinishedWithErrorSlot(int csyncErrno
     _discoveryJob->_vioMutex.unlock();
 }
 
-void DiscoveryMainThread::singleDirectoryJobFirstDirectoryPermissionsSlot(QString p)
+void DiscoveryMainThread::singleDirectoryJobFirstDirectoryPermissionsSlot(const QString &p)
 {
     // Should be thread safe since the sync thread is blocked
     if (!_discoveryJob->_csync_ctx->remote.root_perms) {
@@ -397,6 +469,60 @@ void DiscoveryMainThread::singleDirectoryJobFirstDirectoryPermissionsSlot(QStrin
     }
 }
 
+void DiscoveryMainThread::doGetSizeSlot(const QString& path, qint64* result)
+{
+    QString fullPath = _pathPrefix;
+    if (!_pathPrefix.endsWith('/')) {
+        fullPath += '/';
+    }
+    fullPath += path;
+    // remove trailing slash
+    while (fullPath.endsWith('/')) {
+        fullPath.chop(1);
+    }
+
+    _currentGetSizeResult = result;
+
+    // Schedule the DiscoverySingleDirectoryJob
+    auto propfindJob = new PropfindJob(_account, fullPath, this);
+    propfindJob->setProperties(QList<QByteArray>() << "resourcetype" << "quota-used-bytes");
+    QObject::connect(propfindJob, SIGNAL(finishedWithError()),
+                     this, SLOT(slotGetSizeFinishedWithError()));
+    QObject::connect(propfindJob, SIGNAL(result(QVariantMap)),
+                     this, SLOT(slotGetSizeResult(QVariantMap)));
+    propfindJob->start();
+}
+
+void DiscoveryMainThread::slotGetSizeFinishedWithError()
+{
+    if (! _currentGetSizeResult) {
+        return; // possibly aborted
+    }
+
+    qWarning() << "Error getting the size of the directory";
+    // just let let the discovery job continue then
+    _currentGetSizeResult = 0;
+    QMutexLocker locker(&_discoveryJob->_vioMutex);
+    _discoveryJob->_vioWaitCondition.wakeAll();
+
+}
+
+void DiscoveryMainThread::slotGetSizeResult(const QVariantMap &map)
+{
+    if (! _currentGetSizeResult) {
+        return; // possibly aborted
+    }
+
+    *_currentGetSizeResult = map.value(QLatin1String("quota-used-bytes")).toLongLong();
+    qDebug() << "Size of folder:" << *_currentGetSizeResult;
+    _currentGetSizeResult = 0;
+    QMutexLocker locker(&_discoveryJob->_vioMutex);
+    _discoveryJob->_vioWaitCondition.wakeAll();
+}
+
+
+
+
 // called from SyncEngine
 void DiscoveryMainThread::abort() {
     if (_singleDirJob) {
@@ -414,8 +540,11 @@ void DiscoveryMainThread::abort() {
             _discoveryJob->_vioMutex.unlock();
         }
     }
-
-
+    if (_currentGetSizeResult) {
+        _currentGetSizeResult = 0;
+        QMutexLocker locker(&_discoveryJob->_vioMutex);
+        _discoveryJob->_vioWaitCondition.wakeAll();
+    }
 }
 
 csync_vio_handle_t* DiscoveryJob::remote_vio_opendir_hook (const char *url,
@@ -423,29 +552,29 @@ csync_vio_handle_t* DiscoveryJob::remote_vio_opendir_hook (const char *url,
 {
     DiscoveryJob *discoveryJob = static_cast<DiscoveryJob*>(userdata);
     if (discoveryJob) {
-        qDebug() << Q_FUNC_INFO << discoveryJob << url << "Calling into main thread...";
+        qDebug() << discoveryJob << url << "Calling into main thread...";
 
-        DiscoveryDirectoryResult *directoryResult = new DiscoveryDirectoryResult();
+        QScopedPointer<DiscoveryDirectoryResult> directoryResult(new DiscoveryDirectoryResult());
         directoryResult->code = EIO;
 
         discoveryJob->_vioMutex.lock();
         const QString qurl = QString::fromUtf8(url);
-        emit discoveryJob->doOpendirSignal(qurl, directoryResult);
+        emit discoveryJob->doOpendirSignal(qurl, directoryResult.data());
         discoveryJob->_vioWaitCondition.wait(&discoveryJob->_vioMutex, ULONG_MAX); // FIXME timeout?
         discoveryJob->_vioMutex.unlock();
 
-        qDebug() << Q_FUNC_INFO << discoveryJob << url << "...Returned from main thread";
+        qDebug() << discoveryJob << url << "...Returned from main thread";
 
         // Upon awakening from the _vioWaitCondition, iterator should be a valid iterator.
         if (directoryResult->code != 0) {
-            qDebug() << Q_FUNC_INFO << directoryResult->code << "when opening" << url << "msg=" << directoryResult->msg;
+            qDebug() << directoryResult->code << "when opening" << url << "msg=" << directoryResult->msg;
             errno = directoryResult->code;
             // save the error string to the context
             discoveryJob->_csync_ctx->error_string = qstrdup( directoryResult->msg.toUtf8().constData() );
             return NULL;
         }
 
-        return (csync_vio_handle_t*) directoryResult;
+        return directoryResult.take();
     }
     return NULL;
 }
@@ -479,11 +608,11 @@ void DiscoveryJob::remote_vio_closedir_hook (csync_vio_handle_t *dhandle,  void
 
 void DiscoveryJob::start() {
     _selectiveSyncBlackList.sort();
-    _csync_ctx->checkSelectiveSyncBlackListHook = isInSelectiveSyncBlackListCallBack;
-    _csync_ctx->checkSelectiveSyncBlackListData = this;
-
-    _csync_ctx->callbacks.update_callback = update_job_update_callback;
+    _selectiveSyncWhiteList.sort();
     _csync_ctx->callbacks.update_callback_userdata = this;
+    _csync_ctx->callbacks.update_callback = update_job_update_callback;
+    _csync_ctx->callbacks.checkSelectiveSyncBlackListHook = isInSelectiveSyncBlackListCallback;
+    _csync_ctx->callbacks.checkSelectiveSyncNewFolderHook = checkSelectiveSyncNewFolderCallback;
 
     _csync_ctx->callbacks.remote_opendir_hook = remote_vio_opendir_hook;
     _csync_ctx->callbacks.remote_readdir_hook = remote_vio_readdir_hook;
@@ -496,9 +625,8 @@ void DiscoveryJob::start() {
     _lastUpdateProgressCallbackCall.invalidate();
     int ret = csync_update(_csync_ctx);
 
-    _csync_ctx->checkSelectiveSyncBlackListHook = 0;
-    _csync_ctx->checkSelectiveSyncBlackListData = 0;
-
+    _csync_ctx->callbacks.checkSelectiveSyncNewFolderHook = 0;
+    _csync_ctx->callbacks.checkSelectiveSyncBlackListHook = 0;
     _csync_ctx->callbacks.update_callback = 0;
     _csync_ctx->callbacks.update_callback_userdata = 0;
 
