@@ -32,6 +32,7 @@
 #include <windef.h>
 #include <winbase.h>
 #include <fcntl.h>
+
 #endif
 
 // We use some internals of csync:
@@ -41,10 +42,21 @@ extern "C" void csync_win32_set_file_hidden( const char *file, bool h );
 extern "C" {
 #include "csync.h"
 #include "vio/csync_vio_local.h"
+#include "std/c_path.h"
 }
 
 namespace OCC {
 
+QString FileSystem::longWinPath( const QString& inpath )
+{
+    QString path(inpath);
+
+#ifdef Q_OS_WIN
+    path = QString::fromWCharArray( static_cast<wchar_t*>( c_utf8_path_to_locale(inpath.toUtf8() ) ) );
+#endif
+    return path;
+}
+
 bool FileSystem::fileEquals(const QString& fn1, const QString& fn2)
 {
     // compare two files with given filename and return true if they have the same content
@@ -65,7 +77,7 @@ bool FileSystem::fileEquals(const QString& fn1, const QString& fn2)
     do {
         int r = f1.read(buffer1, BufferSize);
         if (f2.read(buffer2, BufferSize) != r) {
-            // this should normaly not happen: the file are supposed to have the same size.
+            // this should normally not happen: the files are supposed to have the same size.
             return false;
         }
         if (r <= 0) {
@@ -80,7 +92,68 @@ bool FileSystem::fileEquals(const QString& fn1, const QString& fn2)
 
 void FileSystem::setFileHidden(const QString& filename, bool hidden)
 {
-    return csync_win32_set_file_hidden(filename.toUtf8().constData(), hidden);
+#ifdef _WIN32
+    QString fName = longWinPath(filename);
+    DWORD dwAttrs;
+
+    dwAttrs = GetFileAttributesW( (wchar_t*)fName.utf16() );
+
+    if (dwAttrs != INVALID_FILE_ATTRIBUTES) {
+        if (hidden && !(dwAttrs & FILE_ATTRIBUTE_HIDDEN)) {
+            SetFileAttributesW((wchar_t*)fName.utf16(), dwAttrs | FILE_ATTRIBUTE_HIDDEN );
+        } else if (!hidden && (dwAttrs & FILE_ATTRIBUTE_HIDDEN)) {
+            SetFileAttributesW((wchar_t*)fName.utf16(), dwAttrs & ~FILE_ATTRIBUTE_HIDDEN );
+        }
+    }
+#else
+    Q_UNUSED(filename);
+    Q_UNUSED(hidden);
+#endif
+}
+
+static QFile::Permissions getDefaultWritePermissions()
+{
+    QFile::Permissions result = QFile::WriteUser;
+#ifndef Q_OS_WIN
+    mode_t mask = umask(0);
+    umask(mask);
+    if (!(mask & S_IWGRP)) {
+        result |= QFile::WriteGroup;
+    }
+    if (!(mask & S_IWOTH)) {
+        result |= QFile::WriteOther;
+    }
+#endif
+    return result;
+}
+
+void FileSystem::setFileReadOnly(const QString& filename, bool readonly)
+{
+    QFile file(filename);
+    QFile::Permissions permissions = file.permissions();
+
+    QFile::Permissions allWritePermissions =
+            QFile::WriteUser | QFile::WriteGroup | QFile::WriteOther | QFile::WriteOwner;
+    static QFile::Permissions defaultWritePermissions = getDefaultWritePermissions();
+
+    permissions &= ~allWritePermissions;
+    if (!readonly) {
+        permissions |= defaultWritePermissions;
+    }
+    file.setPermissions(permissions);
+}
+
+
+void FileSystem::setFileReadOnlyWeak(const QString& filename, bool readonly)
+{
+    QFile file(filename);
+    QFile::Permissions permissions = file.permissions();
+
+    if (!readonly && (permissions & QFile::WriteOwner)) {
+        return; // already writable enough
+    }
+
+    setFileReadOnly(filename, readonly);
 }
 
 time_t FileSystem::getModTime(const QString &filename)
@@ -127,9 +200,12 @@ bool FileSystem::rename(const QString &originFileName,
     bool success = false;
     QString error;
 #ifdef Q_OS_WIN
+    QString orig = longWinPath(originFileName);
+    QString dest = longWinPath(destinationFileName);
+
     if (isLnkFile(originFileName) || isLnkFile(destinationFileName)) {
-        success = MoveFileEx((wchar_t*)originFileName.utf16(),
-                             (wchar_t*)destinationFileName.utf16(),
+        success = MoveFileEx((wchar_t*)orig.utf16(),
+                             (wchar_t*)dest.utf16(),
                              MOVEFILE_COPY_ALLOWED | MOVEFILE_WRITE_THROUGH);
         if (!success) {
             wchar_t *string = 0;
@@ -213,8 +289,8 @@ bool FileSystem::uncheckedRenameReplace(const QString& originFileName,
     success = orig.fileEngine()->rename(destinationFileName);
     // qDebug() << "Renaming " << tmpFile.fileName() << " to " << fn;
 #else
-    // We want a rename that also overwite.  QFile::rename does not overwite.
-    // Qt 5.1 has QSaveFile::renameOverwrite we cold use.
+    // We want a rename that also overwites.  QFile::rename does not overwite.
+    // Qt 5.1 has QSaveFile::renameOverwrite we could use.
     // ### FIXME
     success = true;
     bool destExists = fileExists(destinationFileName);
@@ -234,9 +310,17 @@ bool FileSystem::uncheckedRenameReplace(const QString& originFileName,
     }
 
 #else //Q_OS_WIN
+    // You can not overwrite a read-only file on windows.
+    if (!QFileInfo(destinationFileName).isWritable()) {
+        setFileReadOnly(destinationFileName, false);
+    }
+
     BOOL ok;
-    ok = MoveFileEx((wchar_t*)originFileName.utf16(),
-                    (wchar_t*)destinationFileName.utf16(),
+    QString orig = longWinPath(originFileName);
+    QString dest = longWinPath(destinationFileName);
+
+    ok = MoveFileEx((wchar_t*)orig.utf16(),
+                    (wchar_t*)dest.utf16(),
                     MOVEFILE_REPLACE_EXISTING+MOVEFILE_COPY_ALLOWED+MOVEFILE_WRITE_THROUGH);
     if (!ok) {
         wchar_t *string = 0;
@@ -274,8 +358,10 @@ bool FileSystem::openAndSeekFileSharedRead(QFile* file, QString* errorOrNull, qi
 
     // Create the file handle.
     SECURITY_ATTRIBUTES securityAtts = { sizeof(SECURITY_ATTRIBUTES), NULL, FALSE };
+    QString fName = longWinPath(file->fileName());
+
     HANDLE fileHandle = CreateFileW(
-            (const wchar_t*)file->fileName().utf16(),
+            (const wchar_t*)fName.utf16(),
             accessRights,
             shareMode,
             &securityAtts,
@@ -333,7 +419,7 @@ static qint64 getSizeWithCsync(const QString& filename)
             && (stat->fields & CSYNC_VIO_FILE_STAT_FIELDS_SIZE)) {
         result = stat->size;
     } else {
-        qDebug() << "Could not get size time for" << filename << "with csync";
+        qDebug() << "Could not get size for" << filename << "with csync";
     }
     csync_vio_file_stat_destroy(stat);
     return result;
@@ -356,7 +442,9 @@ static bool fileExistsWin(const QString& filename)
 {
     WIN32_FIND_DATA FindFileData;
     HANDLE hFind;
-    hFind = FindFirstFileW( (wchar_t*)filename.utf16(), &FindFileData);
+    QString fName = FileSystem::longWinPath(filename);
+
+    hFind = FindFirstFileW( (wchar_t*)fName.utf16(), &FindFileData);
     if (hFind == INVALID_HANDLE_VALUE) {
         return false;
     }
@@ -365,7 +453,7 @@ static bool fileExistsWin(const QString& filename)
 }
 #endif
 
-bool FileSystem::fileExists(const QString& filename)
+bool FileSystem::fileExists(const QString& filename, const QFileInfo& fileInfo)
 {
 #ifdef Q_OS_WIN
     if (isLnkFile(filename)) {
@@ -373,17 +461,25 @@ bool FileSystem::fileExists(const QString& filename)
         return fileExistsWin(filename);
     }
 #endif
-    QFileInfo file(filename);
-    return file.exists();
+    bool re = fileInfo.exists();
+    // if the filename is different from the filename in fileInfo, the fileInfo is
+    // not valid. There needs to be one initialised here. Otherwise the incoming
+    // fileInfo is re-used.
+    if( fileInfo.filePath() != filename ) {
+        QFileInfo myFI(filename);
+        re = myFI.exists();
+    }
+    return re;
 }
 
 #ifdef Q_OS_WIN
 QString FileSystem::fileSystemForPath(const QString & path)
 {
     // See also QStorageInfo (Qt >=5.4) and GetVolumeInformationByHandleW (>= Vista)
-    QString drive = path.left(3);
-    if (! drive.endsWith(":\\"))
+    QString drive = path.left(2);
+    if (! drive.endsWith(":"))
         return QString();
+    drive.append('\\');
 
     const size_t fileSystemBufferSize = 4096;
     TCHAR fileSystemBuffer[fileSystemBufferSize];
@@ -453,4 +549,44 @@ QByteArray FileSystem::calcAdler32( const QString& filename )
 }
 #endif
 
+QString FileSystem::makeConflictFileName(const QString &fn, const QDateTime &dt)
+{
+    QString conflictFileName(fn);
+    // Add _conflict-XXXX  before the extension.
+    int dotLocation = conflictFileName.lastIndexOf('.');
+    // If no extension, add it at the end  (take care of cases like foo/.hidden or foo.bar/file)
+    if (dotLocation <= conflictFileName.lastIndexOf('/') + 1) {
+        dotLocation = conflictFileName.size();
+    }
+    QString timeString = dt.toString("yyyyMMdd-hhmmss");
+
+    // Additional marker
+    QByteArray conflictFileUserName = qgetenv("CSYNC_CONFLICT_FILE_USERNAME");
+    if (conflictFileUserName.isEmpty())
+        conflictFileName.insert(dotLocation, "_conflict-" + timeString);
+    else
+        conflictFileName.insert(dotLocation, "_conflict_" + QString::fromUtf8(conflictFileUserName)  + "-" + timeString);
+
+    return conflictFileName;
+}
+
+bool FileSystem::remove(const QString &fileName, QString *errorString)
+{
+#ifdef Q_OS_WIN
+    // You cannot delete a read-only file on windows, but we want to
+    // allow that.
+    if (!QFileInfo(fileName).isWritable()) {
+        setFileReadOnly(fileName, false);
+    }
+#endif
+    QFile f(fileName);
+    if (!f.remove()) {
+        if (errorString) {
+            *errorString = f.errorString();
+        }
+        return false;
+    }
+    return true;
+}
+
 } // namespace OCC
