@@ -18,6 +18,7 @@
 #include <QNetworkReply>
 #include <QNetworkRequest>
 #include <QSslConfiguration>
+#include <QSslCipher>
 #include <QBuffer>
 #include <QXmlStreamReader>
 #include <QStringList>
@@ -33,215 +34,10 @@
 #include "account.h"
 #include "owncloudpropagator.h"
 
-#include "creds/credentialsfactory.h"
 #include "creds/abstractcredentials.h"
 
-Q_DECLARE_METATYPE(QTimer*)
-
 namespace OCC {
 
-
-AbstractNetworkJob::AbstractNetworkJob(AccountPtr account, const QString &path, QObject *parent)
-    : QObject(parent)
-    , _duration(0)
-    , _timedout(false)
-    , _followRedirects(false)
-    , _ignoreCredentialFailure(false)
-    , _reply(0)
-    , _account(account)
-    , _path(path)
-    , _redirectCount(0)
-{
-    _timer.setSingleShot(true);
-    _timer.setInterval(OwncloudPropagator::httpTimeout() * 1000); // default to 5 minutes.
-    connect(&_timer, SIGNAL(timeout()), this, SLOT(slotTimeout()));
-
-    connect(this, SIGNAL(networkActivity()), SLOT(resetTimeout()));
-
-    // Network activity on the propagator jobs (GET/PUT) keeps all requests alive.
-    // This is a workaround for OC instances which only support one
-    // parallel up and download
-    if (_account) {
-        connect(_account.data(), SIGNAL(propagatorNetworkActivity()), SLOT(resetTimeout()));
-    }
-}
-
-void AbstractNetworkJob::setReply(QNetworkReply *reply)
-{
-    if (reply)
-        reply->setProperty("doNotHandleAuth", true);
-
-    QNetworkReply *old = _reply;
-    _reply = reply;
-    delete old;
-}
-
-void AbstractNetworkJob::setTimeout(qint64 msec)
-{
-    qDebug() << Q_FUNC_INFO << msec;
-
-    _timer.start(msec);
-}
-
-void AbstractNetworkJob::resetTimeout()
-{
-    qint64 interval = _timer.interval();
-    _timer.stop();
-    _timer.start(interval);
-}
-
-void AbstractNetworkJob::setIgnoreCredentialFailure(bool ignore)
-{
-    _ignoreCredentialFailure = ignore;
-}
-
-void AbstractNetworkJob::setPath(const QString &path)
-{
-    _path = path;
-}
-
-void AbstractNetworkJob::setupConnections(QNetworkReply *reply)
-{
-    connect(reply, SIGNAL(finished()), SLOT(slotFinished()));
-#if QT_VERSION >= QT_VERSION_CHECK(5, 1, 0)
-    connect(reply, SIGNAL(encrypted()), SIGNAL(networkActivity()));
-#endif
-    connect(reply->manager(), SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)), SIGNAL(networkActivity()));
-    connect(reply, SIGNAL(sslErrors(QList<QSslError>)), SIGNAL(networkActivity()));
-    connect(reply, SIGNAL(metaDataChanged()), SIGNAL(networkActivity()));
-    connect(reply, SIGNAL(downloadProgress(qint64,qint64)), SIGNAL(networkActivity()));
-    connect(reply, SIGNAL(uploadProgress(qint64,qint64)), SIGNAL(networkActivity()));
-}
-
-QNetworkReply* AbstractNetworkJob::addTimer(QNetworkReply *reply)
-{
-    reply->setProperty("timer", QVariant::fromValue(&_timer));
-    return reply;
-}
-
-QNetworkReply* AbstractNetworkJob::davRequest(const QByteArray &verb, const QString &relPath,
-                                              QNetworkRequest req, QIODevice *data)
-{
-    return addTimer(_account->davRequest(verb, relPath, req, data));
-}
-
-QNetworkReply *AbstractNetworkJob::davRequest(const QByteArray &verb, const QUrl &url, QNetworkRequest req, QIODevice *data)
-{
-    return addTimer(_account->davRequest(verb, url, req, data));
-}
-
-QNetworkReply* AbstractNetworkJob::getRequest(const QString &relPath)
-{
-    return addTimer(_account->getRequest(relPath));
-}
-
-QNetworkReply *AbstractNetworkJob::getRequest(const QUrl &url)
-{
-    return addTimer(_account->getRequest(url));
-}
-
-QNetworkReply *AbstractNetworkJob::headRequest(const QString &relPath)
-{
-    return addTimer(_account->headRequest(relPath));
-}
-
-QNetworkReply *AbstractNetworkJob::headRequest(const QUrl &url)
-{
-    return addTimer(_account->headRequest(url));
-}
-
-void AbstractNetworkJob::slotFinished()
-{
-    _timer.stop();
-
-    if( _reply->error() == QNetworkReply::SslHandshakeFailedError ) {
-        qDebug() << "SslHandshakeFailedError: " << reply()->errorString() << " : can be caused by a webserver wanting SSL client certificates";
-    }
-    
-    if( _reply->error() != QNetworkReply::NoError ) {
-        qDebug() << Q_FUNC_INFO << _reply->error() << _reply->errorString();
-        if (_reply->error() == QNetworkReply::ProxyAuthenticationRequiredError) {
-            qDebug() << Q_FUNC_INFO << _reply->rawHeader("Proxy-Authenticate");
-        }
-        emit networkError(_reply);
-    }
-
-    // get the Date timestamp from reply
-    _responseTimestamp = _reply->rawHeader("Date");
-    _duration = _durationTimer.elapsed();
-
-    if (_followRedirects) {
-        // ### the qWarnings here should be exported via displayErrors() so they
-        // ### can be presented to the user if the job executor has a GUI
-        QUrl requestedUrl = reply()->request().url();
-        QUrl redirectUrl = reply()->attribute(QNetworkRequest::RedirectionTargetAttribute).toUrl();
-        if (!redirectUrl.isEmpty()) {
-            _redirectCount++;
-            if (requestedUrl.scheme() == QLatin1String("https") &&
-                    redirectUrl.scheme() == QLatin1String("http")) {
-                qWarning() << this << "HTTPS->HTTP downgrade detected!";
-            } else if (requestedUrl == redirectUrl || _redirectCount >= maxRedirects()) {
-                qWarning() << this << "Redirect loop detected!";
-            } else {
-                resetTimeout();
-                setReply(getRequest(redirectUrl));
-                setupConnections(reply());
-                return;
-            }
-        }
-    }
-
-    AbstractCredentials *creds = _account->credentials();
-    if (!creds->stillValid(_reply) && ! _ignoreCredentialFailure) {
-        _account->handleInvalidCredentials();
-    }
-
-    bool discard = finished();
-    if (discard) {
-        deleteLater();
-    }
-}
-
-quint64 AbstractNetworkJob::duration()
-{
-    return _duration;
-}
-
-QByteArray AbstractNetworkJob::responseTimestamp()
-{
-    return _responseTimestamp;
-}
-
-AbstractNetworkJob::~AbstractNetworkJob()
-{
-    setReply(0);
-}
-
-void AbstractNetworkJob::start()
-{
-    _timer.start();
-    _durationTimer.start();
-    _duration = 0;
-
-    const QUrl url = account()->url();
-    const QString displayUrl = QString( "%1://%2%3").arg(url.scheme()).arg(url.host()).arg(url.path());
-
-    qDebug() << "!!!" << metaObject()->className() << "created for" << displayUrl << "+" << path();
-}
-
-void AbstractNetworkJob::slotTimeout()
-{
-    _timedout = true;
-    qDebug() <<  this << "Timeout";
-    if (reply()) {
-        reply()->abort();
-    } else {
-        qDebug() << "reply was NULL";
-    }
-}
-
-/*********************************************************************************************/
-
 RequestEtagJob::RequestEtagJob(AccountPtr account, const QString &path, QObject *parent)
     : AbstractNetworkJob(account, path, parent)
 {
@@ -250,11 +46,16 @@ RequestEtagJob::RequestEtagJob(AccountPtr account, const QString &path, QObject
 void RequestEtagJob::start()
 {
     QNetworkRequest req;
-    // Let's always request all entries inside a directory. There are/were bugs in the server
-    // where a root or root-folder ETag is not updated when its contents change. We work around
-    // this by concatenating the ETags of the root and its contents.
-    req.setRawHeader("Depth", "1");
-    // See https://github.com/owncloud/core/issues/5255 and others
+    if (_account && _account->rootEtagChangesNotOnlySubFolderEtags()) {
+        // Fixed from 8.1 https://github.com/owncloud/client/issues/3730
+        req.setRawHeader("Depth", "0");
+    } else {
+        // Let's always request all entries inside a directory. There are/were bugs in the server
+        // where a root or root-folder ETag is not updated when its contents change. We work around
+        // this by concatenating the ETags of the root and its contents.
+        req.setRawHeader("Depth", "1");
+        // See https://github.com/owncloud/core/issues/5255 and others
+    }
 
     QByteArray xml("<?xml version=\"1.0\" ?>\n"
                    "<d:propfind xmlns:d=\"DAV:\">\n"
@@ -307,8 +108,12 @@ MkColJob::MkColJob(AccountPtr account, const QString &path, QObject *parent)
 
 void MkColJob::start()
 {
-    // assumes ownership
-   QNetworkReply *reply = davRequest("MKCOL", path());
+   // add 'Content-Length: 0' header (see https://github.com/owncloud/client/issues/3256)
+   QNetworkRequest req;
+   req.setRawHeader("Content-Length", "0");
+
+   // assumes ownership
+   QNetworkReply *reply = davRequest("MKCOL", path(), req);
    setReply(reply);
    setupConnections(reply);
    AbstractNetworkJob::start();
@@ -350,7 +155,7 @@ LsColXMLParser::LsColXMLParser()
 
 }
 
-bool LsColXMLParser::parse( const QByteArray& xml, QHash<QString, qint64> *sizes)
+bool LsColXMLParser::parse( const QByteArray& xml, QHash<QString, qint64> *sizes, const QString& expectedPath)
 {
     // Parse DAV response
     QXmlStreamReader reader(xml);
@@ -371,7 +176,14 @@ bool LsColXMLParser::parse( const QByteArray& xml, QHash<QString, qint64> *sizes
         // Start elements with DAV:
         if (type == QXmlStreamReader::StartElement && reader.namespaceUri() == QLatin1String("DAV:")) {
             if (name == QLatin1String("href")) {
-                currentHref = QUrl::fromPercentEncoding(reader.readElementText().toUtf8());
+                // We don't use URL encoding in our request URL (which is the expected path) (QNAM will do it for us)
+                // but the result will have URL encoding..
+                QString hrefString = QString::fromUtf8(QByteArray::fromPercentEncoding(reader.readElementText().toUtf8()));
+                if (!hrefString.startsWith(expectedPath)) {
+                    qDebug() << "Invalid href" << hrefString << "expected starting with" << expectedPath;
+                    return false;
+                }
+                currentHref = hrefString;
             } else if (name == QLatin1String("response")) {
             } else if (name == QLatin1String("propstat")) {
                 insidePropstat = true;
@@ -504,7 +316,7 @@ void LsColJob::start()
 
 // TODO: Instead of doing all in this slot, we should iteratively parse in readyRead(). This
 // would allow us to be more asynchronous in processing while data is coming from the network,
-// not in all in one big blobb at the end.
+// not all in one big blob at the end.
 bool LsColJob::finished()
 {
     QString contentType = reply()->header(QNetworkRequest::ContentTypeHeader).toString();
@@ -520,7 +332,8 @@ bool LsColJob::finished()
         connect( &parser, SIGNAL(finishedWithoutError()),
                  this, SIGNAL(finishedWithoutError()) );
 
-        if( !parser.parse( reply()->readAll(), &_sizes ) ) {
+        QString expectedPath = reply()->request().url().path(); // something like "/owncloud/remote.php/webdav/folder"
+        if( !parser.parse( reply()->readAll(), &_sizes, expectedPath ) ) {
             // XML parse error
             emit finishedWithError(reply());
         }
@@ -554,6 +367,8 @@ void CheckServerJob::start()
 {
     setReply(getRequest(path()));
     setupConnections(reply());
+    connect(reply(), SIGNAL(metaDataChanged()), this, SLOT(metaDataChangedSlot()));
+    connect(reply(), SIGNAL(encrypted()), this, SLOT(encryptedSlot()));
     AbstractNetworkJob::start();
 }
 
@@ -583,11 +398,46 @@ bool CheckServerJob::installed(const QVariantMap &info)
     return info.value(QLatin1String("installed")).toBool();
 }
 
-bool CheckServerJob::finished()
+static void mergeSslConfigurationForSslButton(const QSslConfiguration &config, AccountPtr account)
+{
+    if (config.peerCertificateChain().length() > 0) {
+        account->_peerCertificateChain = config.peerCertificateChain();
+    }
+    if (!config.sessionCipher().isNull()) {
+        account->_sessionCipher = config.sessionCipher();
+    }
+#if QT_VERSION > QT_VERSION_CHECK(5, 2, 0)
+    if (config.sessionTicket().length() > 0) {
+        account->_sessionTicket = config.sessionTicket();
+    }
+#endif
+}
+
+void CheckServerJob::encryptedSlot()
+{
+    mergeSslConfigurationForSslButton(reply()->sslConfiguration(), account());
+}
+
+void CheckServerJob::metaDataChangedSlot()
 {
     account()->setSslConfiguration(reply()->sslConfiguration());
+    mergeSslConfigurationForSslButton(reply()->sslConfiguration(), account());
+}
 
-    // The serverInstalls to /owncloud. Let's try that if the file wasn't found
+
+bool CheckServerJob::finished()
+{
+#if QT_VERSION > QT_VERSION_CHECK(5, 2, 0)
+    if (reply()->request().url().scheme() == QLatin1String("https")
+            && reply()->sslConfiguration().sessionTicket().isEmpty()
+            && reply()->error() == QNetworkReply::NoError) {
+        qDebug() << "No SSL session identifier / session ticket is used, this might impact sync performance negatively.";
+    }
+#endif
+
+    mergeSslConfigurationForSslButton(reply()->sslConfiguration(), account());
+
+    // The server installs to /owncloud. Let's try that if the file wasn't found
     // at the original location
     if ((reply()->error() == QNetworkReply::ContentNotFoundError) && (!_subdirFallback)) {
         _subdirFallback = true;
@@ -693,7 +543,7 @@ bool PropfindJob::finished()
             QXmlStreamReader::TokenType type = reader.readNext();
             if (type == QXmlStreamReader::StartElement) {
                 if (!curElement.isEmpty() && curElement.top() == QLatin1String("prop")) {
-                    items.insert(reader.name().toString(), reader.readElementText());
+                    items.insert(reader.name().toString(), reader.readElementText(QXmlStreamReader::SkipChildElements));
                 } else {
                     curElement.push(reader.name().toString());
                 }
@@ -706,7 +556,82 @@ bool PropfindJob::finished()
         }
         emit result(items);
     } else {
-        qDebug() << "Quota request *not* successful, http result code is" << http_result_code
+        qDebug() << "PROPFIND request *not* successful, http result code is" << http_result_code
+                 << (http_result_code == 302 ? reply()->header(QNetworkRequest::LocationHeader).toString()  : QLatin1String(""));
+        emit finishedWithError();
+    }
+    return true;
+}
+
+/*********************************************************************************************/
+
+ProppatchJob::ProppatchJob(AccountPtr account, const QString &path, QObject *parent)
+    : AbstractNetworkJob(account, path, parent)
+{
+
+}
+
+void ProppatchJob::start()
+{
+    if (_properties.isEmpty()) {
+        qWarning() << "Proppatch with no properties!";
+    }
+    QNetworkRequest req;
+
+    QByteArray propStr;
+    QMapIterator<QByteArray, QByteArray> it(_properties);
+    while (it.hasNext()) {
+        it.next();
+        QByteArray keyName = it.key();
+        QByteArray keyNs;
+        if (keyName.contains(':')) {
+            int colIdx = keyName.lastIndexOf(":");
+            keyNs = keyName.left(colIdx);
+            keyName = keyName.mid(colIdx+1);
+        }
+
+        propStr += "    <" + keyName;
+        if (!keyNs.isEmpty()) {
+            propStr += " xmlns=\"" + keyNs + "\" ";
+        }
+        propStr += ">";
+        propStr += it.value();
+        propStr += "</" + keyName + ">\n";
+    }
+    QByteArray xml = "<?xml version=\"1.0\" ?>\n"
+                     "<d:propertyupdate xmlns:d=\"DAV:\">\n"
+                     "  <d:set><d:prop>\n"
+                     + propStr +
+                     "  </d:prop></d:set>\n"
+                     "</d:propertyupdate>\n";
+
+    QBuffer *buf = new QBuffer(this);
+    buf->setData(xml);
+    buf->open(QIODevice::ReadOnly);
+    setReply(davRequest("PROPPATCH", path(), req, buf));
+    buf->setParent(reply());
+    setupConnections(reply());
+    AbstractNetworkJob::start();
+}
+
+void ProppatchJob::setProperties(QMap<QByteArray, QByteArray> properties)
+{
+    _properties = properties;
+}
+
+QMap<QByteArray, QByteArray> ProppatchJob::properties() const
+{
+    return _properties;
+}
+
+bool ProppatchJob::finished()
+{
+    int http_result_code = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
+    if (http_result_code == 207) {
+        emit success();
+    } else {
+        qDebug() << "PROPPATCH request *not* successful, http result code is" << http_result_code
                  << (http_result_code == 302 ? reply()->header(QNetworkRequest::LocationHeader).toString()  : QLatin1String(""));
         emit finishedWithError();
     }
@@ -735,84 +660,22 @@ bool EntityExistsJob::finished()
 
 /*********************************************************************************************/
 
-CheckQuotaJob::CheckQuotaJob(AccountPtr account, const QString &path, QObject *parent)
-    : AbstractNetworkJob(account, path, parent)
-{
-}
-
-void CheckQuotaJob::start()
-{
-    QNetworkRequest req;
-    req.setRawHeader("Depth", "0");
-    QByteArray xml("<?xml version=\"1.0\" ?>\n"
-                   "<d:propfind xmlns:d=\"DAV:\">\n"
-                   "  <d:prop>\n"
-                   "    <d:quota-available-bytes/>\n"
-                   "    <d:quota-used-bytes/>\n"
-                   "  </d:prop>\n"
-                   "</d:propfind>\n");
-    QBuffer *buf = new QBuffer(this);
-    buf->setData(xml);
-    buf->open(QIODevice::ReadOnly);
-    // assumes ownership
-    setReply(davRequest("PROPFIND", path(), req, buf));
-    buf->setParent(reply());
-    setupConnections(reply());
-    AbstractNetworkJob::start();
-}
-
-bool CheckQuotaJob::finished()
-{
-    if (reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute) == 207) {
-        // Parse DAV response
-        QXmlStreamReader reader(reply());
-        reader.addExtraNamespaceDeclaration(QXmlStreamNamespaceDeclaration("d", "DAV:"));
-        qint64 quotaAvailableBytes = 0;
-        qint64 quotaUsedBytes = 0;
-        while (!reader.atEnd()) {
-            QXmlStreamReader::TokenType type = reader.readNext();
-            if (type == QXmlStreamReader::StartElement &&
-                    reader.namespaceUri() == QLatin1String("DAV:")) {
-                QString name = reader.name().toString();
-                if (name == QLatin1String("quota-available-bytes")) {
-                    // I have seen the server returning frational bytes:
-                    //   <d:quota-available-bytes>1374532061.2</d:quota-available-bytes>
-                    quotaAvailableBytes = reader.readElementText().toDouble();
-                } else if (name == QLatin1String("quota-used-bytes")) {
-                    quotaUsedBytes = reader.readElementText().toDouble();
-                }
-            }
-        }
-        qint64 total = quotaUsedBytes + quotaAvailableBytes;
-        emit quotaRetrieved(total, quotaUsedBytes);
-    }
-    return true;
-}
-
-NetworkJobTimeoutPauser::NetworkJobTimeoutPauser(QNetworkReply *reply)
-{
-    _timer = reply->property("timer").value<QTimer*>();
-    if(!_timer.isNull()) {
-        _timer->stop();
-    }
-}
-
-NetworkJobTimeoutPauser::~NetworkJobTimeoutPauser()
-{
-    if(!_timer.isNull()) {
-        _timer->start();
-    }
-}
-
 JsonApiJob::JsonApiJob(const AccountPtr &account, const QString& path, QObject* parent): AbstractNetworkJob(account, path, parent)
 { }
 
+void JsonApiJob::addQueryParams(QList< QPair<QString,QString> > params)
+{
+    _additionalParams = params;
+}
+
 void JsonApiJob::start()
 {
     QNetworkRequest req;
     req.setRawHeader("OCS-APIREQUEST", "true");
     QUrl url = Account::concatUrlPath(account()->url(), path());
-    url.setQueryItems(QList<QPair<QString, QString> >() << qMakePair(QString::fromLatin1("format"), QString::fromLatin1("json")));
+    QList<QPair<QString, QString> > params = _additionalParams;
+    params << qMakePair(QString::fromLatin1("format"), QString::fromLatin1("json"));
+    url.setQueryItems(params);
     setReply(davRequest("GET", url, req));
     setupConnections(reply());
     AbstractNetworkJob::start();
@@ -820,27 +683,41 @@ void JsonApiJob::start()
 
 bool JsonApiJob::finished()
 {
+    int statusCode = 0;
+
     if (reply()->error() != QNetworkReply::NoError) {
         qWarning() << "Network error: " << path() << reply()->errorString() << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute);
-        emit jsonRecieved(QVariantMap());
+        emit jsonReceived(QVariantMap(), statusCode);
         return true;
     }
 
-    bool success = false;
     QString jsonStr = QString::fromUtf8(reply()->readAll());
+    if( jsonStr.contains( "<?xml version=\"1.0\"?>") ) {
+        QRegExp rex("<statuscode>(\\d+)</statuscode>");
+        if( jsonStr.contains(rex) ) {
+            // this is a error message coming back from ocs.
+            statusCode = rex.cap(1).toInt();
+        }
+
+    } else {
+        QRegExp rex("\"statuscode\":(\\d+),");
+        // example: "{"ocs":{"meta":{"status":"ok","statuscode":100,"message":null},"data":{"version":{"major":8,"minor":"... (504)
+        if( jsonStr.contains(rex) ) {
+            statusCode = rex.cap(1).toInt();
+        }
+    }
+
+    bool success = false;
     QVariantMap json = QtJson::parse(jsonStr, success).toMap();
     // empty or invalid response
     if (!success || json.isEmpty()) {
         qWarning() << "invalid JSON!" << jsonStr;
-        emit jsonRecieved(QVariantMap());
+        emit jsonReceived(QVariantMap(), statusCode);
         return true;
     }
 
-    emit jsonRecieved(json);
+    emit jsonReceived(json, statusCode);
     return true;
 }
 
-
-
-
 } // namespace OCC
