@@ -22,9 +22,6 @@
 #include "propagateremotemove.h"
 #include "propagateremotemkdir.h"
 #include "propagatorjobs.h"
-#ifdef USE_NEON
-#include "propagator_legacy.h"
-#endif
 #include "configfile.h"
 #include "utility.h"
 #include "account.h"
@@ -45,24 +42,56 @@
 
 namespace OCC {
 
+qint64 criticalFreeSpaceLimit()
+{
+    qint64 value = 50 * 1000 * 1000LL;
+
+    static bool hasEnv = false;
+    static qint64 env = qgetenv("OWNCLOUD_CRITICAL_FREE_SPACE_BYTES").toLongLong(&hasEnv);
+    if (hasEnv) {
+        value = env;
+    }
+
+    return qBound(0LL, value, freeSpaceLimit());
+}
+
+qint64 freeSpaceLimit()
+{
+    qint64 value = 250 * 1000 * 1000LL;
+
+    static bool hasEnv = false;
+    static qint64 env = qgetenv("OWNCLOUD_FREE_SPACE_BYTES").toLongLong(&hasEnv);
+    if (hasEnv) {
+        value = env;
+    }
+
+    return value;
+}
+
 OwncloudPropagator::~OwncloudPropagator()
 {}
 
-/* The maximum number of active job in parallel  */
+/* The maximum number of active jobs in parallel  */
 int OwncloudPropagator::maximumActiveJob()
 {
     static int max = qgetenv("OWNCLOUD_MAX_PARALLEL").toUInt();
     if (!max) {
         max = 3; //default
     }
+
+    if (_downloadLimit.fetchAndAddAcquire(0) != 0 || _uploadLimit.fetchAndAddAcquire(0) != 0) {
+        // disable parallelism when there is a network limit.
+        return 1;
+    }
+
     return max;
 }
 
-/** Updates or creates a blacklist entry for the given item.
+/** Updates, creates or removes a blacklist entry for the given item.
  *
  * Returns whether the file is in the blacklist now.
  */
-static bool blacklist(SyncJournalDb* journal, const SyncFileItem& item)
+static bool blacklistCheck(SyncJournalDb* journal, const SyncFileItem& item)
 {
     SyncJournalErrorBlacklistRecord oldEntry = journal->errorBlacklistEntry(item._file);
     SyncJournalErrorBlacklistRecord newEntry = SyncJournalErrorBlacklistRecord::update(oldEntry, item);
@@ -79,15 +108,15 @@ static bool blacklist(SyncJournalDb* journal, const SyncFileItem& item)
 void PropagateItemJob::done(SyncFileItem::Status status, const QString &errorString)
 {
     _state = Finished;
-    if (_item._isRestoration) {
+    if (_item->_isRestoration) {
         if( status == SyncFileItem::Success || status == SyncFileItem::Conflict) {
             status = SyncFileItem::Restoration;
         } else {
-            _item._errorString += tr("; Restoration Failed: %1").arg(errorString);
+            _item->_errorString += tr("; Restoration Failed: %1").arg(errorString);
         }
     } else {
-        if( _item._errorString.isEmpty() ) {
-            _item._errorString = errorString;
+        if( _item->_errorString.isEmpty() ) {
+            _item->_errorString = errorString;
         }
     }
 
@@ -103,20 +132,20 @@ void PropagateItemJob::done(SyncFileItem::Status status, const QString &errorStr
         // do not blacklist in case of soft error or fatal error.
         break;
     case SyncFileItem::NormalError:
-        if (blacklist(_propagator->_journal, _item) && _item._hasBlacklistEntry) {
+        if (blacklistCheck(_propagator->_journal, *_item) && _item->_hasBlacklistEntry) {
             // do not error if the item was, and continues to be, blacklisted
             status = SyncFileItem::FileIgnored;
-            _item._errorString.prepend(tr("Continue blacklisting:") + " ");
+            _item->_errorString.prepend(tr("Continue blacklisting:") + " ");
         }
         break;
     case SyncFileItem::Success:
     case SyncFileItem::Restoration:
-        if( _item._hasBlacklistEntry ) {
+        if( _item->_hasBlacklistEntry ) {
             // wipe blacklist entry.
-            _propagator->_journal->wipeErrorBlacklistEntry(_item._file);
+            _propagator->_journal->wipeErrorBlacklistEntry(_item->_file);
             // remove a blacklist entry in case the file was moved.
-            if( _item._originalFile != _item._file ) {
-                _propagator->_journal->wipeErrorBlacklistEntry(_item._originalFile);
+            if( _item->_originalFile != _item->_file ) {
+                _propagator->_journal->wipeErrorBlacklistEntry(_item->_originalFile);
             }
         }
         break;
@@ -127,9 +156,9 @@ void PropagateItemJob::done(SyncFileItem::Status status, const QString &errorStr
         break;
     }
 
-    _item._status = status;
+    _item->_status = status;
 
-    emit completed(_item);
+    emit itemCompleted(*_item, *this);
     emit finished(status);
 }
 
@@ -143,46 +172,43 @@ bool PropagateItemJob::checkForProblemsWithShared(int httpStatusCode, const QStr
 {
     PropagateItemJob *newJob = NULL;
 
-    if( httpStatusCode == 403 && _propagator->isInSharedDirectory(_item._file )) {
-        if( !_item._isDirectory ) {
-            SyncFileItem downloadItem(_item);
-            if (downloadItem._instruction == CSYNC_INSTRUCTION_NEW) {
+    if( httpStatusCode == 403 && _propagator->isInSharedDirectory(_item->_file )) {
+        if( !_item->_isDirectory ) {
+            SyncFileItemPtr downloadItem(new SyncFileItem(*_item));
+            if (downloadItem->_instruction == CSYNC_INSTRUCTION_NEW
+                    || downloadItem->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
                 // don't try to recover pushing new files
                 return false;
-            } else if (downloadItem._instruction == CSYNC_INSTRUCTION_SYNC) {
-                // we modified the file locally, jsut create a conflict then
-                downloadItem._instruction = CSYNC_INSTRUCTION_CONFLICT;
+            } else if (downloadItem->_instruction == CSYNC_INSTRUCTION_SYNC) {
+                // we modified the file locally, just create a conflict then
+                downloadItem->_instruction = CSYNC_INSTRUCTION_CONFLICT;
 
                 // HACK to avoid continuation: See task #1448:  We do not know the _modtime from the
                 //  server, at this point, so just set the current one. (rather than the one locally)
-                downloadItem._modtime = Utility::qDateTimeToTime_t(QDateTime::currentDateTime());
+                downloadItem->_modtime = Utility::qDateTimeToTime_t(QDateTime::currentDateTime());
             } else {
                 // the file was removed or renamed, just recover the old one
-                downloadItem._instruction = CSYNC_INSTRUCTION_SYNC;
+                downloadItem->_instruction = CSYNC_INSTRUCTION_SYNC;
             }
-            downloadItem._direction = SyncFileItem::Down;
-#ifdef USE_NEON
-            newJob = new PropagateDownloadFileLegacy(_propagator, downloadItem);
-#else
+            downloadItem->_direction = SyncFileItem::Down;
             newJob = new PropagateDownloadFileQNAM(_propagator, downloadItem);
-#endif
         } else {
             // Directories are harder to recover.
             // But just re-create the directory, next sync will be able to recover the files
-            SyncFileItem mkdirItem(_item);
-            mkdirItem._instruction = CSYNC_INSTRUCTION_SYNC;
-            mkdirItem._direction = SyncFileItem::Down;
+            SyncFileItemPtr mkdirItem(new SyncFileItem(*_item));
+            mkdirItem->_instruction = CSYNC_INSTRUCTION_SYNC;
+            mkdirItem->_direction = SyncFileItem::Down;
             newJob = new PropagateLocalMkdir(_propagator, mkdirItem);
             // Also remove the inodes and fileid from the db so no further renames are tried for
             // this item.
-            _propagator->_journal->avoidRenamesOnNextSync(_item._file);
+            _propagator->_journal->avoidRenamesOnNextSync(_item->_file);
             _propagator->_anotherSyncNeeded = true;
         }
         if( newJob )  {
             newJob->setRestoreJobMsg(msg);
             _restoreJob.reset(newJob);
-            connect(_restoreJob.data(), SIGNAL(completed(SyncFileItem)),
-                    this, SLOT(slotRestoreJobCompleted(SyncFileItem)));
+            connect(_restoreJob.data(), SIGNAL(itemCompleted(const SyncFileItemPtr &, const PropagatorJob &)),
+                    this, SLOT(slotRestoreJobCompleted(const SyncFileItemPtr &)));
             QMetaObject::invokeMethod(newJob, "start");
         }
         return true;
@@ -202,46 +228,50 @@ void PropagateItemJob::slotRestoreJobCompleted(const SyncFileItem& item )
             || item._status == SyncFileItem::Restoration) {
         done( SyncFileItem::SoftError, msg);
     } else {
-        done( item._status, tr("A file or directory was removed from a read only share, but restoring failed: %1").arg(item._errorString) );
+        done( item._status, tr("A file or folder was removed from a read only share, but restoring failed: %1").arg(item._errorString) );
     }
 }
 
 // ================================================================================
 
-PropagateItemJob* OwncloudPropagator::createJob(const SyncFileItem& item) {
-    switch(item._instruction) {
+PropagateItemJob* OwncloudPropagator::createJob(const SyncFileItemPtr &item) {
+    bool deleteExisting = item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE;
+    switch(item->_instruction) {
         case CSYNC_INSTRUCTION_REMOVE:
-            if (item._direction == SyncFileItem::Down) return new PropagateLocalRemove(this, item);
+            if (item->_direction == SyncFileItem::Down) return new PropagateLocalRemove(this, item);
             else return new PropagateRemoteDelete(this, item);
         case CSYNC_INSTRUCTION_NEW:
-            if (item._isDirectory) {
-                if (item._direction == SyncFileItem::Down) return new PropagateLocalMkdir(this, item);
-                else return new PropagateRemoteMkdir(this, item);
-            }   //fall trough
+        case CSYNC_INSTRUCTION_TYPE_CHANGE:
+            if (item->_isDirectory) {
+                if (item->_direction == SyncFileItem::Down) {
+                    auto job = new PropagateLocalMkdir(this, item);
+                    job->setDeleteExistingFile(deleteExisting);
+                    return job;
+                } else {
+                    auto job = new PropagateRemoteMkdir(this, item);
+                    job->setDeleteExisting(deleteExisting);
+                    return job;
+                }
+            }   //fall through
         case CSYNC_INSTRUCTION_SYNC:
         case CSYNC_INSTRUCTION_CONFLICT:
-            if (item._isDirectory) {
+            if (item->_isDirectory) {
                 // Should we set the mtime?
                 return 0;
             }
-#ifdef USE_NEON
-            if (useLegacyJobs()) {
-                if (item._direction != SyncFileItem::Up) {
-                    return new PropagateDownloadFileLegacy(this, item);
-                } else {
-                    return new PropagateUploadFileLegacy(this, item);
-                }
-            } else
-#endif
             {
-                if (item._direction != SyncFileItem::Up) {
-                    return new PropagateDownloadFileQNAM(this, item);
+                if (item->_direction != SyncFileItem::Up) {
+                    auto job = new PropagateDownloadFileQNAM(this, item);
+                    job->setDeleteExistingFolder(deleteExisting);
+                    return job;
                 } else {
-                    return new PropagateUploadFileQNAM(this, item);
+                    auto job = new PropagateUploadFileQNAM(this, item);
+                    job->setDeleteExisting(deleteExisting);
+                    return job;
                 }
             }
         case CSYNC_INSTRUCTION_RENAME:
-            if (item._direction == SyncFileItem::Up) {
+            if (item->_direction == SyncFileItem::Up) {
                 return new PropagateRemoteMove(this, item);
             } else {
                 return new PropagateLocalRename(this, item);
@@ -261,86 +291,112 @@ void OwncloudPropagator::start(const SyncFileItemVector& items)
 
     /* Check and log the transmission checksum type */
     ConfigFile cfg;
-    const QString checksumType = cfg.transmissionChecksum().toUpper();
+    const QString checksumType = cfg.transmissionChecksum();
 
-    /* if the checksum type is empty, it is not send. No error */
+    /* if the checksum type is empty, it is not sent. No error */
     if( !checksumType.isEmpty() ) {
-        if( checksumType == checkSumAdlerUpperC ||
+        if( checksumType == checkSumAdlerC ||
                 checksumType == checkSumMD5C    ||
                 checksumType == checkSumSHA1C ) {
-            qDebug() << "Client sends and expects transmission checksum type" << checksumType;
+            qDebug() << "Client sends transmission checksum type" << checksumType;
         } else {
             qWarning() << "Unknown transmission checksum type from config" << checksumType;
         }
     }
 
-    /* This builds all the job needed for the propagation.
-     * Each directories is a PropagateDirectory job, which contains the files in it.
+    /* This builds all the jobs needed for the propagation.
+     * Each directory is a PropagateDirectory job, which contains the files in it.
      * In order to do that we loop over the items. (which are sorted by destination)
-     * When we enter adirectory, we can create the directory job and push it on the stack. */
+     * When we enter a directory, we can create the directory job and push it on the stack. */
 
     _rootJob.reset(new PropagateDirectory(this));
     QStack<QPair<QString /* directory name */, PropagateDirectory* /* job */> > directories;
     directories.push(qMakePair(QString(), _rootJob.data()));
     QVector<PropagatorJob*> directoriesToRemove;
     QString removedDirectory;
-    foreach(const SyncFileItem &item, items) {
+    foreach(const SyncFileItemPtr &item, items) {
 
-        if (!removedDirectory.isEmpty() && item._file.startsWith(removedDirectory)) {
+        if (!removedDirectory.isEmpty() && item->_file.startsWith(removedDirectory)) {
             // this is an item in a directory which is going to be removed.
-            PropagateDirectory *delDirJob = dynamic_cast<PropagateDirectory*>(directoriesToRemove.last());
+            PropagateDirectory *delDirJob = dynamic_cast<PropagateDirectory*>(directoriesToRemove.first());
 
-            if (item._instruction == CSYNC_INSTRUCTION_REMOVE) {
-                //already taken care of.  (by the removal of the parent directory)
+            if (item->_instruction == CSYNC_INSTRUCTION_REMOVE) {
+                // already taken care of. (by the removal of the parent directory)
 
                 // increase the number of subjobs that would be there.
                 if( delDirJob ) {
                     delDirJob->increaseAffectedCount();
                 }
                 continue;
-            } else if (item._instruction == CSYNC_INSTRUCTION_NEW && item._isDirectory) {
+            } else if (item->_isDirectory
+                       && (item->_instruction == CSYNC_INSTRUCTION_NEW
+                           || item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE)) {
                 // create a new directory within a deleted directory? That can happen if the directory
-                // etag were not fetched properly on the previous sync because the sync was aborted
+                // etag was not fetched properly on the previous sync because the sync was aborted
                 // while uploading this directory (which is now removed).  We can ignore it.
                 if( delDirJob ) {
                     delDirJob->increaseAffectedCount();
                 }
                 continue;
-            } else if (item._instruction == CSYNC_INSTRUCTION_IGNORE) {
+            } else if (item->_instruction == CSYNC_INSTRUCTION_IGNORE) {
                 continue;
+            } else if (item->_instruction == CSYNC_INSTRUCTION_RENAME) {
+                // all is good, the rename will be executed before the directory deletion
+            } else {
+                qWarning() << "WARNING:  Job within a removed directory?  This should not happen!"
+                           << item->_file << item->_instruction;
             }
-
-            qWarning() << "WARNING:  Job within a removed directory?  This should not happen!"
-                       << item._file << item._instruction;
         }
 
-        while (!item.destination().startsWith(directories.top().first)) {
+        while (!item->destination().startsWith(directories.top().first)) {
             directories.pop();
         }
 
-        if (item._isDirectory) {
+        if (item->_isDirectory) {
             PropagateDirectory *dir = new PropagateDirectory(this, item);
             dir->_firstJob.reset(createJob(item));
-            if (item._instruction == CSYNC_INSTRUCTION_REMOVE) {
-                //We do the removal of directories at the end, because there might be moves from
-                // this directories that will happen later.
-                directoriesToRemove.append(dir);
-                removedDirectory = item._file + "/";
+
+            if (item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE
+                    && item->_direction == SyncFileItem::Up) {
+                // Skip all potential uploads to the new folder.
+                // Processing them now leads to problems with permissions:
+                // checkForPermissions() has already run and used the permissions
+                // of the file we're about to delete to decide whether uploading
+                // to the new dir is ok...
+                foreach(const SyncFileItemPtr &item2, items) {
+                    if (item2->destination().startsWith(item->destination() + "/")) {
+                        item2->_instruction = CSYNC_INSTRUCTION_NONE;
+                        _anotherSyncNeeded = true;
+                    }
+                }
+            }
+
+            if (item->_instruction == CSYNC_INSTRUCTION_REMOVE) {
+                // We do the removal of directories at the end, because there might be moves from
+                // these directories that will happen later.
+                directoriesToRemove.prepend(dir);
+                removedDirectory = item->_file + "/";
 
                 // We should not update the etag of parent directories of the removed directory
                 // since it would be done before the actual remove (issue #1845)
                 // NOTE: Currently this means that we don't update those etag at all in this sync,
                 //       but it should not be a problem, they will be updated in the next sync.
                 for (int i = 0; i < directories.size(); ++i) {
-                    directories[i].second->_item._should_update_etag = false;
+                    directories[i].second->_item->_should_update_metadata = false;
                 }
             } else {
                 PropagateDirectory* currentDirJob = directories.top().second;
                 currentDirJob->append(dir);
             }
-            directories.push(qMakePair(item.destination() + "/" , dir));
+            directories.push(qMakePair(item->destination() + "/" , dir));
         } else if (PropagateItemJob* current = createJob(item)) {
-            directories.top().second->append(current);
+            if (item->_instruction == CSYNC_INSTRUCTION_TYPE_CHANGE) {
+                // will delete directories, so defer execution
+                directoriesToRemove.prepend(current);
+                removedDirectory = item->_file + "/";
+            } else {
+                directories.top().second->append(current);
+            }
         }
     }
 
@@ -348,12 +404,13 @@ void OwncloudPropagator::start(const SyncFileItemVector& items)
         _rootJob->append(it);
     }
 
-    connect(_rootJob.data(), SIGNAL(completed(SyncFileItem)), this, SIGNAL(completed(SyncFileItem)));
-    connect(_rootJob.data(), SIGNAL(progress(SyncFileItem,quint64)), this, SIGNAL(progress(SyncFileItem,quint64)));
+    connect(_rootJob.data(), SIGNAL(itemCompleted(const SyncFileItem &, const PropagatorJob &)),
+            this, SIGNAL(itemCompleted(const SyncFileItem &, const PropagatorJob &)));
+    connect(_rootJob.data(), SIGNAL(progress(const SyncFileItem &,quint64)), this, SIGNAL(progress(const SyncFileItem &,quint64)));
     connect(_rootJob.data(), SIGNAL(finished(SyncFileItem::Status)), this, SLOT(emitFinished()));
     connect(_rootJob.data(), SIGNAL(ready()), this, SLOT(scheduleNextJob()), Qt::QueuedConnection);
 
-    qDebug() << (useLegacyJobs() ? "Using legacy libneon/HTTP sequential code path" : "Using QNAM/HTTP parallel code path");
+    qDebug() << "Using QNAM/HTTP parallel code path";
 
     QTimer::singleShot(0, this, SLOT(scheduleNextJob()));
 }
@@ -361,7 +418,7 @@ void OwncloudPropagator::start(const SyncFileItemVector& items)
 bool OwncloudPropagator::isInSharedDirectory(const QString& file)
 {
     bool re = false;
-    if( _remoteDir.contains("remote.php/webdav/Shared") ) {
+    if( _remoteDir.contains( _account->davPath() + QLatin1String("Shared") ) ) {
         // The Shared directory is synced as its own sync connection
         re = true;
     } else {
@@ -373,51 +430,6 @@ bool OwncloudPropagator::isInSharedDirectory(const QString& file)
     return re;
 }
 
-/**
- * Return true if we should use the legacy jobs.
- * Some feature are not supported by QNAM and therefore we still use the legacy jobs
- * for this case.
- */
-bool OwncloudPropagator::useLegacyJobs()
-{
-#ifdef USE_NEON
-    // Allow an environement variable for debugging
-    QByteArray env = qgetenv("OWNCLOUD_USE_LEGACY_JOBS");
-    if (env=="true" || env =="1") {
-        qDebug() << "Force Legacy Propagator ACTIVATED";
-        return true;
-    }
-
-    if (_downloadLimit.fetchAndAddAcquire(0) != 0 || _uploadLimit.fetchAndAddAcquire(0) != 0) {
-        // QNAM bandwith limiting only work with version of Qt greater or equal to 5.3.3
-        // (It needs Qt commits 097b641 and b99fa32)
-#if QT_VERSION >= QT_VERSION_CHECK(5,3,3)
-        return false;
-#elif QT_VERSION >= QT_VERSION_CHECK(5,0,0)
-        env = qgetenv("OWNCLOUD_NEW_BANDWIDTH_LIMITING");
-        if (env=="true" || env =="1") {
-            qDebug() << "New Bandwidth Limiting Code ACTIVATED";
-            return false;
-        }
-
-        // Do a runtime check.
-        // (Poor man's version comparison)
-        const char *v = qVersion(); // "x.y.z";
-        if (QLatin1String(v) >= QLatin1String("5.3.3")) {
-            return false;
-        } else {
-            qDebug() << "Use legacy jobs because qt version is only" << v << "while 5.3.3 is needed";
-            return true;
-        }
-#else
-        qDebug() << "Use legacy jobs because of Qt4";
-        return true;
-#endif
-    }
-#endif // USE_NEON
-    return false;
-}
-
 int OwncloudPropagator::httpTimeout()
 {
     static int timeout;
@@ -432,6 +444,20 @@ int OwncloudPropagator::httpTimeout()
     return timeout;
 }
 
+quint64 OwncloudPropagator::chunkSize()
+{
+    static uint chunkSize;
+    if (!chunkSize) {
+        chunkSize = qgetenv("OWNCLOUD_CHUNK_SIZE").toUInt();
+        if (chunkSize == 0) {
+            ConfigFile cfg;
+            chunkSize = cfg.chunkSize();
+        }
+    }
+    return chunkSize;
+}
+
+
 bool OwncloudPropagator::localFileNameClash( const QString& relFile )
 {
     bool re = false;
@@ -472,7 +498,7 @@ bool OwncloudPropagator::localFileNameClash( const QString& relFile )
             }
         }
 #else
-        // On Linux, the file system is case sensitive, but this code is usefull for testing.
+        // On Linux, the file system is case sensitive, but this code is useful for testing.
         // Just check that there is no other file with the same name and different casing.
         QFileInfo fileInfo(file);
         const QString fn = fileInfo.fileName();
@@ -525,6 +551,24 @@ AccountPtr OwncloudPropagator::account() const
     return _account;
 }
 
+OwncloudPropagator::DiskSpaceResult OwncloudPropagator::diskSpaceCheck() const
+{
+    const qint64 freeBytes = Utility::freeDiskSpace(_localDir);
+    if (freeBytes < 0) {
+        return DiskSpaceOk;
+    }
+
+    if (freeBytes < criticalFreeSpaceLimit()) {
+        return DiskSpaceCritical;
+    }
+
+    if (freeBytes - _rootJob->committedDiskSpace() < freeSpaceLimit()) {
+        return DiskSpaceFailure;
+    }
+
+    return DiskSpaceOk;
+}
+
 // ================================================================================
 
 PropagatorJob::JobParallelism PropagateDirectory::parallelism()
@@ -605,24 +649,24 @@ void PropagateDirectory::slotSubJobFinished(SyncFileItem::Status status)
             (sender() == _firstJob.data() && status != SyncFileItem::Success && status != SyncFileItem::Restoration)) {
         abort();
         _state = Finished;
+        emit itemCompleted(*_item, *this);
         emit finished(status);
         return;
     } else if (status == SyncFileItem::NormalError || status == SyncFileItem::SoftError) {
         _hasError = status;
     }
     _runningNow--;
+    _jobsFinished++;
 
-    int total = _subJobs.count();
-    if (!_firstJob) {
-        total--;
+    int totalJobs = _subJobs.count();
+    if (_firstJob) {
+        totalJobs++;
     }
 
-    _current++;
-
-    // We finished to processing all the jobs
+    // We finished processing all the jobs
     // check if we finished
-    if (_current >= total) {
-        Q_ASSERT(!_runningNow); // how can we finished if there are still jobs running now
+    if (_jobsFinished >= totalJobs) {
+        Q_ASSERT(!_runningNow); // how can we be finished if there are still jobs running now
         finalize();
     } else {
         emit ready();
@@ -631,24 +675,38 @@ void PropagateDirectory::slotSubJobFinished(SyncFileItem::Status status)
 
 void PropagateDirectory::finalize()
 {
-    if (!_item.isEmpty() && _hasError == SyncFileItem::NoStatus) {
-        if( !_item._renameTarget.isEmpty() ) {
-            _item._file = _item._renameTarget;
+    if (!_item->isEmpty() && _hasError == SyncFileItem::NoStatus) {
+        if( !_item->_renameTarget.isEmpty() ) {
+            _item->_file = _item->_renameTarget;
         }
 
-        if (_item._should_update_etag && _item._instruction != CSYNC_INSTRUCTION_REMOVE) {
+        if (_item->_should_update_metadata && _item->_instruction != CSYNC_INSTRUCTION_REMOVE) {
             if (PropagateRemoteMkdir* mkdir = qobject_cast<PropagateRemoteMkdir*>(_firstJob.data())) {
                 // special case from MKDIR, get the fileId from the job there
-                if (_item._fileId.isEmpty() && !mkdir->_item._fileId.isEmpty()) {
-                    _item._fileId = mkdir->_item._fileId;
+                if (_item->_fileId.isEmpty() && !mkdir->_item->_fileId.isEmpty()) {
+                    _item->_fileId = mkdir->_item->_fileId;
                 }
             }
-            SyncJournalFileRecord record(_item,  _propagator->_localDir + _item._file);
-            _propagator->_journal->setFileRecord(record);
+            SyncJournalFileRecord record(*_item,  _propagator->_localDir + _item->_file);
+            _propagator->_journal->setFileRecordMetadata(record);
         }
     }
     _state = Finished;
-    emit finished(_hasError == SyncFileItem::NoStatus ? SyncFileItem::Success : _hasError);
+    // Just to make sure that the SocketApi will know by looking in
+    // SyncEngine::_syncedItems that this folder is done synchronizing.
+    _item->_status = SyncFileItem::Success;
+
+    emit itemCompleted(*_item, *this);
+    emit finished(_item->_status);
+}
+
+qint64 PropagateDirectory::committedDiskSpace() const
+{
+    qint64 needed = 0;
+    foreach (PropagatorJob* job, _subJobs) {
+        needed += job->committedDiskSpace();
+    }
+    return needed;
 }
 
 CleanupPollsJob::~CleanupPollsJob()
@@ -664,9 +722,8 @@ void CleanupPollsJob::start()
 
     auto info = _pollInfos.first();
     _pollInfos.pop_front();
-    SyncFileItem item;
-    item._file = info._file;
-    item._modtime = info._modtime;
+    SyncFileItemPtr item(new SyncFileItem(
+            _journal->getFileRecord(info._file).toSyncFileItem()));
     PollJob *job = new PollJob(_account, info._url, item, _journal, _localPath, this);
     connect(job, SIGNAL(finishedSignal()), SLOT(slotPollFinished()));
     job->start();
@@ -676,13 +733,13 @@ void CleanupPollsJob::slotPollFinished()
 {
     PollJob *job = qobject_cast<PollJob *>(sender());
     Q_ASSERT(job);
-    if (job->_item._status == SyncFileItem::FatalError) {
-        emit aborted(job->_item._errorString);
+    if (job->_item->_status == SyncFileItem::FatalError) {
+        emit aborted(job->_item->_errorString);
         return;
-    } else if (job->_item._status != SyncFileItem::Success) {
-        qDebug() << "There was an error with file " << job->_item._file << job->_item._errorString;
+    } else if (job->_item->_status != SyncFileItem::Success) {
+        qDebug() << "There was an error with file " << job->_item->_file << job->_item->_errorString;
     } else {
-        _journal->setFileRecord(SyncJournalFileRecord(job->_item, _localPath + job->_item._file));
+        _journal->setFileRecord(SyncJournalFileRecord(*job->_item, _localPath + job->_item->_file));
     }
     // Continue with the next entry, or finish
     start();
