@@ -22,7 +22,7 @@
 #include "utility.h"
 #include "filesystem.h"
 #include "propagatorjobs.h"
-#include "transmissionchecksumvalidator.h"
+#include "checksums.h"
 
 #include <json.h>
 #include <QNetworkAccessManager>
@@ -35,7 +35,7 @@ namespace OCC {
 
 // Always coming in with forward slashes.
 // In csync_excluded_no_ctx we ignore all files with longer than 254 chars
-// This function also adds a dot at the begining of the filename to hide the file on OS X and Linux
+// This function also adds a dot at the beginning of the filename to hide the file on OS X and Linux
 QString OWNCLOUDSYNC_EXPORT createDownloadTmpFileName(const QString &previous) {
     QString tmpFileName;
     QString tmpPath;
@@ -57,7 +57,7 @@ QString OWNCLOUDSYNC_EXPORT createDownloadTmpFileName(const QString &previous) {
     }
 }
 
-// DOES NOT take owncership of the device.
+// DOES NOT take ownership of the device.
 GETFileJob::GETFileJob(AccountPtr account, const QString& path, QFile *device,
                     const QMap<QByteArray, QByteArray> &headers, const QByteArray &expectedEtagForResume,
                     quint64 resumeStart,  QObject* parent)
@@ -171,7 +171,7 @@ void GETFileJob::slotMetaDataChanged()
     if (start != _resumeStart) {
         qDebug() << Q_FUNC_INFO <<  "Wrong content-range: "<< ranges << " while expecting start was" << _resumeStart;
         if (ranges.isEmpty()) {
-            // device don't support range, just stry again from scratch
+            // device doesn't support range, just try again from scratch
             _device->close();
             if (!_device->open(QIODevice::WriteOnly)) {
                 _errorString = _device->errorString();
@@ -242,7 +242,7 @@ void GETFileJob::slotReadyRead()
         if (_bandwidthLimited) {
             toRead = qMin(qint64(bufferSize), _bandwidthQuota);
             if (toRead == 0) {
-                qDebug() << Q_FUNC_INFO << "Out of quota";
+                //qDebug() << Q_FUNC_INFO << "Out of quota";
                 break;
             }
             _bandwidthQuota -= toRead;
@@ -286,6 +286,7 @@ void GETFileJob::slotReadyRead()
 
 void GETFileJob::slotTimeout()
 {
+    qDebug() << "Timeout" << reply()->request().url();
     _errorString =  tr("Connection Timeout");
     _errorStatus = SyncFileItem::FatalError;
     reply()->abort();
@@ -307,25 +308,34 @@ void PropagateDownloadFileQNAM::start()
     if (_propagator->_abortRequested.fetchAndAddRelaxed(0))
         return;
 
-    qDebug() << Q_FUNC_INFO << _item._file << _propagator->_activeJobs;
+    qDebug() << Q_FUNC_INFO << _item->_file << _propagator->_activeJobs;
+
+    if (_deleteExisting) {
+        deleteExistingFolder();
+
+        // check for error with deletion
+        if (_state == Finished) {
+            return;
+        }
+    }
 
     // do a klaas' case clash check.
-    if( _propagator->localFileNameClash(_item._file) ) {
+    if( _propagator->localFileNameClash(_item->_file) ) {
         done( SyncFileItem::NormalError, tr("File %1 can not be downloaded because of a local file name clash!")
-              .arg(QDir::toNativeSeparators(_item._file)) );
+              .arg(QDir::toNativeSeparators(_item->_file)) );
         return;
     }
 
-    emit progress(_item, 0);
+    emit progress(*_item, 0);
 
     QString tmpFileName;
     QByteArray expectedEtagForResume;
-    const SyncJournalDb::DownloadInfo progressInfo = _propagator->_journal->getDownloadInfo(_item._file);
+    const SyncJournalDb::DownloadInfo progressInfo = _propagator->_journal->getDownloadInfo(_item->_file);
     if (progressInfo._valid) {
         // if the etag has changed meanwhile, remove the already downloaded part.
-        if (progressInfo._etag != _item._etag) {
-            QFile::remove(_propagator->getFilePath(progressInfo._tmpfile));
-            _propagator->_journal->setDownloadInfo(_item._file, SyncJournalDb::DownloadInfo());
+        if (progressInfo._etag != _item->_etag) {
+            FileSystem::remove(_propagator->getFilePath(progressInfo._tmpfile));
+            _propagator->_journal->setDownloadInfo(_item->_file, SyncJournalDb::DownloadInfo());
         } else {
             tmpFileName = progressInfo._tmpfile;
             expectedEtagForResume = progressInfo._etag;
@@ -334,7 +344,7 @@ void PropagateDownloadFileQNAM::start()
     }
 
     if (tmpFileName.isEmpty()) {
-        tmpFileName = createDownloadTmpFileName(_item._file);
+        tmpFileName = createDownloadTmpFileName(_item->_file);
     }
 
     _tmpFile.setFileName(_propagator->getFilePath(tmpFileName));
@@ -345,21 +355,9 @@ void PropagateDownloadFileQNAM::start()
 
     FileSystem::setFileHidden(_tmpFile.fileName(), true);
 
-    {
-        SyncJournalDb::DownloadInfo pi;
-        pi._etag = _item._etag;
-        pi._tmpfile = tmpFileName;
-        pi._valid = true;
-        _propagator->_journal->setDownloadInfo(_item._file, pi);
-        _propagator->_journal->commit("download file start");
-    }
-
-
-    QMap<QByteArray, QByteArray> headers;
-
-    quint64 startSize = _tmpFile.size();
-    if (startSize > 0) {
-        if (startSize == _item._size) {
+    _resumeStart = _tmpFile.size();
+    if (_resumeStart > 0) {
+        if (_resumeStart == _item->_size) {
             qDebug() << "File is already complete, no need to download";
             _tmpFile.close();
             downloadFinished();
@@ -367,23 +365,49 @@ void PropagateDownloadFileQNAM::start()
         }
     }
 
-    if (_item._directDownloadUrl.isEmpty()) {
+    // If there's not enough space to fully download this file, stop.
+    const auto diskSpaceResult = _propagator->diskSpaceCheck();
+    if (diskSpaceResult == OwncloudPropagator::DiskSpaceFailure) {
+        _item->_errorMayBeBlacklisted = true;
+        done(SyncFileItem::NormalError,
+             tr("The download would reduce free disk space below %1").arg(
+                 Utility::octetsToString(freeSpaceLimit())));
+        return;
+    } else if (diskSpaceResult == OwncloudPropagator::DiskSpaceCritical) {
+        done(SyncFileItem::FatalError,
+             tr("Free space on disk is less than %1").arg(
+                 Utility::octetsToString(criticalFreeSpaceLimit())));
+        return;
+    }
+
+    {
+        SyncJournalDb::DownloadInfo pi;
+        pi._etag = _item->_etag;
+        pi._tmpfile = tmpFileName;
+        pi._valid = true;
+        _propagator->_journal->setDownloadInfo(_item->_file, pi);
+        _propagator->_journal->commit("download file start");
+    }
+
+    QMap<QByteArray, QByteArray> headers;
+
+    if (_item->_directDownloadUrl.isEmpty()) {
         // Normal job, download from oC instance
         _job = new GETFileJob(_propagator->account(),
-                            _propagator->_remoteFolder + _item._file,
-                            &_tmpFile, headers, expectedEtagForResume, startSize);
+                            _propagator->_remoteFolder + _item->_file,
+                            &_tmpFile, headers, expectedEtagForResume, _resumeStart);
     } else {
         // We were provided a direct URL, use that one
-        qDebug() << Q_FUNC_INFO << "directDownloadUrl given for " << _item._file << _item._directDownloadUrl;
+        qDebug() << Q_FUNC_INFO << "directDownloadUrl given for " << _item->_file << _item->_directDownloadUrl;
 
-        if (!_item._directDownloadCookies.isEmpty()) {
-            headers["Cookie"] = _item._directDownloadCookies.toUtf8();
+        if (!_item->_directDownloadCookies.isEmpty()) {
+            headers["Cookie"] = _item->_directDownloadCookies.toUtf8();
         }
 
-        QUrl url = QUrl::fromUserInput(_item._directDownloadUrl);
+        QUrl url = QUrl::fromUserInput(_item->_directDownloadUrl);
         _job = new GETFileJob(_propagator->account(),
                               url,
-                              &_tmpFile, headers, expectedEtagForResume, startSize);
+                              &_tmpFile, headers, expectedEtagForResume, _resumeStart);
     }
     _job->setBandwidthManager(&_propagator->_bandwidthManager);
     connect(_job, SIGNAL(finishedSignal()), this, SLOT(slotGetFinished()));
@@ -392,6 +416,19 @@ void PropagateDownloadFileQNAM::start()
     _job->start();
 }
 
+qint64 PropagateDownloadFileQNAM::committedDiskSpace() const
+{
+    if (_state == Running) {
+        return qBound(0ULL, _item->_size - _resumeStart - _downloadProgress, _item->_size);
+    }
+    return 0;
+}
+
+void PropagateDownloadFileQNAM::setDeleteExistingFolder(bool enabled)
+{
+    _deleteExisting = enabled;
+}
+
 const char owncloudCustomSoftErrorStringC[] = "owncloud-custom-soft-error-string";
 void PropagateDownloadFileQNAM::slotGetFinished()
 {
@@ -402,32 +439,41 @@ void PropagateDownloadFileQNAM::slotGetFinished()
 
     qDebug() << Q_FUNC_INFO << job->reply()->request().url() << "FINISHED WITH STATUS"
              << job->reply()->error()
-             << (job->reply()->error() == QNetworkReply::NoError ? QLatin1String("") : job->reply()->errorString());
+             << (job->reply()->error() == QNetworkReply::NoError ? QLatin1String("") : job->reply()->errorString())
+             << _item->_httpErrorCode
+             << _tmpFile.size() << _item->_size << job->resumeStart()
+             << job->reply()->rawHeader("Content-Range") << job->reply()->rawHeader("Content-Length");
 
     QNetworkReply::NetworkError err = job->reply()->error();
     if (err != QNetworkReply::NoError) {
-        _item._httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+        _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
 
         // If we sent a 'Range' header and get 416 back, we want to retry
         // without the header.
-        bool badRangeHeader = job->resumeStart() > 0 && _item._httpErrorCode == 416;
+        const bool badRangeHeader = job->resumeStart() > 0 && _item->_httpErrorCode == 416;
         if (badRangeHeader) {
             qDebug() << Q_FUNC_INFO << "server replied 416 to our range request, trying again without";
             _propagator->_anotherSyncNeeded = true;
         }
 
-        // Don't keep the temporary file if it is empty or we
-        // used a bad range header.
-        if (_tmpFile.size() == 0 || badRangeHeader) {
-            _tmpFile.close();
-            _tmpFile.remove();
-            _propagator->_journal->setDownloadInfo(_item._file, SyncJournalDb::DownloadInfo());
+        // Getting a 404 probably means that the file was deleted on the server.
+        const bool fileNotFound = _item->_httpErrorCode == 404;
+        if (fileNotFound) {
+            qDebug() << Q_FUNC_INFO << "server replied 404, assuming file was deleted";
         }
 
-        if(!_item._directDownloadUrl.isEmpty() && err != QNetworkReply::OperationCanceledError) {
+        // Don't keep the temporary file if it is empty or we
+        // used a bad range header or the file's not on the server anymore.
+        if (_tmpFile.size() == 0 || badRangeHeader || fileNotFound) {
+            _tmpFile.close();
+            FileSystem::remove(_tmpFile.fileName());
+            _propagator->_journal->setDownloadInfo(_item->_file, SyncJournalDb::DownloadInfo());
+        }
+
+        if(!_item->_directDownloadUrl.isEmpty() && err != QNetworkReply::OperationCanceledError) {
             // If this was with a direct download, retry without direct download
-            qWarning() << "Direct download of" << _item._directDownloadUrl << "failed. Retrying through owncloud.";
-            _item._directDownloadUrl.clear();
+            qWarning() << "Direct download of" << _item->_directDownloadUrl << "failed. Retrying through owncloud.";
+            _item->_directDownloadUrl.clear();
             start();
             return;
         }
@@ -439,19 +485,21 @@ void PropagateDownloadFileQNAM::slotGetFinished()
         if (err == QNetworkReply::OperationCanceledError && reply->property(owncloudCustomSoftErrorStringC).isValid()) {
             job->setErrorString(reply->property(owncloudCustomSoftErrorStringC).toString());
             job->setErrorStatus(SyncFileItem::SoftError);
+        } else if (badRangeHeader) {
+            // Can't do this in classifyError() because 416 without a
+            // Range header should result in NormalError.
+            job->setErrorStatus(SyncFileItem::SoftError);
+        } else if (fileNotFound) {
+            job->setErrorString(tr("File was deleted from server"));
+            job->setErrorStatus(SyncFileItem::SoftError);
         }
 
         SyncFileItem::Status status = job->errorStatus();
-
-
         if (status == SyncFileItem::NoStatus) {
-            status = classifyError(err, _item._httpErrorCode);
-        }
-        if (badRangeHeader) {
-            // Can't do this in classifyError() because 416 without a
-            // Range header should result in NormalError.
-            status = SyncFileItem::SoftError;
+            status = classifyError(err, _item->_httpErrorCode,
+                                   &_propagator->_anotherSyncNeeded);
         }
+
         done(status, job->errorString());
         return;
     }
@@ -459,15 +507,15 @@ void PropagateDownloadFileQNAM::slotGetFinished()
     if (!job->etag().isEmpty()) {
         // The etag will be empty if we used a direct download URL.
         // (If it was really empty by the server, the GETFileJob will have errored
-        _item._etag = parseEtag(job->etag());
+        _item->_etag = parseEtag(job->etag());
     }
     if (job->lastModified()) {
         // It is possible that the file was modified on the server since we did the discovery phase
         // so make sure we have the up-to-date time
-        _item._modtime = job->lastModified();
+        _item->_modtime = job->lastModified();
     }
-    _item._requestDuration = job->duration();
-    _item._responseTimeStamp = job->responseTimestamp();
+    _item->_requestDuration = job->duration();
+    _item->_responseTimeStamp = job->responseTimestamp();
 
     _tmpFile.close();
     _tmpFile.flush();
@@ -479,6 +527,15 @@ void PropagateDownloadFileQNAM::slotGetFinished()
     const QByteArray sizeHeader("Content-Length");
     quint64 bodySize = job->reply()->rawHeader(sizeHeader).toULongLong();
 
+    if (!job->reply()->rawHeader(sizeHeader).isEmpty() && _tmpFile.size() > 0 && bodySize == 0) {
+        // Strange bug with broken webserver or webfirewall https://github.com/owncloud/client/issues/3373#issuecomment-122672322
+        // This happened when trying to resume a file. The Content-Range header was files, Content-Length was == 0
+        qDebug() << bodySize << _item->_size << _tmpFile.size() << job->resumeStart();
+        FileSystem::remove(_tmpFile.fileName());
+        done(SyncFileItem::SoftError, QLatin1String("Broken webserver returning empty content length for non-empty file on resume"));
+        return;
+    }
+
     if(bodySize > 0 && bodySize != _tmpFile.size() - job->resumeStart() ) {
         qDebug() << bodySize << _tmpFile.size() << job->resumeStart();
         _propagator->_anotherSyncNeeded = true;
@@ -489,49 +546,59 @@ void PropagateDownloadFileQNAM::slotGetFinished()
     // Do checksum validation for the download. If there is no checksum header, the validator
     // will also emit the validated() signal to continue the flow in slot downloadFinished()
     // as this is (still) also correct.
-    TransmissionChecksumValidator *validator = new TransmissionChecksumValidator(_tmpFile.fileName(), this);
-    connect(validator, SIGNAL(validated(QByteArray)), this, SLOT(downloadFinished()));
-    connect(validator, SIGNAL(validationFailed(QString)), this, SLOT(slotChecksumFail(QString)));
-    validator->downloadValidation(job->reply()->rawHeader(checkSumHeaderC));
-
+    ValidateChecksumHeader *validator = new ValidateChecksumHeader(this);
+    connect(validator, SIGNAL(validated(QByteArray,QByteArray)),
+            SLOT(downloadFinished()));
+    connect(validator, SIGNAL(validationFailed(QString)),
+            SLOT(slotChecksumFail(QString)));
+    auto checksumHeader = job->reply()->rawHeader(checkSumHeaderC);
+    if (!downloadChecksumEnabled()) {
+        checksumHeader.clear();
+    }
+    validator->start(_tmpFile.fileName(), checksumHeader);
 }
 
 void PropagateDownloadFileQNAM::slotChecksumFail( const QString& errMsg )
 {
-    _tmpFile.remove();
+    FileSystem::remove(_tmpFile.fileName());
     _propagator->_anotherSyncNeeded = true;
     done(SyncFileItem::SoftError, errMsg ); // tr("The file downloaded with a broken checksum, will be redownloaded."));
 }
 
-QString makeConflictFileName(const QString &fn, const QDateTime &dt)
+void PropagateDownloadFileQNAM::deleteExistingFolder()
 {
-    QString conflictFileName(fn);
-    // Add _conflict-XXXX  before the extention.
-    int dotLocation = conflictFileName.lastIndexOf('.');
-    // If no extention, add it at the end  (take care of cases like foo/.hidden or foo.bar/file)
-    if (dotLocation <= conflictFileName.lastIndexOf('/') + 1) {
-        dotLocation = conflictFileName.size();
+    QString existingDir = _propagator->getFilePath(_item->_file);
+    if (!QFileInfo(existingDir).isDir()) {
+        return;
     }
-    QString timeString = dt.toString("yyyyMMdd-hhmmss");
 
-    // Additional marker
-    QByteArray conflictFileUserName = qgetenv("CSYNC_CONFLICT_FILE_USERNAME");
-    if (conflictFileUserName.isEmpty())
-        conflictFileName.insert(dotLocation, "_conflict-" + timeString);
-    else
-        conflictFileName.insert(dotLocation, "_conflict_" + QString::fromUtf8(conflictFileUserName)  + "-" + timeString);
+    // Delete the directory if it is empty!
+    QDir dir(existingDir);
+    if (dir.entryList(QDir::NoDotAndDotDot|QDir::AllEntries).count() == 0) {
+        if (dir.rmdir(existingDir)) {
+            return;
+        }
+        // on error, just try to move it away...
+    }
 
-    return conflictFileName;
+    QString conflictDir = FileSystem::makeConflictFileName(
+            existingDir, Utility::qDateTimeFromTime_t(_item->_modtime));
+
+    _propagator->addTouchedFile(existingDir);
+    _propagator->addTouchedFile(conflictDir);
+    QString renameError;
+    if (!FileSystem::rename(existingDir, conflictDir, &renameError)) {
+        done(SyncFileItem::NormalError, renameError);
+    }
 }
 
-
 namespace { // Anonymous namespace for the recall feature
 static QString makeRecallFileName(const QString &fn)
 {
     QString recallFileName(fn);
-    // Add _recall-XXXX  before the extention.
+    // Add _recall-XXXX  before the extension.
     int dotLocation = recallFileName.lastIndexOf('.');
-    // If no extention, add it at the end  (take care of cases like foo/.hidden or foo.bar/file)
+    // If no extension, add it at the end  (take care of cases like foo/.hidden or foo.bar/file)
     if (dotLocation <= recallFileName.lastIndexOf('/') + 1) {
         dotLocation = recallFileName.size();
     }
@@ -562,33 +629,32 @@ static void handleRecallFile(const QString &fn)
         QString fpath = thisDir.filePath(line);
         QString rpath = makeRecallFileName(fpath);
 
-        // if previously recalled file exists then remove it (copy will not overwrite it)
-        QFile(rpath).remove();
         qDebug() << "Copy recall file: " << fpath << " -> " << rpath;
-        QFile::copy(fpath,rpath);
+        QString error;
+        FileSystem::uncheckedRenameReplace(fpath, rpath, &error);
     }
 }
 } // end namespace
 
 void PropagateDownloadFileQNAM::downloadFinished()
 {
-    QString fn = _propagator->getFilePath(_item._file);
+    QString fn = _propagator->getFilePath(_item->_file);
 
     // In case of file name clash, report an error
     // This can happen if another parallel download saved a clashing file.
-    if (_propagator->localFileNameClash(_item._file)) {
+    if (_propagator->localFileNameClash(_item->_file)) {
         done( SyncFileItem::NormalError, tr("File %1 cannot be saved because of a local file name clash!")
-              .arg(QDir::toNativeSeparators(_item._file)) );
+              .arg(QDir::toNativeSeparators(_item->_file)) );
         return;
     }
 
     // In case of conflict, make a backup of the old file
     // Ignore conflicts where both files are binary equal
-    bool isConflict = _item._instruction == CSYNC_INSTRUCTION_CONFLICT
+    bool isConflict = _item->_instruction == CSYNC_INSTRUCTION_CONFLICT
             && !FileSystem::fileEquals(fn, _tmpFile.fileName());
     if (isConflict) {
         QString renameError;
-        QString conflictFileName = makeConflictFileName(fn, Utility::qDateTimeFromTime_t(_item._modtime));
+        QString conflictFileName = FileSystem::makeConflictFileName(fn, Utility::qDateTimeFromTime_t(_item->_modtime));
         if (!FileSystem::rename(fn, conflictFileName, &renameError)) {
             //If the rename fails, don't replace it.
             done(SyncFileItem::SoftError, renameError);
@@ -597,10 +663,10 @@ void PropagateDownloadFileQNAM::downloadFinished()
         qDebug() << "Created conflict file" << fn << "->" << conflictFileName;
     }
 
-    FileSystem::setModTime(_tmpFile.fileName(), _item._modtime);
-    // We need to fetch the time again because some file system such as FAT have a less than a second
+    FileSystem::setModTime(_tmpFile.fileName(), _item->_modtime);
+    // We need to fetch the time again because some file systems such as FAT have worse than a second
     // Accuracy, and we really need the time from the file system. (#3103)
-    _item._modtime = FileSystem::getModTime(_tmpFile.fileName());
+    _item->_modtime = FileSystem::getModTime(_tmpFile.fileName());
 
     if (FileSystem::fileExists(fn)) {
         // Preserve the existing file permissions.
@@ -613,8 +679,8 @@ void PropagateDownloadFileQNAM::downloadFinished()
         // phase by comparing size and mtime to the previous values. This
         // is necessary to avoid overwriting user changes that happened between
         // the discovery phase and now.
-        const qint64 expectedSize = _item.log._other_size;
-        const time_t expectedMtime = _item.log._other_modtime;
+        const qint64 expectedSize = _item->log._other_size;
+        const time_t expectedMtime = _item->log._other_modtime;
         if (! FileSystem::verifyFileUnchanged(fn, expectedSize, expectedMtime)) {
             _propagator->_anotherSyncNeeded = true;
             done(SyncFileItem::SoftError, tr("File has changed since discovery"));
@@ -622,6 +688,10 @@ void PropagateDownloadFileQNAM::downloadFinished()
         }
     }
 
+    // Apply the remote permissions
+    FileSystem::setFileReadOnlyWeak(_tmpFile.fileName(),
+                                    !_item->_remotePerm.contains('W'));
+
     QString error;
     _propagator->addTouchedFile(fn);
     // The fileChanged() check is done above to generate better error messages.
@@ -648,15 +718,15 @@ void PropagateDownloadFileQNAM::downloadFinished()
 
     // Maybe we downloaded a newer version of the file than we thought we would...
     // Get up to date information for the journal.
-    _item._size = FileSystem::getSize(fn);
+    _item->_size = FileSystem::getSize(fn);
 
-    _propagator->_journal->setFileRecord(SyncJournalFileRecord(_item, fn));
-    _propagator->_journal->setDownloadInfo(_item._file, SyncJournalDb::DownloadInfo());
+    _propagator->_journal->setFileRecord(SyncJournalFileRecord(*_item, fn));
+    _propagator->_journal->setDownloadInfo(_item->_file, SyncJournalDb::DownloadInfo());
     _propagator->_journal->commit("download file start2");
     done(isConflict ? SyncFileItem::Conflict : SyncFileItem::Success);
 
     // handle the special recall file
-    if(_item._file == QLatin1String(".sys.admin#recall#") || _item._file.endsWith("/.sys.admin#recall#")) {
+    if(_item->_file == QLatin1String(".sys.admin#recall#") || _item->_file.endsWith("/.sys.admin#recall#")) {
         handleRecallFile(fn);
     }
 }
@@ -664,7 +734,8 @@ void PropagateDownloadFileQNAM::downloadFinished()
 void PropagateDownloadFileQNAM::slotDownloadProgress(qint64 received, qint64)
 {
     if (!_job) return;
-    emit progress(_item, received + _job->resumeStart());
+    _downloadProgress = received;
+    emit progress(*_item, _resumeStart + received);
 }
 
 
