@@ -18,6 +18,10 @@
 
 namespace OCC {
 
+/**
+ * @brief The DeleteJob class
+ * @ingroup libsync
+ */
 class DeleteJob : public AbstractNetworkJob {
     Q_OBJECT
 public:
@@ -33,11 +37,15 @@ signals:
     void finishedSignal();
 };
 
+/**
+ * @brief The PropagateRemoteDelete class
+ * @ingroup libsync
+ */
 class PropagateRemoteDelete : public PropagateItemJob {
     Q_OBJECT
     QPointer<DeleteJob> _job;
 public:
-    PropagateRemoteDelete (OwncloudPropagator* propagator,const SyncFileItem& item)
+    PropagateRemoteDelete (OwncloudPropagator* propagator,const SyncFileItemPtr& item)
         : PropagateItemJob(propagator, item) {}
     void start() Q_DECL_OVERRIDE;
     void abort() Q_DECL_OVERRIDE;
@@ -46,4 +54,4 @@ private slots:
 
 };
 
-}
\ No newline at end of file
+}
