@@ -22,7 +22,9 @@
 #include "utility.h"
 #include "filesystem.h"
 #include "propagatorjobs.h"
-#include "transmissionchecksumvalidator.h"
+#include "checksums.h"
+#include "syncengine.h"
+#include "propagateremotedelete.h"
 
 #include <json.h>
 #include <QNetworkAccessManager>
@@ -31,10 +33,6 @@
 #include <cmath>
 #include <cstring>
 
-#ifdef USE_NEON
-#include "propagator_legacy.h"
-#endif
-
 #if QT_VERSION < QT_VERSION_CHECK(5, 4, 2)
 namespace {
 const char owncloudShouldSoftCancelPropertyName[] = "owncloud-should-soft-cancel";
@@ -57,22 +55,11 @@ static bool fileIsStillChanging(const SyncFileItem & item)
     const QDateTime modtime = Utility::qDateTimeFromTime_t(item._modtime);
     const qint64 msSinceMod = modtime.msecsTo(QDateTime::currentDateTime());
 
-    return msSinceMod < 2000
+    return msSinceMod < SyncEngine::minimumFileAgeForUpload
             // if the mtime is too much in the future we *do* upload the file
             && msSinceMod > -10000;
 }
 
-static qint64 chunkSize() {
-    static uint chunkSize;
-    if (!chunkSize) {
-        chunkSize = qgetenv("OWNCLOUD_CHUNK_SIZE").toUInt();
-        if (chunkSize == 0) {
-            chunkSize = 5*1024*1024; // default to 5 MiB
-        }
-    }
-    return chunkSize;
-}
-
 PUTFileJob::~PUTFileJob()
 {
     // Make sure that we destroy the QNetworkReply before our _device of which it keeps an internal pointer.
@@ -110,6 +97,7 @@ void PUTFileJob::start() {
 }
 
 void PUTFileJob::slotTimeout() {
+    qDebug() << "Timeout" << reply()->request().url();
     _errorString =  tr("Connection Timeout");
     reply()->abort();
 }
@@ -137,19 +125,19 @@ bool PollJob::finished()
 {
     QNetworkReply::NetworkError err = reply()->error();
     if (err != QNetworkReply::NoError) {
-        _item._httpErrorCode = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-        _item._status = classifyError(err, _item._httpErrorCode);
-        _item._errorString = reply()->errorString();
+        _item->_httpErrorCode = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+        _item->_status = classifyError(err, _item->_httpErrorCode);
+        _item->_errorString = reply()->errorString();
 
         if (reply()->hasRawHeader("OC-ErrorString")) {
-            _item._errorString = reply()->rawHeader("OC-ErrorString");
+            _item->_errorString = reply()->rawHeader("OC-ErrorString");
         }
 
-        if (_item._status == SyncFileItem::FatalError || _item._httpErrorCode >= 400) {
-            if (_item._status != SyncFileItem::FatalError
-                    && _item._httpErrorCode != 503) {
+        if (_item->_status == SyncFileItem::FatalError || _item->_httpErrorCode >= 400) {
+            if (_item->_status != SyncFileItem::FatalError
+                    && _item->_httpErrorCode != 503) {
                 SyncJournalDb::PollInfo info;
-                info._file = _item._file;
+                info._file = _item->_file;
                 // no info._url removes it from the database
                 _journal->setPollInfo(info);
                 _journal->commit("remove poll info");
@@ -167,8 +155,8 @@ bool PollJob::finished()
     qDebug() << Q_FUNC_INFO << ">" << jsonData << "<" << reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
     QVariantMap status = QtJson::parse(QString::fromUtf8(jsonData), ok).toMap();
     if (!ok || status.isEmpty()) {
-        _item._errorString = tr("Invalid JSON reply from the poll URL");
-        _item._status = SyncFileItem::NormalError;
+        _item->_errorString = tr("Invalid JSON reply from the poll URL");
+        _item->_status = SyncFileItem::NormalError;
         emit finishedSignal();
         return true;
     }
@@ -178,14 +166,14 @@ bool PollJob::finished()
         return false;
     }
 
-    _item._errorString = status["error"].toString();
-    _item._status = _item._errorString.isEmpty() ? SyncFileItem::Success : SyncFileItem::NormalError;
-    _item._fileId = status["fileid"].toByteArray();
-    _item._etag = status["etag"].toByteArray();
-    _item._responseTimeStamp = responseTimestamp();
+    _item->_errorString = status["error"].toString();
+    _item->_status = _item->_errorString.isEmpty() ? SyncFileItem::Success : SyncFileItem::NormalError;
+    _item->_fileId = status["fileid"].toByteArray();
+    _item->_etag = status["etag"].toByteArray();
+    _item->_responseTimeStamp = responseTimestamp();
 
     SyncJournalDb::PollInfo info;
-    info._file = _item._file;
+    info._file = _item->_file;
     // no info._url removes it from the database
     _journal->setPollInfo(info);
     _journal->commit("remove poll info");
@@ -200,52 +188,126 @@ void PropagateUploadFileQNAM::start()
         return;
     }
 
-    const QString filePath = _propagator->getFilePath(_item._file);
+    _propagator->_activeJobs++;
+
+    if (!_deleteExisting) {
+        return slotComputeContentChecksum();
+    }
+
+    auto job = new DeleteJob(_propagator->account(),
+                             _propagator->_remoteFolder + _item->_file,
+                             this);
+    _jobs.append(job);
+    connect(job, SIGNAL(finishedSignal()), SLOT(slotComputeContentChecksum()));
+    connect(job, SIGNAL(destroyed(QObject*)), SLOT(slotJobDestroyed(QObject*)));
+    job->start();
+}
+
+void PropagateUploadFileQNAM::slotComputeContentChecksum()
+{
+    if (_propagator->_abortRequested.fetchAndAddRelaxed(0)) {
+        return;
+    }
+
+    _propagator->_activeJobs--; // from start
+
+    const QString filePath = _propagator->getFilePath(_item->_file);
 
     // remember the modtime before checksumming to be able to detect a file
     // change during the checksum calculation
-    _item._modtime = FileSystem::getModTime(filePath);
+    _item->_modtime = FileSystem::getModTime(filePath);
 
     _stopWatch.start();
 
-    // do whatever is needed to add a checksum to the http upload request.
-    // in any case, the validator will emit signal startUpload to let the flow
-    // continue in slotStartUpload here.
-    TransmissionChecksumValidator *validator = new TransmissionChecksumValidator(filePath, this);
-    connect(validator, SIGNAL(validated(QByteArray)), this, SLOT(slotStartUpload(QByteArray)));
-    validator->uploadValidation();
+    QByteArray contentChecksumType;
+    // We currently only do content checksums for the particular .eml case
+    // This should be done more generally in the future!
+    if (filePath.endsWith(QLatin1String(".eml"), Qt::CaseInsensitive)) {
+        contentChecksumType = "MD5";
+    }
+
+    // Maybe the discovery already computed the checksum?
+    if (_item->_contentChecksumType == contentChecksumType
+            && !_item->_contentChecksum.isEmpty()) {
+        slotComputeTransmissionChecksum(contentChecksumType, _item->_contentChecksum);
+        return;
+    }
+
+    // Compute the content checksum.
+    auto computeChecksum = new ComputeChecksum(this);
+    computeChecksum->setChecksumType(contentChecksumType);
+
+    connect(computeChecksum, SIGNAL(done(QByteArray,QByteArray)),
+            SLOT(slotComputeTransmissionChecksum(QByteArray,QByteArray)));
+    computeChecksum->start(filePath);
 }
 
-void PropagateUploadFileQNAM::slotStartUpload(const QByteArray& checksum)
+void PropagateUploadFileQNAM::setDeleteExisting(bool enabled)
 {
-    const QString fullFilePath(_propagator->getFilePath(_item._file));
+    _deleteExisting = enabled;
+}
 
-    _item._checksum = checksum;
+void PropagateUploadFileQNAM::slotComputeTransmissionChecksum(const QByteArray& contentChecksumType, const QByteArray& contentChecksum)
+{
+    _item->_contentChecksum = contentChecksum;
+    _item->_contentChecksumType = contentChecksumType;
+
+    _stopWatch.addLapTime(QLatin1String("ContentChecksum"));
+    _stopWatch.start();
+
+    // Reuse the content checksum as the transmission checksum if possible
+    const auto supportedTransmissionChecksums =
+            _propagator->account()->capabilities().supportedChecksumTypes();
+    if (supportedTransmissionChecksums.contains(contentChecksumType)) {
+        slotStartUpload(contentChecksumType, contentChecksum);
+        return;
+    }
+
+    // Compute the transmission checksum.
+    auto computeChecksum = new ComputeChecksum(this);
+    if (uploadChecksumEnabled()) {
+        computeChecksum->setChecksumType(_propagator->account()->capabilities().preferredChecksumType());
+    } else {
+        computeChecksum->setChecksumType(QByteArray());
+    }
+
+    connect(computeChecksum, SIGNAL(done(QByteArray,QByteArray)),
+            SLOT(slotStartUpload(QByteArray,QByteArray)));
+    const QString filePath = _propagator->getFilePath(_item->_file);
+    computeChecksum->start(filePath);
+}
+
+void PropagateUploadFileQNAM::slotStartUpload(const QByteArray& transmissionChecksumType, const QByteArray& transmissionChecksum)
+{
+    _transmissionChecksum = transmissionChecksum;
+    _transmissionChecksumType = transmissionChecksumType;
+
+    const QString fullFilePath = _propagator->getFilePath(_item->_file);
 
     if (!FileSystem::fileExists(fullFilePath)) {
         done(SyncFileItem::SoftError, tr("File Removed"));
         return;
     }
-    _stopWatch.addLapTime(QLatin1String("Checksum"));
+    _stopWatch.addLapTime(QLatin1String("TransmissionChecksum"));
 
-    time_t prevModtime = _item._modtime; // the _item value was set in PropagateUploadFileQNAM::start()
+    time_t prevModtime = _item->_modtime; // the _item value was set in PropagateUploadFileQNAM::start()
     // but a potential checksum calculation could have taken some time during which the file could
     // have been changed again, so better check again here.
 
-    _item._modtime = FileSystem::getModTime(fullFilePath);
-    if( prevModtime != _item._modtime ) {
+    _item->_modtime = FileSystem::getModTime(fullFilePath);
+    if( prevModtime != _item->_modtime ) {
         _propagator->_anotherSyncNeeded = true;
         done(SyncFileItem::SoftError, tr("Local file changed during syncing. It will be resumed."));
         return;
     }
 
     quint64 fileSize = FileSystem::getSize(fullFilePath);
-    _item._size = fileSize;
+    _item->_size = fileSize;
 
     // But skip the file if the mtime is too close to 'now'!
     // That usually indicates a file that is still being changed
     // or not yet fully copied to the destination.
-    if (fileIsStillChanging(_item)) {
+    if (fileIsStillChanging(*_item)) {
         _propagator->_anotherSyncNeeded = true;
         done(SyncFileItem::SoftError, tr("Local file changed during sync."));
         return;
@@ -253,20 +315,20 @@ void PropagateUploadFileQNAM::slotStartUpload(const QByteArray& checksum)
 
     _chunkCount = std::ceil(fileSize/double(chunkSize()));
     _startChunk = 0;
-    _transferId = qrand() ^ _item._modtime ^ (_item._size << 16);
+    _transferId = qrand() ^ _item->_modtime ^ (_item->_size << 16);
 
-    const SyncJournalDb::UploadInfo progressInfo = _propagator->_journal->getUploadInfo(_item._file);
+    const SyncJournalDb::UploadInfo progressInfo = _propagator->_journal->getUploadInfo(_item->_file);
 
-    if (progressInfo._valid && Utility::qDateTimeToTime_t(progressInfo._modtime) == _item._modtime ) {
+    if (progressInfo._valid && Utility::qDateTimeToTime_t(progressInfo._modtime) == _item->_modtime ) {
         _startChunk = progressInfo._chunk;
         _transferId = progressInfo._transferid;
-        qDebug() << Q_FUNC_INFO << _item._file << ": Resuming from chunk " << _startChunk;
+        qDebug() << Q_FUNC_INFO << _item->_file << ": Resuming from chunk " << _startChunk;
     }
 
     _currentChunk = 0;
     _duration.start();
 
-    emit progress(_item, 0);
+    emit progress(*_item, 0);
     this->startNextChunk();
 }
 
@@ -413,47 +475,51 @@ void PropagateUploadFileQNAM::startNextChunk()
     if (! _jobs.isEmpty() &&  _currentChunk + _startChunk >= _chunkCount - 1) {
         // Don't do parallel upload of chunk if this might be the last chunk because the server cannot handle that
         // https://github.com/owncloud/core/issues/11106
-        // We return now and when the _jobs will be finished we will proceed the last chunk
-        // NOTE: Some other part of the code such as slotUploadProgress assume also that the last chunk
+        // We return now and when the _jobs are finished we will proceed with the last chunk
+        // NOTE: Some other parts of the code such as slotUploadProgress also assume that the last chunk
         // is sent last.
         return;
     }
-    quint64 fileSize = _item._size;
+    quint64 fileSize = _item->_size;
     QMap<QByteArray, QByteArray> headers;
     headers["OC-Total-Length"] = QByteArray::number(fileSize);
     headers["OC-Async"] = "1";
     headers["OC-Chunk-Size"]= QByteArray::number(quint64(chunkSize()));
     headers["Content-Type"] = "application/octet-stream";
-    headers["X-OC-Mtime"] = QByteArray::number(qint64(_item._modtime));
+    headers["X-OC-Mtime"] = QByteArray::number(qint64(_item->_modtime));
 
-    if(_item._file.contains(".sys.admin#recall#")) {
+    if(_item->_file.contains(".sys.admin#recall#")) {
         // This is a file recall triggered by the admin.  Note: the
         // recall list file created by the admin and downloaded by the
         // client (.sys.admin#recall#) also falls into this category
         // (albeit users are not supposed to mess up with it)
 
         // We use a special tag header so that the server may decide to store this file away in some admin stage area
-        // And not directly in the user's area (what would trigger redownloads etc).
+        // And not directly in the user's area (which would trigger redownloads etc).
         headers["OC-Tag"] = ".sys.admin#recall#";
     }
 
-    if (!_item._etag.isEmpty() && _item._etag != "empty_etag" &&
-            _item._instruction != CSYNC_INSTRUCTION_NEW  // On new files never send a If-Match
+    if (!_item->_etag.isEmpty() && _item->_etag != "empty_etag"
+            && _item->_instruction != CSYNC_INSTRUCTION_NEW  // On new files never send a If-Match
+            && _item->_instruction != CSYNC_INSTRUCTION_TYPE_CHANGE
+            && !_deleteExisting
             ) {
-        // We add quotes because the owncloud server always add quotes around the etag, and
-        //  csync_owncloud.c's owncloud_file_id always strip the quotes.
-        headers["If-Match"] = '"' + _item._etag + '"';
+        // We add quotes because the owncloud server always adds quotes around the etag, and
+        //  csync_owncloud.c's owncloud_file_id always strips the quotes.
+        headers["If-Match"] = '"' + _item->_etag + '"';
     }
 
-    QString path = _item._file;
+    QString path = _item->_file;
 
     UploadDevice *device = new UploadDevice(&_propagator->_bandwidthManager);
     qint64 chunkStart = 0;
     qint64 currentChunkSize = fileSize;
+    bool isFinalChunk = false;
     if (_chunkCount > 1) {
         int sendingChunk = (_currentChunk + _startChunk) % _chunkCount;
-        // XOR with chunk size to make sure everything goes well if chunk size change between runs
+        // XOR with chunk size to make sure everything goes well if chunk size changes between runs
         uint transid = _transferId ^ chunkSize();
+        qDebug() << "Upload chunk" << sendingChunk << "of" << _chunkCount << "transferid(remote)=" << transid;
         path +=  QString("-chunking-%1-%2-%3").arg(transid).arg(_chunkCount).arg(sendingChunk);
 
         headers["OC-Chunked"] = "1";
@@ -462,21 +528,22 @@ void PropagateUploadFileQNAM::startNextChunk()
         currentChunkSize = chunkSize();
         if (sendingChunk == _chunkCount - 1) { // last chunk
             currentChunkSize = (fileSize % chunkSize());
-            if( currentChunkSize == 0 ) { // if the last chunk pretents to be 0, its actually the full chunk size.
+            if( currentChunkSize == 0 ) { // if the last chunk pretends to be 0, its actually the full chunk size.
                 currentChunkSize = chunkSize();
             }
-            if( !_item._checksum.isEmpty() ) {
-                headers[checkSumHeaderC] = _item._checksum;
-            }
+            isFinalChunk = true;
         }
     } else {
-        // checksum if its only one chunk
-        if( !_item._checksum.isEmpty() ) {
-            headers[checkSumHeaderC] = _item._checksum;
-        }
+        // if there's only one chunk, it's the final one
+        isFinalChunk = true;
     }
 
-    if (! device->prepareAndOpen(_propagator->getFilePath(_item._file), chunkStart, currentChunkSize)) {
+    if (isFinalChunk && !_transmissionChecksumType.isEmpty()) {
+        headers[checkSumHeaderC] = makeChecksumHeader(
+                _transmissionChecksumType, _transmissionChecksum);
+    }
+
+    if (! device->prepareAndOpen(_propagator->getFilePath(_item->_file), chunkStart, currentChunkSize)) {
         qDebug() << "ERR: Could not prepare upload device: " << device->errorString();
         // Soft error because this is likely caused by the user modifying his files while syncing
         abortWithError( SyncFileItem::SoftError, device->errorString() );
@@ -500,11 +567,7 @@ void PropagateUploadFileQNAM::startNextChunk()
     if (!env.isEmpty()) {
         parallelChunkUpload = env != "false" && env != "0";
     } else {
-        auto version = _propagator->account()->serverVersion();
-        auto components = version.split('.');
-        int versionNum = (components.value(0).toInt() << 16)
-                       + (components.value(1).toInt() << 8)
-                       + components.value(2).toInt();
+        int versionNum = _propagator->account()->serverVersionInt();
         if (versionNum < 0x080003) {
             // Disable parallel chunk upload severs older than 8.0.3 to avoid too many
             // internal sever errors (#2743, #2938)
@@ -542,7 +605,7 @@ void PropagateUploadFileQNAM::slotPutFinished()
     _propagator->_activeJobs--;
 
     if (_finished) {
-        // We have send the finished signal already. We don't need to handle any remaining jobs
+        // We have sent the finished signal already. We don't need to handle any remaining jobs
         return;
     }
 
@@ -554,14 +617,14 @@ void PropagateUploadFileQNAM::slotPutFinished()
         // This works around a bug in QNAM wich might reuse a non-empty buffer for the next request.
         qDebug() << "Forcing job abort on HTTP connection reset with Qt < 5.4.2.";
         _propagator->_anotherSyncNeeded = true;
-        done(SyncFileItem::SoftError, tr("Forcing job abort on HTTP connection reset with Qt < 5.4.2."));
+        abortWithError(SyncFileItem::SoftError, tr("Forcing job abort on HTTP connection reset with Qt < 5.4.2."));
         return;
     }
 #endif
 
     if (err != QNetworkReply::NoError) {
-        _item._httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-        if(checkForProblemsWithShared(_item._httpErrorCode,
+        _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+        if(checkForProblemsWithShared(_item->_httpErrorCode,
             tr("The file was edited locally but is part of a read only share. "
                "It is restored and your edit is in the conflict file."))) {
             return;
@@ -574,20 +637,22 @@ void PropagateUploadFileQNAM::slotPutFinished()
             errorString = job->reply()->rawHeader("OC-ErrorString");
         }
 
-        if (_item._httpErrorCode == 412) {
+        if (_item->_httpErrorCode == 412) {
             // Precondition Failed:   Maybe the bad etag is in the database, we need to clear the
             // parent folder etag so we won't read from DB next sync.
-            _propagator->_journal->avoidReadFromDbOnNextSync(_item._file);
+            _propagator->_journal->avoidReadFromDbOnNextSync(_item->_file);
             _propagator->_anotherSyncNeeded = true;
         }
 
-        abortWithError(classifyError(err, _item._httpErrorCode), errorString);
+        SyncFileItem::Status status = classifyError(err, _item->_httpErrorCode,
+                                                    &_propagator->_anotherSyncNeeded);
+        abortWithError(status, errorString);
         return;
     }
 
-    _item._httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
-    // The server needs some time to process the request and provide with a poll URL
-    if (_item._httpErrorCode == 202) {
+    _item->_httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+    // The server needs some time to process the request and provide us with a poll URL
+    if (_item->_httpErrorCode == 202) {
         _finished = true;
         QString path =  QString::fromUtf8(job->reply()->rawHeader("OC-Finish-Poll"));
         if (path.isEmpty()) {
@@ -611,7 +676,7 @@ void PropagateUploadFileQNAM::slotPutFinished()
     bool finished = etag.length() > 0;
 
     // Check if the file still exists
-    const QString fullFilePath(_propagator->getFilePath(_item._file));
+    const QString fullFilePath(_propagator->getFilePath(_item->_file));
     if( !FileSystem::fileExists(fullFilePath) ) {
         if (!finished) {
             abortWithError(SyncFileItem::SoftError, tr("The local file was removed during sync."));
@@ -622,7 +687,7 @@ void PropagateUploadFileQNAM::slotPutFinished()
     }
 
     // Check whether the file changed since discovery.
-    if (! FileSystem::verifyFileUnchanged(fullFilePath, _item._size, _item._modtime)) {
+    if (! FileSystem::verifyFileUnchanged(fullFilePath, _item->_size, _item->_modtime)) {
         _propagator->_anotherSyncNeeded = true;
         if( !finished ) {
             abortWithError(SyncFileItem::SoftError, tr("Local file changed during sync."));
@@ -640,21 +705,29 @@ void PropagateUploadFileQNAM::slotPutFinished()
                 return;
             }
             _finished = true;
-            done(SyncFileItem::NormalError, tr("The server did not acknowledge the last chunk. (No e-tag were present)"));
+            done(SyncFileItem::NormalError, tr("The server did not acknowledge the last chunk. (No e-tag was present)"));
             return;
         }
 
+        // Deletes an existing blacklist entry on successful chunk upload
+        if (_item->_hasBlacklistEntry) {
+            _propagator->_journal->wipeErrorBlacklistEntry(_item->_file);
+            _item->_hasBlacklistEntry = false;
+        }
+
         SyncJournalDb::UploadInfo pi;
         pi._valid = true;
         auto currentChunk = job->_chunk;
         foreach (auto *job, _jobs) {
             // Take the minimum finished one
-            currentChunk = qMin(currentChunk, job->_chunk - 1);
+            if (auto putJob = qobject_cast<PUTFileJob*>(job)) {
+                currentChunk = qMin(currentChunk, putJob->_chunk - 1);
+            }
         }
         pi._chunk = (currentChunk + _startChunk + 1) % _chunkCount ; // next chunk to start with
         pi._transferid = _transferId;
-        pi._modtime =  Utility::qDateTimeFromTime_t(_item._modtime);
-        _propagator->_journal->setUploadInfo(_item._file, pi);
+        pi._modtime =  Utility::qDateTimeFromTime_t(_item->_modtime);
+        _propagator->_journal->setUploadInfo(_item->_file, pi);
         _propagator->_journal->commit("Upload info");
         startNextChunk();
         return;
@@ -665,49 +738,46 @@ void PropagateUploadFileQNAM::slotPutFinished()
     // the file id should only be empty for new files up- or downloaded
     QByteArray fid = job->reply()->rawHeader("OC-FileID");
     if( !fid.isEmpty() ) {
-        if( !_item._fileId.isEmpty() && _item._fileId != fid ) {
-            qDebug() << "WARN: File ID changed!" << _item._fileId << fid;
+        if( !_item->_fileId.isEmpty() && _item->_fileId != fid ) {
+            qDebug() << "WARN: File ID changed!" << _item->_fileId << fid;
         }
-        _item._fileId = fid;
+        _item->_fileId = fid;
     }
 
-    _item._etag = etag;
+    _item->_etag = etag;
 
-    _item._responseTimeStamp = job->responseTimestamp();
+    _item->_responseTimeStamp = job->responseTimestamp();
 
     if (job->reply()->rawHeader("X-OC-MTime") != "accepted") {
         // X-OC-MTime is supported since owncloud 5.0.   But not when chunking.
-        // Normaly Owncloud 6 always put X-OC-MTime
-        qWarning() << "Server do not support X-OC-MTime" << job->reply()->rawHeader("X-OC-MTime");
-#ifdef USE_NEON
-        PropagatorJob *newJob = new UpdateMTimeAndETagJob(_propagator, _item);
-        QObject::connect(newJob, SIGNAL(completed(SyncFileItem)), this, SLOT(finalize(SyncFileItem)));
-        QMetaObject::invokeMethod(newJob, "start");
-        return;
-#else
+        // Normally Owncloud 6 always puts X-OC-MTime
+        qWarning() << "Server does not support X-OC-MTime" << job->reply()->rawHeader("X-OC-MTime");
         // Well, the mtime was not set
-#endif
+        done(SyncFileItem::SoftError, "Server does not support X-OC-MTime");
     }
 
     // performance logging
-    _item._requestDuration = _stopWatch.stop();
-    qDebug() << "*==* duration UPLOAD" << _item._size << _stopWatch.durationOfLap(QLatin1String("Checksum")) << _item._requestDuration;
+    _item->_requestDuration = _stopWatch.stop();
+    qDebug() << "*==* duration UPLOAD" << _item->_size
+             << _stopWatch.durationOfLap(QLatin1String("ContentChecksum"))
+             << _stopWatch.durationOfLap(QLatin1String("TransmissionChecksum"))
+             << _item->_requestDuration;
 
-    finalize(_item);
+    finalize(*_item);
 }
 
 void PropagateUploadFileQNAM::finalize(const SyncFileItem &copy)
 {
     // Normally, copy == _item,   but when it comes from the UpdateMTimeAndETagJob, we need to do
     // some updates
-    _item._etag = copy._etag;
-    _item._fileId = copy._fileId;
+    _item->_etag = copy._etag;
+    _item->_fileId = copy._fileId;
 
-    _item._requestDuration = _duration.elapsed();
+    _item->_requestDuration = _duration.elapsed();
 
-    _propagator->_journal->setFileRecord(SyncJournalFileRecord(_item, _propagator->getFilePath(_item._file)));
+    _propagator->_journal->setFileRecord(SyncJournalFileRecord(*_item, _propagator->getFilePath(_item->_file)));
     // Remove from the progress database:
-    _propagator->_journal->setUploadInfo(_item._file, SyncJournalDb::UploadInfo());
+    _propagator->_journal->setUploadInfo(_item->_file, SyncJournalDb::UploadInfo());
     _propagator->_journal->commit("upload file start");
 
     _finished = true;
@@ -716,16 +786,21 @@ void PropagateUploadFileQNAM::finalize(const SyncFileItem &copy)
 
 void PropagateUploadFileQNAM::slotUploadProgress(qint64 sent, qint64 total)
 {
+    // Completion is signaled with sent=0, total=0; avoid accidentally
+    // resetting progress due to the sent being zero by ignoring it.
+    // finishedSignal() is bound to be emitted soon anyway.
+    // See https://bugreports.qt.io/browse/QTBUG-44782.
     if (sent == 0 && total == 0) {
-        return; // QNAM bug https://bugreports.qt.io/browse/QTBUG-44782
+        return;
     }
+
     int progressChunk = _currentChunk + _startChunk - 1;
     if (progressChunk >= _chunkCount)
         progressChunk = _currentChunk - 1;
 
     // amount is the number of bytes already sent by all the other chunks that were sent
     // not including this one.
-    // FIXME: this assume all chunks have the same size, which is true only if the last chunk
+    // FIXME: this assumes all chunks have the same size, which is true only if the last chunk
     // has not been finished (which should not happen because the last chunk is sent sequentially)
     quint64 amount = progressChunk * chunkSize();
 
@@ -739,7 +814,7 @@ void PropagateUploadFileQNAM::slotUploadProgress(qint64 sent, qint64 total)
         // sender() is the only current job, no need to look at the byteWritten properties
         amount += sent;
     }
-    emit progress(_item, amount);
+    emit progress(*_item, amount);
 }
 
 void PropagateUploadFileQNAM::startPollJob(const QString& path)
@@ -748,9 +823,9 @@ void PropagateUploadFileQNAM::startPollJob(const QString& path)
                                _propagator->_journal, _propagator->_localDir, this);
     connect(job, SIGNAL(finishedSignal()), SLOT(slotPollFinished()));
     SyncJournalDb::PollInfo info;
-    info._file = _item._file;
+    info._file = _item->_file;
     info._url = path;
-    info._modtime = _item._modtime;
+    info._modtime = _item->_modtime;
     _propagator->_journal->setPollInfo(info);
     _propagator->_journal->commit("add poll info");
     _propagator->_activeJobs++;
@@ -764,13 +839,13 @@ void PropagateUploadFileQNAM::slotPollFinished()
 
     _propagator->_activeJobs--;
 
-    if (job->_item._status != SyncFileItem::Success) {
+    if (job->_item->_status != SyncFileItem::Success) {
         _finished = true;
-        done(job->_item._status, job->_item._errorString);
+        done(job->_item->_status, job->_item->_errorString);
         return;
     }
 
-    finalize(job->_item);
+    finalize(*job->_item);
 }
 
 void PropagateUploadFileQNAM::slotJobDestroyed(QObject* job)
@@ -782,7 +857,7 @@ void PropagateUploadFileQNAM::abort()
 {
     foreach(auto *job, _jobs) {
         if (job->reply()) {
-            qDebug() << Q_FUNC_INFO << job << this->_item._file;
+            qDebug() << Q_FUNC_INFO << job << this->_item->_file;
             job->reply()->abort();
         }
     }
