@@ -20,6 +20,10 @@
 
 namespace OCC {
 
+/**
+ * @brief The SyncFileStatus class
+ * @ingroup libsync
+ */
 class OWNCLOUDSYNC_EXPORT SyncFileStatus
 {
 public:
@@ -32,7 +36,7 @@ public:
         STATUS_NEW,
         STATUS_CONFLICT,
         STATUS_IGNORE,
-        STATUS_SYNC,
+        STATUS_UPTODATE,
         STATUS_STAT_ERROR,
         STATUS_ERROR,
         STATUS_UPDATED
