@@ -47,6 +47,8 @@ Theme* Theme::instance() {
     return _instance;
 }
 
+Theme::~Theme() {}
+
 QString Theme::statusHeaderText( SyncResult::Status status ) const
 {
     QString resultStr;
@@ -122,14 +124,14 @@ QIcon Theme::themeIcon( const QString& name, bool sysTray ) const
         flavor = QLatin1String("colored");
     }
 
-    if( QIcon::hasThemeIcon( name )) {
-        // use from theme
-        return QIcon::fromTheme( name );
-    }
-
     QString key = name + "," + flavor;
     QIcon & cached = _iconCache[key];
     if (cached.isNull()) {
+        if(QIcon::hasThemeIcon(name)) {
+            // use from theme
+            return cached = QIcon::fromTheme(name);
+        }
+
         QList<int> sizes;
         sizes <<16 << 22 << 32 << 48 << 64 << 128 << 256;
         foreach (int size, sizes) {
@@ -193,12 +195,17 @@ Theme::Theme() :
 
 }
 
-// if this option return true, the client only supports one folder to sync.
-// The Add-Button is removed accoringly.
+// If this option returns true, the client only supports one folder to sync.
+// The Add-Button is removed accordingly.
 bool Theme::singleSyncFolder() const {
     return false;
 }
 
+bool Theme::multiAccount() const
+{
+    return true;
+}
+
 QString Theme::defaultServerFolder() const
 {
     return QLatin1String("/");
@@ -246,6 +253,13 @@ QString Theme::transmissionChecksum() const
     return QString::null; // No transmission by default.
 }
 
+qint64 Theme::newBigFolderSizeLimit() const
+{
+    // Default to 500MB
+    return 500;
+}
+
+
 QString Theme::gitSHA1() const
 {
     QString devString;
@@ -393,6 +407,15 @@ bool Theme::wizardSelectiveSyncDefaultNothing() const
     return false;
 }
 
+QString Theme::webDavPath() const
+{
+    return QLatin1String("remote.php/webdav/");
+}
+
+QString Theme::webDavPathNonShib() const
+{
+    return QLatin1String("remote.php/nonshib-webdav/");
+}
 
 } // end namespace client
 
